      subroutine global_cc(mode, bin, nrot, nannuli, nview_mod,
     $     idim1, idim2, res_min, res_max, pixres_min, pixres_max,
     $     ann_lo, ann_hi, jcut, radius, sym, verbose, mag_cen,
     $     mag_step, mag_num, mod_ts, mod_ps, prjs,
     $     varopt, ncol, nrow, nptles, nfiles, img_fnames, param_fnames,
     $     ptle_count, indx, theta, phi, omega, x, y, mags,
     $     pft_ccs, prj_ccs, cmp_ccs, compare_mode, filt_fac, sigcut,
     $     xcen_model, ycen_model, pftrad_lo, pftrad_hi, quick_omega)

      implicit none

#ifdef MPI
      include 'mpif.h'
      integer   stat(mpi_status_size)
      integer :: mcw = mpi_comm_world
#endif

      include '../include/allprogs.inc'
      include '../include/pifhead.inc'
      type(pifhead_t) maphead, imghead, prjhead, pfthead
      common /pifhead/ maphead, imghead, prjhead, pfthead

      common/ctf_com/pixsiz, units, volts, amp_fac, focus_maj, 
     $     focus_min, ang_maj, cs_coeff

      real pixsiz(999), volts(999), amp_fac(999), focus_maj(999), 
     $     focus_min(999), ang_maj(999), cs_coeff(999)
      integer units(999)
      
      integer, intent(in) :: nrow, ncol, nview_mod, nfiles, nptles, jcut
      integer, intent(in) :: mode, bin, nrot, nannuli, idim1, idim2
      integer, intent(in) :: quick_omega, ann_lo, ann_hi, mag_num
      integer, intent(in) :: ptle_count(nfiles), indx(nptles)
      integer, intent(in) :: compare_mode, sym, verbose
      real, intent(in)    :: mod_ts(nview_mod), mod_ps(nview_mod)
      real, intent(in)    :: mag_step, mag_cen, xcen_model, ycen_model
      real, intent(in)    :: filt_fac(10), sigcut, pftrad_lo, pftrad_hi
      real, intent(in)    :: pixres_min, pixres_max, radius
      real, intent(in)    :: res_min, res_max
      character(len=256), intent(in) :: img_fnames(999)
      character(len=256), intent(in) :: param_fnames(999)
      logical, intent(in) :: varopt
      
      real, intent(inout) :: prjs(ncol,nrow,nview_mod)
      real, intent(inout) :: x(nptles), y(nptles), mags(nptles)
      real, intent(out)   :: theta(nptles), phi(nptles), omega(nptles)
      real, intent(out)   :: pft_ccs(nptles), prj_ccs(nptles)
      real, intent(out)   :: cmp_ccs(nptles)

      character(len=256) new_name

      integer i, j, k, id, file, ctfmode, filter, flag
      integer local_max, local_count, buffer_size(0:255), alloc_stat
      integer ipx(nptles), ipy(nptles)

      real peakx1(nptles), peaky1(nptles), step1(nptles)
      real peakx2(nptles), peaky2(nptles), step2(nptles)

      real img_bkg, tfac, cc_avg
      real del_ang, del_pixres, fft_origx, fft_origy, xcen, ycen
      real best_mag, mag, mag_lo, weight, res1, res2
      real cc_res_global(2,20), cc_rad_global(2,400)
      real cc_res_globall(2,20), cc_rad_globall(2,400)
      data cc_res_global/40*0.0/, cc_rad_global/800*0.0/
      data cc_res_globall/40*0.0/, cc_rad_globall/800*0.0/

      real, allocatable :: real_results_local(:,:), r4_img(:,:)
      real, allocatable :: correction(:,:), abs_ctf(:,:), prj_rot(:,:)
      real, allocatable :: pftprj(:,:,:), pmap(:,:), fltpmap(:,:)
      real, allocatable :: pprj(:,:), prjs_gold(:,:,:), tmp0(:,:)
      real, allocatable :: pftimg(:,:), prj(:,:), prj_avg(:,:)
      real, allocatable :: img(:,:)
      integer, allocatable :: integer_results_local(:,:)
      logical, allocatable :: var(:,:)
      complex, allocatable :: tmp1(:,:), tmp2(:,:)
      complex, allocatable :: pfft(:,:), fft_prj_avg(:,:)
      complex, allocatable :: fft_r4_img(:,:), fft_prj_rot(:,:)

      allocate(pprj(nrot,nannuli), 
     $     prj(ncol,nrow),
     $     prj_rot(ncol,nrow),
     $     prj_avg(ncol,nrow),
     $     pftprj(nrot/4,nannuli,nview_mod),
     $     pftimg(nrot/4,nannuli), 
     $     pfft(nrot,nannuli),
     $     pmap(nrot,nannuli), 
     $     fltpmap(nrot,nannuli), 
     $     var(nrot/4,nannuli), 
     $     fft_r4_img(idim1,idim2),
     $     fft_prj_avg(idim1,idim2),
     $     fft_prj_rot(idim1,idim2),
     $     img(ncol,nrow), 
     $     r4_img(ncol,nrow), 
     $     tmp0(ncol,nrow), 
     $     tmp1(idim1,idim2),
     $     tmp2(idim1,idim2),
     $     correction(idim1,idim2),
     $     abs_ctf(idim1,idim2),
     $     stat = alloc_stat)

      if (alloc_stat > 0) then
         write(*,*) 'could not allocate memory in global_cc.'
         call tsbend
      end if

      fft_origx  = ncol/2
      fft_origy  = nrow/2
      del_pixres = 0.025
      del_ang    = 360.0/real(nrot)

      if(compare_mode == 1) then
         allocate(prjs_gold(ncol,nrow,nview_mod),stat=alloc_stat)
         if (alloc_stat > 0) then
            write(*,*) 'could not allocate prjs_gold in global_cc.'
            call tsbend
         end if
         prjs_gold  = prjs      ! Fortran 90 array syntax
      endif

C     Calculate buffer sizes
      buffer_size = 0
      do i=0,num_nodes-1
         buffer_size(i) = nptles/num_nodes
         if(i < mod(nptles,num_nodes)) then
            buffer_size(i) = buffer_size(i) + 1
         endif
      enddo

C     Calculate the maximum number of particles assigned to a process
      local_max = maxval(buffer_size)

C     Allocate global and local arrays for particle parameters
      allocate(real_results_local(15,local_max),
     $     integer_results_local(2,local_max))
      
      id = 0
      local_count = 0
      do file=1,nfiles
         
c     compare_mode == 1: Apply CTF correction to projections
         if(compare_mode == 1) then
            prjs = prjs_gold  ! Fortran 90 array syntax
            do id = 1,nview_mod
               call map_fft_fill_ppft(prjs(1,1,id), ncol, nrow,
     $              tmp1, idim1, idim2, 0.0)
               call fft_2d(tmp1, idim1, idim2)
               ctfmode = 1
               filter = 0
               weight = 1.0
               tfac = 0.0
               res1 = 1.0
               res2 = 1.0
               call ctf_para(correction, abs_ctf, 
     $              1, idim1, 1, idim2, .false.,
     $              ctfmode, filter, tfac, filt_fac(1), filt_fac(2),
     $              bin*pixsiz(file), units(file), volts(file),
     $              focus_maj(file), focus_min(file), ang_maj(file),
     $              amp_fac(file), cs_coeff(file), weight, res1,
     $              res2)
               tmp1 = tmp1 * correction
               call fft_2d_back(tmp1, idim1, idim2)
               call fft_map_fill_ppft(tmp1, idim1, idim2, prjs(1,1,id),
     $              ncol, nrow,0.0)
            enddo
         endif

C     compare_mode == 2 : Conway method of weighting data. Will be
C     applied later to images
         if(compare_mode == 2) then
            tfac    = 0.0
            ctfmode = 1
            filter  = 0
            weight  = 1.0
            res1    = 1.0
            res2    = 1.0
            call ctf_para(correction, abs_ctf, 
     $           1, idim1, 1, idim2, .false.,
     $           ctfmode, filter, tfac, filt_fac(1), filt_fac(2),
     $           bin*pixsiz(file), units(file), volts(file),
     $           focus_maj(file), focus_min(file), ang_maj(file),
     $           amp_fac(file), cs_coeff(file), weight, res1, res2)
         endif
               
C     compare_mode == 3 : Calculate inverse CTF correction. Will be
C     applied later to images
         if(compare_mode == 3) then
            tfac    = 0.0
            ctfmode = 1
            filter  = 2
            weight  = 1.0
            res1    = 1.0
            res2    = 1.0
            call ctf_para(correction, abs_ctf, 
     $           1, idim1, 1, idim2, .false.,
     $           ctfmode, filter, tfac, filt_fac(1), filt_fac(2),
     $           bin*pixsiz(file), units(file), volts(file),
     $           focus_maj(file), focus_min(file), ang_maj(file),
     $           amp_fac(file), cs_coeff(file), weight, res1, res2)
         endif
         
C     For the first micrograph, need to calculate PFTs of projections,
C     VAR array, and (optionally) average of projections. For
C     compare_mode==3, need to do this for all subsequent micrographs
C     since CTF correction is applied to projections.

         if(file == 1 .or. compare_mode == 1) then
            do k=1,nview_mod
               call map_polar(prjs(1,1,k), ncol, nrow, pmap, nrot,
     $              nannuli,xcen_model, ycen_model, pftrad_lo,
     $              pftrad_hi, del_ang)
               call pmap_fft (pmap, pfft, nrot, nannuli)
               do j = 1,nannuli
                  do i = 1,(nrot/4)
                     pftprj(i,j,k) = abs(pfft(i,j))
                  enddo
               enddo
            enddo
            
            call calc_var(pftprj, nrot, nannuli, nview_mod, jcut,
     $           pixres_min,pixres_max, ann_lo, ann_hi, varopt,
     $           sigcut, var)
      
            if(abs(mode) == 1) then
               prj_avg = 0.0
               do k=1,nview_mod
                  prj_avg = prj_avg + prjs(:,:,k)
               enddo
               prj_avg = prj_avg/real(nview_mod)
               call map_sym_cavg_ppft(prj_avg, ncol, nrow, xcen_model,
     $              ycen_model)
               call prjavg_fft(prj_avg, ncol, nrow, nview_mod,
     $              fft_prj_avg, idim1, idim2, mode, prjhead)
            endif
         endif

         do j=1,ptle_count(file)
            id = id + 1
            if(iam == mod(id-1,num_nodes)) then
               local_count = local_count + 1

               CC_AVG = 0.0
               
               IMGHEAD%LUNUM = 7
               CALL PIF_OPEN(IMG_FNAMES(FILE), IMGHEAD%LUNUM, 'r')
               CALL PIF_READ_GH(IMGHEAD)
               IMGHEAD%IMGNUM = INDX(ID) ! SPECIFY WHICH "IMG" YOU WANT
               CALL PIF_READ_DH(IMGHEAD)
               IMG_BKG = IMGHEAD%IMG_BKG
               if(imgHead%mode == 46) then
                  CALL PIF_READ_IMGI4(IMGHEAD, IMG, NCOL, NROW, INDX(ID))
               else
                  print *, 'Invalid PIF mode - ', imgHead%mode
                  call tsbend
               endif
               IMGHEAD%LUNUM = 7 ! UNIT FOR INPUT IMAGE FILE
               CALL PIF_CLOSE (IMGHEAD%LUNUM)
               
C     Reads for now invalid modes
C     CALL PIF_READ_IMGI1 (IMGHEAD, IMG, NCOL, NROW, INDX(ID)) !mode==6
C     CALL PIF_READ_IMGI2 (IMGHEAD, IMG, NCOL, NROW, INDX(ID)) !mode==7
               
C     Calculate transform of image
               CALL MAP_FFT_FILL_PPFT(IMG, NCOL, NROW, FFT_R4_IMG,
     $              IDIM1, IDIM2, IMG_BKG)
               CALL FFT_2D (FFT_R4_IMG, IDIM1, IDIM2)
               
C     Divide image by inverse CTF for compare_mode == 3
               if(compare_mode == 3) then
                  fft_r4_img = fft_r4_img * correction
               endif
               
C     Low-pass filter transform of image and then back transform
               CALL FFT_LOP (FFT_R4_IMG, IDIM1, IDIM2, RES_MAX, 
     $              2.0, .TRUE.)
               tmp1 = fft_r4_img ! Fortran 90 array syntax
               CALL FFT_2D_BACK(tmp1, IDIM1, IDIM2)
               
C     Store modified version of image into R4_IMG and IMG
               CALL FFT_MAP_FILL_PPFT(tmp1, IDIM1, IDIM2,
     $              R4_IMG, NCOL, NROW, 0.0)
               CALL FFT_MAP_FILL_PPFT(tmp1, IDIM1, IDIM2,
     $              IMG, NCOL, NROW, IMG_BKG)
                              
C     For compare_mode == 2, apply CTF to FFT to downweight data
C     according to Steve Fuller scheme
               IF(COMPARE_MODE == 2) THEN
                  
                  fft_r4_img = fft_r4_img * correction
                  
                  tmp1 = fft_r4_img ! Fortran 90 array syntax
                  CALL FFT_2D_BACK(tmp1, IDIM1, IDIM2)
                  CALL FFT_MAP_FILL_PPFT(tmp1, IDIM1, IDIM2,
     $                 R4_IMG, NCOL, NROW, 0.0)
                  WRITE(*,'(A,I4,A,2F8.4,A,F8.4)')
     $                 'Applied CTF to image ',ID,
     $                 ' with defocus values', FOCUS_MAJ(FILE)
     $                 ,FOCUS_MIN(FILE), ' and ANG_MAJ =', ANG_MAJ(FILE)
               ENDIF
               
C     ********************************************************************
C     *   I. Get initial MAG scale factor (1.0 is default)               *
C     *      A) GLOBAL search mode, initial sc. factor = MAG_CEN         *
C     *      B) GLOBAL mode, read from input parameter file              *
C     *  II. Get initial X,Y origins                                     *
C     *      A) ABS(MODE).EQ.1, estimate by cross-correlation with cir-  *
C     *         cularly averaged reference, RES_MIN=1.0 (no choice)      *
C     *      B) ABS(MODE).EQ.2, read from input parameter file.          *
C     * III. If MAG does not equal 1.0, scale image by MAG               *
C     *  IV. Compute FFT of image                                        *
C     ********************************************************************
               
               IF(ABS(MODE) == 1) THEN ! Global mode #1 (no origins)
                  CALL GET_XY(FFT_R4_IMG, FFT_PRJ_AVG, XCEN, YCEN,
     $                 IDIM1, IDIM2, RES_MAX, FFT_ORIGX, FFT_ORIGY,
     $                 flag, ipx(id), ipy(id), peakx1(id), peakx2(id),
     $                 peaky1(id), peaky2(id), step1(id), step2(id))
                  
                  if(flag .lt. 0) then
                     print *, 'WARNING:  Center is more than 30% off'
                     print *, '          Particle #', id
                  endif
                  MAG = MAG_CEN
               ELSE             ! Global mode #2 (with origins)
                  XCEN = ((X(ID) + 1.0)/BIN) - 1.0 ! NEED IN BINNED FFT SPACE
                  YCEN = ((Y(ID) + 1.0)/BIN) - 1.0 ! NEED IN BINNED FFT SPACE
                  MAG  = MAGS(ID)
               ENDIF
               
c     Mag .ne. 0 code can be found in versions 1.16 and earlier of this file
               if(mag .ne. 1.0) then
                  print *, 'MAG not equal to 1.0 ignored'
               endif
               
               xcen = xcen + 1.0 ! Get back into real space
               ycen = ycen + 1.0
               
C     *******************************************************************
C     *  Sometimes EMPFT crashes with a division-by-0 error in          *
C     *  GET_THEPHI (called below). It turns out that GET_XY            *
C     *  (above) returns a bad origin due to poor matching between an   *
C     *  image and its reference, and then MAP_POLAR (below) fills the  *
C     *  polar array with zero since the origin can be too far off the  *
C     *  edge of the cartesian array. The div-by-0 occurs because the   *
C     *  sum-of-sqaures for the polar array is zero.  Check here        * 
C     *  for (XCEN,YCEN) being away from the center of the image array  *
C     *  (NCOL/2,NROW/2) and post warnings. If it is way off, the user  *
C     *  can try to skip this particle and go on to the next, rather    *
C     *  than crash in GET_THEPHI.                                      *
C     *******************************************************************
               
               IF (ABS(XCEN-FLOAT(NCOL/2)) > FLOAT(NCOL/2) .OR. 
     1              ABS(YCEN-FLOAT(NROW/2)) > FLOAT(NROW/2)) THEN
C     WRITE (*,410) INDX(ID), XCEN*BIN, YCEN*BIN
               ELSE IF (ABS(XCEN-FLOAT(NCOL/2)) > FLOAT(NCOL/10) .OR.
     1                 ABS(YCEN-FLOAT(NROW/2)) > FLOAT(NROW/10)) THEN
                  WRITE (*,415) INDX(ID), XCEN*BIN, YCEN*BIN
               ENDIF
               
 410           format (' WARNING - estimated center is off ',
     $              'edge of array:',/,
     $              ' img_num, xcen, ycen = ', i6, 2f10.3)
 415           format (' WARNING - estimated center is more ',
     $              'than 10% from array center:',/,
     $              ' img_num, xcen, ycen = ', i6, 2f10.3)

               CALL GET_TPO_G (NCOL, NROW, NROT, NANNULI,
     $              NVIEW_MOD,JCUT, XCEN, YCEN, XCEN_MODEL,
     $              YCEN_MODEL, RADIUS,DEL_ANG, PIXRES_MIN, PIXRES_MAX,
     $              ANN_LO, ANN_HI, VAR,
     $              MOD_TS, MOD_PS, theta(id), phi(id), omega(id),
     $              PFTPRJ, PRJS, pft_ccs(id), prj_ccs(id), 
     $              R4_IMG, PMAP, PFFT, PRJ, PPRJ, PFTIMG, QUICK_OMEGA)

C     Refine X,Y origin
               PRJ_ROT = PRJ    !Fortran 90 array syntax
               IF (omega(id) .NE.0.0) THEN
                  CALL INTERP2D (PRJ_ROT, NCOL, NROW, XCEN_MODEL-1.0,
     $                 YCEN_MODEL-1.0, XCEN_MODEL-1.0, YCEN_MODEL-1.0,
     $                 omega(id), 1.0, 0.0)
               ENDIF
               CALL MAP_FFT_FILL_PPFT(PRJ_ROT, NCOL, NROW, FFT_PRJ_ROT,
     $              IDIM1, IDIM2, 0.0)
               CALL FFT_2D (FFT_PRJ_ROT, IDIM1, IDIM2)

               CALL GET_XY(FFT_R4_IMG, FFT_PRJ_ROT, XCEN, YCEN, IDIM1,
     $              IDIM2, RES_MAX,FFT_ORIGX, FFT_ORIGY, flag,
     $              ipx(id), ipy(id), peakx1(id), peakx2(id),
     $              peaky1(id), peaky2(id), step1(id), step2(id))

               if(flag < 0) then
                  print *, ' WARNING:  Center is more than 30% off'
                  print *, '          Particle #', id
                  print *, '          flag', flag
               endif
               XCEN = XCEN + 1.0 ! Get back into real space
               YCEN = YCEN + 1.0
               
C     Test centers to determine whether they are off the Cartesian grid
C     or more than 10% from the array center. These may be indicative of
C     poor data.
               
               IF (ABS(XCEN-FLOAT(NCOL/2)) > FLOAT(NCOL/2) .OR. 
     1              ABS(YCEN-FLOAT(NROW/2)) > FLOAT(NROW/2))  THEN
                  WRITE (*,410) INDX(ID), XCEN*BIN, YCEN*BIN
               ELSE IF (ABS(XCEN-FLOAT(NCOL/2)) > FLOAT(NCOL/10) .OR.
     1                 ABS(YCEN-FLOAT(NROW/2)) > FLOAT(NROW/10))  THEN
                  WRITE (*,415) INDX(ID), XCEN*BIN, YCEN*BIN
               ENDIF
               
C     Find/refine MAG scale factor and CMP coefficient, use a routine
C     adapted from CMP program, IMG and PRJ are compared in real space
               
               IF (MAG_CEN <= 0.0) THEN
                  MAG_LO = MAG     - (((MAG_NUM-1)/2)*MAG_STEP)
               ELSE
                  MAG_LO = MAG_CEN - (((MAG_NUM-1)/2)*MAG_STEP)
               ENDIF
               

C     If verbose is less than zero, don't need to calculate cc_cmp
C     IF VERBOSE > 0                                              
C     RECOMPUTE POLAR MAP OF RAW IMAGE WITH NEWEST X,Y ORIGIN
C     RECOMPUTE POLAR MAP OF ROTATED PROJECTION
C     COMPUTE COEFFICIENTS AS A FUNCTION OF RESOLUTION AND RADIUS
C     WRITE TO EXTRA OUTPUT FILES (UNITS 8,9,10)

               if(verbose >= 0) then
                  CALL GET_BESTMAG(IMG, PRJ, TMP0, TMP1, NCOL, NROW,
     $                 IDIM1,IDIM2, real(ANN_LO), real(ANN_HI), RES_MIN,
     $                 RES_MAX,XCEN-1.0,YCEN-1.0, omega(id), MAG_LO,
     $                 MAG_NUM, MAG_STEP,BEST_MAG, cmp_ccs(id), IMG_BKG)
                  MAG = BEST_MAG
                  CALL LIST_CCS(BIN, NCOL, NROW, IDIM1, IDIM2, NROT,
     $                 NANNULI, IMG, TMP0, R4_IMG, 
     $                 FFT_R4_IMG, FFT_PRJ_ROT, TMP1, TMP2, 
     $                 PRJ_ROT, PPRJ, PMAP, FLTPMAP, INDX(ID),
     $                 IMG_BKG, XCEN,YCEN, XCEN_MODEL, YCEN_MODEL,
     $                 RES_MIN, RES_MAX, ANN_LO,
     $                 ANN_HI, RADIUS, PIXRES_MAX,DEL_PIXRES, DEL_ANG,
     $                 CC_AVG, cc_res_globall, cc_rad_globall, verbose)
               else
                  MAG = 1.0
                  cmp_ccs(id) = 0.0
               endif
               
C     Return orientation to asymmetric unit
               if(sym .ne. 1) then
                  call to_asym_unit(sym, theta(id), phi(id), omega(id))
               endif

C     Make sure that omega stays in the range 0 to 360
               if(omega(id) < 0.0)    omega(id) = omega(id) + 360.0
               if(omega(id) >= 360.0) omega(id) = omega(id) - 360.0

               x(id)      = (xcen-1.0)*bin  
               y(id)      = (ycen-1.0)*bin  
               mags(id)    = mag             

C     Pack results into arrays that will be collected by root process
               real_results_local(1,local_count)    = theta(id)
               real_results_local(2,local_count)    = phi(id)
               real_results_local(3,local_count)    = omega(id)
               real_results_local(4,local_count)    = x(id)
               real_results_local(5,local_count)    = y(id)
               real_results_local(6,local_count)    = mags(id)
               real_results_local(7,local_count)    = pft_ccs(id)
               real_results_local(8,local_count)    = prj_ccs(id)
               real_results_local(9,local_count)    = cmp_ccs(id)
               real_results_local(10,local_count)   = peakx1(id)
               real_results_local(11,local_count)   = peaky1(id)
               real_results_local(12,local_count)   = step1(id)
               real_results_local(13,local_count)   = peakx2(id)
               real_results_local(14,local_count)   = peaky2(id)
               real_results_local(15,local_count)   = step2(id)
               integer_results_local(1,local_count) = ipx(id)
               integer_results_local(2,local_count) = ipy(id)
            endif
            
         enddo
      enddo      
      
C     Collect results back onto root process
#ifdef MPI
      call mpi_reduce(cc_res_globall, cc_res_global, 40,  mpi_real,
     +     mpi_sum, 0, mpi_comm_world, mpi_err)
      call mpi_reduce(cc_rad_globall, cc_rad_global, 800, mpi_real,
     +     mpi_sum, 0, mpi_comm_world, mpi_err)

      do i=1,num_nodes-1
         if(iam == i) then
            call mpi_send(real_results_local, 15*buffer_size(i),
     $           mpi_real, 0, i, mcw, mpi_err)
            call mpi_send(integer_results_local, 2*buffer_size(i),
     $           mpi_integer, 0, i, mcw, mpi_err)
         endif
         if(iam == 0) then
            call mpi_recv(real_results_local, 15*buffer_size(i),
     $           mpi_real, i, i, mcw, stat, mpi_err)
            call mpi_recv(integer_results_local, 2*buffer_size(i),
     $           mpi_integer, i, i, mcw, stat, mpi_err)
            
            do j = 1, buffer_size(i)
               k = i+1+(j-1)*num_nodes
               theta(k)   = real_results_local(1,j)
               phi(k)     = real_results_local(2,j)
               omega(k)   = real_results_local(3,j)
               x(k)       = real_results_local(4,j)
               y(k)       = real_results_local(5,j)
               mags(k)    = real_results_local(6,j)
               pft_ccs(k) = real_results_local(7,j)
               prj_ccs(k) = real_results_local(8,j)
               cmp_ccs(k) = real_results_local(9,j)
               peakx1(k)  = real_results_local(10,j)
               peaky1(k)  = real_results_local(11,j)
               step1(k)   = real_results_local(12,j)
               peakx2(k)  = real_results_local(13,j)
               peaky2(k)  = real_results_local(14,j)
               step2(k)   = real_results_local(15,j)
               ipx(k)     = integer_results_local(1,j)
               ipy(k)     = integer_results_local(2,j)
            enddo
         endif
      enddo
#else
      cc_res_global = cc_res_globall ! F90 array syntax
      cc_rad_global = cc_rad_globall ! F90 array syntax
#endif

      if(iam == 0) then

         do id = 1, nptles
            write(*,'(a,2i6)')"Best origin by simple search    :",
     $           ipx(id),ipy(id)
            write(*,'(a,3f12.4)')"Best origin by surface fit, step:",
     $           peakx1(id), peaky1(id), step1(id)
            write(*,'(a,3f12.4)')"Best origin by surface fit, step:",
     $           peakx2(id), peaky2(id), step2(id)
            write(*,1020) indx(id), theta(id), phi(id), omega(id),
     $           x(id), y(id), mags(id),
     $           pft_ccs(id), prj_ccs(id), cmp_ccs(id)
         enddo
         
         id = 0
         do i=1,nfiles
            call increment_fname(param_fnames(i), new_name)
            open(unit=11, file=new_name)
            write(11,'(A)') ' '//trim(adjustl(img_fnames(i)))
            write(11,1010) pixsiz(i), units(i), volts(i),
     $           amp_fac(i),focus_maj(i), focus_min(i),
     $           ang_maj(i), cs_coeff(i)
            do j=1,ptle_count(i)
               id = id+1
               write(11,1020) indx(id), theta(id), phi(id), omega(id),
     $              x(id), y(id), mags(id),
     $              pft_ccs(id), prj_ccs(id), cmp_ccs(id)
            enddo
            close(11)
         enddo
         
         if(verbose >= 1) then
            j = nint(1/(pixres_max*del_pixres))
            write(8,1209)
            write(8,'(a,10f7.3)') ' AVG', 
     $           (cc_res_global(1,i)/real(nptles), i=1,j)
            write(10,1209)
            write(10,'(A,10F7.3)') ' AVG', 
     1           (cc_res_global(2,i)/real(nptles), i=1,j)
            write(9,1210) nptles
            do i=1,nannuli
               j = (i-1)*bin
               write(9,1211) j, cc_rad_global(1,i)/real(nptles), 
     1              cc_rad_global(2,i)/real(nptles)
            enddo
         endif
 
      endif
      
 1010 format (1x,f8.4,',',i2,',',f10.1,',',3(f10.4,','),f10.4,',',f8.2)
 1020 format (1x,i6,',',4(f8.3,','),f7.3,',',3(f6.3,','),f6.3)
 1209 format(74('-')) 
 1210 format(' AVERAGE correlation coefficients as function ',
     $     'of radius for', I6,' images'/,' CCs are for unfiltered/',
     $     'filtered IMG vs. POLAR_PRJ',/
     $     ,' Radius Unfiltered Filtered', /
     $     ,'---------------------------')
 1211 format(i5,2f10.3)

      if(compare_mode == 1) then
         deallocate(prjs_gold)
      endif
      
      deallocate(pprj, prj, prj_rot, prj_avg, pftprj, pftimg, pfft,
     $     pmap, fltpmap, var, fft_r4_img, fft_prj_avg, fft_prj_rot,
     $     img, r4_img, tmp0, tmp1, tmp2, correction, abs_ctf)
      
      return
      end subroutine global_cc
