      subroutine rpifimag_f (dimem, pixels)

C     Author: Hong Lin (linh@cs.purdue.edu)                        
C     Date:   1998                                                 
C     Used in: read_files -- The subroutine reading PIF file       
C     Function: Read in a pixel frame from an external file.       
C
C     Input:                                                       
C       dimem (Integer) -- The limits on the size of array "pixels"
C     Output:                                                      
C       pixels (Real) -- The pixel frame in real type.             
C     
C     Algorithm:                                                   
C
C     The storage format of the pixel frames in external file are
C     determined by "mode". Currently, only three modes are used in the
C     3DR program: Mode 0, Mode 1 and Mode 6. In Mode 0 and Mode 6, the
C     pixels are represented as bytes; whilst in Mode 1, the pixels are
C     represented as short integers. To read a pixel frame from the
C     file, two buffers are used correspondenly, in byte or in in short
C     integer. No matter what type is the buffer, the pixels are
C     converted into real number and put into output array "pixels". The
C     data conversion is done immediately after the pixel frame is read,
C     i.e., in subroutine "pif_read_byte_image" and "pif_read_imgi1".
C
C  Modification history:                                        
C     - Hong Lin, Version 1.0, Aug.18, 1998                    
C     - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002       
C     - Robert Sinkovits, general cleanup 11/28/2006

      implicit none
      INCLUDE '../include/allprogs.inc'
      INCLUDE '../include/pifhead.inc'
      type (PIFHEAD_T) MAPHEAD, IMGHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD

C     There is a global head and a local head before each pixel frame.
C     "IMGHEAD" is used to read these heads.

      integer, intent(in) :: dimem
      real, intent(out)   :: pixels(dimem)

C     "pixels_buf_byte" -- Buffer for reading pixel frame in Mode 0 and 6
C     "pixels_buf_int2" -- Buffer for reading pixel frame in Mode 1

      character pixels_buf_byte(dimem)
      integer*2 pixels_buf_int2(dimem)

      integer   check_index, check_index1, i 
      integer*2 j
      real    rj

      if (IMGHEAD%mode == 0 .or. IMGHEAD%mode == 6) then

         call pif_read_byte_image (IMGHEAD%luNum, pixels_buf_byte,
     $        IMGHEAD%imgOffset, IMGHEAD%numCols, IMGHEAD%numRows, 1)
         j = ichar(pixels_buf_byte(1))
         i = j
         if (i < 0) then
            i = i + 256
         endif
         do check_index = 1, dimem
            j = ichar(pixels_buf_byte(check_index))
            check_index1 = j
            if (check_index1 < 0) then
               check_index1 = check_index1 + 256
            endif
            pixels(check_index) = real(check_index1 - i)
         enddo

      elseif (IMGHEAD%mode == 1) then

         call pif_read_short_image (IMGHEAD%luNum, pixels_buf_int2,
     $        IMGHEAD%imgOffset, IMGHEAD%numCols, IMGHEAD%numRows, 1)
         j = pixels_buf_int2(1)
         i = j
         if (i < 0) then
            i = i + 256*256
         endif
         do check_index = 1, dimem
            j = pixels_buf_int2(check_index)
            check_index1 = j
            if (check_index1 < 0) then
               check_index1 = check_index1 + 256*256
            endif
            pixels(check_index) = real(check_index1-i)
         enddo

      elseif (IMGHEAD%mode == 46) then

         call pif_read_boxi4(IMGHEAD, pixels)
         rj = pixels(1)
         do check_index = 1, dimem
            pixels(check_index) = pixels(check_index) - rj
         enddo

      else

         print *, IMGHEAD%mode
         call error_stop(progname, 'rpifimag_f',
     +        'Undefined mode. Program terminated!')

      endif

      return
      end subroutine rpifimag_f
