C  ****************************************************************************
C  *                                                                          *
C  *                     *****     FFTLIB1.SUBS     *****                     *
C  *                                                                          *
C  *    THIS IS A LIBRARY OF SUBROUTINES WHICH ARE USED TO MANIPULATE "FFT"   *
C  *     ARRAY DATA.  THE OBJECT CODE FOR ALL THE ROUTINES ARE KEPT IN THE    *
C  *     LIBRARY SYS$EMLIB:TSBLIB.OLB                                         *
C  *                                                                          *
C  *    LIST OF SUBROUTINES:                                                  *
C  *                                                                          *
C  *      FFT_1D, FFT_1DLOP, FFT_1DSETDIM_DEF, FFT_2D, FFT_2D_BACK, FFT_2DBT, *
C  *      FFT_CLEAR, FFT_CAVG, FFT_COPY1, FFT_FILL,    *
C  *      FFT_FLIP, FFT_FLT, FFT_FLTMSK, FFT_FLTLLMSK, FFT_HIP, FFT_IMG_FILL, *
C  *      FFT_MAP_FILL, FFT_LOP, FFT_MAP, FFT_MRCTRANSF, FFT_MULT,            *
C  *      FFT_OPEN, FFT_PSHIFT, FFT_SETDIM, FFT_SETDIM_DEF,                   *
C  *      FFT_SETDIM_DEF_SAME, FFT_SPT_BACK, FFT_SPT_BOX, FFT_SPT_BOXES,      *
C  *      FFT_SPT_BOXOUT, FFT_SPT_GRID, FFT_SPT_INFO, FFT_SPT_LAT,            *
C  *      FFT_SPT_NOBACK, FFT_STATS, FFT_TO_ATBT, FFT_TO_ATBT2, FFT_TYPE      *
C  *                                                                          *
C  *    APRIL 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.                     *
C  ****************************************************************************
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FFTLIB2.SUBS     *****                      *
C  *                                                                          *
C  *    THIS CONTAINS THE FORTRAN SOURCE CODE FOR A VARIETY OF FOURIER        *
C  *     TRANSFORM SUBROUTINES WHICH ARE CONTAINED IN THE OBJECT LIBRARY      *
C  *     "DEXTRO3:TSBLIB.OLB"                                                 *
C  *                                                                          *
C  *    NOTE: THESE SUBROUTINES SHOULD ALL BE COMPILED WITH DEFAULT INT*4     *
C  *                                                                          *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  ****************************************************************************
C
C
C
C
C

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE FFT_1D     *****                  *
C  *                                                                          *
C  *    COMPUTE FOURIER TRANSFORM OF 1-DIMENSIONAL ARRAY (OF DIMENSION IDIM)  *
C  *     IN FORWARD OR REVERSE DIRECTION.                                     *
C  *                                                                          *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *    LAST MODIFIED  7-SEP-1989 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE FFT_1D (FFT, IDIM, DIRECTION)
C
      IMPLICIT NONE
C
      INTEGER*4 I, IDIM, DIRECTION
      REAL*4    FFT(1), DIV, work
C  ----------------------------------------------------------------------------
C
      work = 0.0
      IF(DIRECTION.EQ.1) THEN
         CALL FOURT (FFT, IDIM, 1, 1, 0, work)      ! FORWARD TRANSFORM
        ELSE IF (DIRECTION.EQ.-1) THEN
         CALL FOURT (FFT, IDIM, 1,-1, 1, work)      ! REVERSE TRANSFORM
        ELSE
         CALL BELL (2)
         PRINT *, '******************************************'
         PRINT *, '* FFT_1D Subroutine arguement to signify *'
         PRINT *, '*  forward(1) or reverse(-1) FFT not     *'
         PRINT *, '*  specified correctly.                  *'
         PRINT *, '******************************************'
         RETURN
      ENDIF
C
      DIV = SQRT(FLOAT(IDIM))
      DO I = 1,2*IDIM
        FFT(I) = FFT(I)/DIV
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FFT_1DLOP     *****                *
C  *                                                                          *
C  *    APPLY LOW-PASS MASK TO 1-DIMENSIONAL FOURIER TRANSFORM.               *
C  *                                                                          *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *    LAST MODIFIED  7-SEP-1989 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE FFT_1DLOP (FFT, IDIM, RES_MIN, RES_MAX)
C
      IMPLICIT NONE
C
      INTEGER*4 I, IDIM, IDIMBY2
      REAL*4    RES_MIN, RES_MAX, RES_MINSQ, RSQ, DELSQ, WEIGHT
      REAL*4    FFT(2,IDIM)
C  ----------------------------------------------------------------------------
C
      RES_MINSQ = RES_MIN**2
      DELSQ     = (RES_MAX-RES_MIN)**2
      IDIMBY2   = IDIM/2
C
      DO I = 1,IDIM
        IF(I.LE.IDIMBY2) THEN
           RSQ = FLOAT(I-1)**2
         ELSE
           RSQ = FLOAT(IDIM-I+1)**2
        ENDIF
        IF(RSQ.GT.RES_MINSQ)THEN
          IF(DELSQ.EQ.0.0) THEN
             WEIGHT = 0.0
           ELSE
             IF(RES_MIN.GT.0.0) THEN
                WEIGHT = EXP(-((SQRT(RSQ)-RES_MIN)**2/DELSQ))
              ELSE
                WEIGHT = EXP(-RSQ/DELSQ)
             ENDIF
          ENDIF
          FFT(1,I) = WEIGHT*FFT(1,I)
          FFT(2,I) = WEIGHT*FFT(2,I)
        ENDIF
      ENDDO
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *             *****     SUBROUTINE FFT_1DSETDIM_DEF     *****            *
C  *                                                                        *
C  *    SET UP DEFAULT 1D FOURIER TRANSFORM DIMENSION.                      *
C  *                                                                        *
C  *    THE SIZE OF THE 1D FFT DIMENSION (IDIM) WILL BE THE SMALLEST POWER  *
C  *     OF 2 ALLOWED FOR THE NPTS DIMENSION MULTIPLIED BY NFAC.            *
C  *                                                                        *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    LAST MODIFIED 31-AUG-1989 BY T. BAKER.                              *
C  **************************************************************************
C
      SUBROUTINE FFT_1DSETDIM_DEF (NPTS, IDIM, NFAC)
C
      IMPLICIT NONE
C
      INTEGER*4 NPTS, I, IDIM, NFAC, NUMCHARI4, N1
      REAL*4    TEST
C  ----------------------------------------------------------------------------
C
C  ************************************
C  *    DETERMINE MINIMUM FFT SIZE    *
C  ************************************
      TEST = FLOAT(NPTS*(2**(NFAC-1)))
      I    = 0
      DO WHILE (TEST.GT.1.0)
        I    = I + 1
        TEST = TEST/2.0
      ENDDO
      IDIM = 2**I
C
      N1 = NUMCHARI4(IDIM) + 1
CCC      WRITE(*,'('' 1D FFT dimension is'',I<N1>)') IDIM
C
      RETURN
      END


C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE FFT_2DBT     *****              *
C  *                                                                  *
C  *    COMPUTE IN-CORE 2D REVERSE FFT OF THE FULL-SIZE "FFT" ARRAY   *
C  *                                                                  *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW "IMAGE".      *
C  *                                                                  *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).             *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible). *
C  ********************************************************************
C
      SUBROUTINE FFT_2DBT (FFT, IDIM1, IDIM2, IMG, NCOL, NROW,
     1 FFT_FULL, IMG_BKG)
C
      IMPLICIT NONE
C
      INTEGER*4 NN(2), IDIM1, IDIM2, I, J, NCOL, NROW
      INTEGER*4 ARRAY_SIZE
      real*4    IMG_BKG,IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2), WORK, DIV
      real*4, allocatable :: fft2(:)
      LOGICAL   FFT_FULL
      integer alloc_stat
C  -----------------------------------------------------------------------------
C
      NN(1)      = IDIM1
      NN(2)      = IDIM2
      DIV        = FLOAT(IDIM1)*FLOAT(IDIM2)
      ARRAY_SIZE = 2*IDIM1*IDIM2

      allocate(fft2(array_size), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate memory in FFT_2DBT'
         call tsbend
      endif

      CALL COPY_R4 (FFT, FFT2, ARRAY_SIZE)
C
      CALL FOURT (FFT, NN, 2, -1, 1, WORK)               ! TRANSFORM THE DATA
      DO J = 1,IDIM2
        DO I = 1,2*IDIM1
          FFT(I,J) = FFT(I,J)/DIV
        ENDDO
      ENDDO
C
      FFT_FULL = .TRUE.
C
      DO J = 1,NROW
        DO I = 1,NCOL
          IMG(I,J) = FFT((2*I)-1,J) + IMG_BKG
        ENDDO
      ENDDO
C
      CALL COPY_R4 (FFT2, FFT, ARRAY_SIZE)
      deallocate(fft2)

      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *          *****     SUBROUTINE FFT_2DBT_R4     *****                *
C  *                                                                    *
C  *    COMPUTE IN-CORE 2D REVERSE FFT OF THE FULL-SIZE "FFT" ARRAY     *
C  *                                                                    *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW REAL*4 ARRAY.   *
C  *                                                                    *
C  *    AUGUST 1999 PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    LAST MODIFIED 24-AUG-1999 BY T. BAKER (ADAPTED FROM FFT_2DBT).  *
C  **********************************************************************
C
      SUBROUTINE FFT_2DBT_R4 (FFT, IDIM1, IDIM2, MAP, NCOL, NROW,
     1 FFT_FULL, IMG_BKG)
C
      IMPLICIT NONE
C
      INTEGER*4 NN(2), IDIM1, IDIM2, I, J, NCOL, NROW
      INTEGER*4 ARRAY_SIZE
      real*4    IMG_BKG
      REAL*4    FFT(2*IDIM1,IDIM2), MAP(NCOL,NROW), WORK, DIV
      real*4, allocatable :: fft2(:)
      integer   alloc_stat
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      NN(1)      = IDIM1
      NN(2)      = IDIM2
      DIV        = FLOAT(IDIM1)*FLOAT(IDIM2)
      ARRAY_SIZE = 2*IDIM1*IDIM2

      allocate(fft2(array_size), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate memory in FFT_2DBT_R4'
         call tsbend
      endif

      CALL COPY_R4 (FFT, FFT2, ARRAY_SIZE)
C
      CALL FOURT (FFT, NN, 2, -1, 1, WORK)               ! TRANSFORM THE DATA
      DO J = 1,IDIM2
        DO I = 1,2*IDIM1
          FFT(I,J) = FFT(I,J)/DIV
        ENDDO
      ENDDO
C
      FFT_FULL = .TRUE.
C
      DO J = 1,NROW
        DO I = 1,NCOL
          MAP(I,J) = FFT((2*I)-1,J) + IMG_BKG
        ENDDO
      ENDDO
C
      CALL COPY_R4 (FFT2, FFT, ARRAY_SIZE)
      deallocate(fft2)

      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE FFT_CLEAR     *****             *
C  *                                                                  *
C  *    ZEROES EITHER FULL OR HALF SIZE FFT ARRAYS.                   *
C  *                                                                  *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.            *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).        *
C  ********************************************************************
C
      SUBROUTINE FFT_CLEAR (FFT, IDIM1, IDIM2, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, I, ARRAY_SIZE
      REAL*4    FFT(1), ZERO
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      IF(FFT_FULL) THEN
         ARRAY_SIZE = 2*IDIM1*IDIM2
       ELSE
         ARRAY_SIZE = IDIM1*(IDIM2 + 2)
      ENDIF
      ZERO = 0.0
C
      DO I = 1,ARRAY_SIZE
        FFT(I) = ZERO
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                  *****  SUBROUTINE FFT_COPY1  *****                *
C  *                                                                    *
C  *    COPY TOP HALF OF FFT2 ARRAY INTO FFT1 ARRAY.                    *
C  *                                                                    *
C  *    JUNE 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.                *
C  *    MODIFIED 12-JUN-1992 BY T. BAKER.                               *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).   *
C  **********************************************************************
C
      SUBROUTINE FFT_COPY1 (FFT2, FFT1, IDIM1, IDIM2)
C
      IMPLICIT NONE
C
      INTEGER*4 I, J, IDIM1, IDIM2, SIZE
      REAL*4    FFT1(2*IDIM1,((IDIM2/2)+1)), FFT2(2*IDIM1,IDIM2)
C  ----------------------------------------------------------------------------
C 
CCC      SIZE = 8*IDIM1
C
      DO J = 1,(IDIM2/2)+1
        DO I = 1,2*IDIM1
          FFT1(I,J) = FFT2(I,J)
CCC          CALL LIB$MOVC3 (SIZE, FFT2(1,J), FFT1(1,J))
        ENDDO
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                  *****  SUBROUTINE FFT_CAVG  *****                 *
C  *                                                                    *
C  *    COMPUTE CIRCULARLY AVERAGED FFT.                                *
C  *                                                                    *
C  *    SEPTEMBER 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED  3-MAR-1992 BY T. BAKER.                          *
C  *    Commented out GRAPHICS options 19 May 1994, D. Belnap           *
C  **********************************************************************
C
      SUBROUTINE FFT_CAVG (FFT, IDIM1, IDIM2, INTERP, NTERP, RINT,
     1                    IRNUM)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 NTERP, I, J, K, IDIM1, IDIM2, XTEST
      INTEGER*4 INUM, IR, IPUCK, IXTAB, IYTAB, IRNUM(NTERP), ICOL
      INTEGER*4 XOFF, YOFF, MULT, VALUE
      REAL*4    FFT(2*IDIM1,IDIM2), RINT(NTERP), RINT_MIN, RINT_MAX
      REAL*4    SCALE, X, Y, YSQ, TEST, RADIUS, APART, BPART, INTERP
      LOGICAL   GRAPHICS, ANS, DUMP, AMPS
C -----------------------------------------------------------------------------
C
c  100      WRITE(*,'(A)') '$DISPLAY ON LEXIDATA GRAPHICS? Y/<N>: '
c      CALL GET_YESNO (ANS, .FALSE.)
c      IF(ANS) THEN
c         GRAPHICS = .TRUE.
c       ELSE
c         GRAPHICS = .FALSE.
c      ENDIF
c      IF(GRAPHICS) THEN
c        LXDEV_1280 = .TRUE.
c        CALL LX_INIT (.TRUE.)
c        CALL LX_TABINIT
c        CALL LX_CTSET (2, 1, 255, 255, 0)
c        CALL LX_CTSET (4, 1, 200, 200, 200)
c        ICOL = 2
c      ENDIF
 200  WRITE(*,'(A)') '$AMPLITUDES<CR> OR INTENSITIES(1): '
      READ (*,*,ERR=200) I
      IF(I.EQ.1) THEN
         AMPS = .FALSE.
       ELSE
         AMPS = .TRUE.
      ENDIF
C
      INUM = NTERP
C
      DO I = 1,INUM                      ! INITIALIZE 1D INTERPOLATION ARRAY
        RINT(I)  = 0.0
        IRNUM(I) = 0
      ENDDO
C  *********************************************************
C  *    STEP THROUGH FFT AND REDUCE TO CIRCULAR AVERAGE    *
C  *********************************************************
      XTEST = IDIM1/2
C
      DO J = 1,(IDIM2/2)+1
         Y = FLOAT(J-1)
         YSQ = Y*Y
         DO I = 1,IDIM1
            APART = FFT(2*I-1,J)
            BPART = FFT(2*I,J)
            IF(I.LT.XTEST) THEN
               X = FLOAT(I-1)
             ELSE
               X = FLOAT(I-IDIM1-1)
            ENDIF
            IR = NINT(INTERP*SQRT(X*X+YSQ)) + 1
            IRNUM(IR) = IRNUM(IR) + 1
            IF(AMPS) THEN
               RINT(IR)  = RINT(IR) + SQRT(APART*APART + BPART*BPART)
             ELSE
               RINT(IR)  = RINT(IR) + APART*APART + BPART*BPART
            ENDIF
         ENDDO
      ENDDO
C  ********************************
C  *    NORMALIZE THE 1D ARRAY    *
C  ********************************
      RINT_MIN = R4_MAX
      RINT_MAX = R4_MIN
C
      WRITE(*,'(A)') '$DUMP DATA INTO FFT_1D.TMP FILE Y/<N>: '
      CALL GET_YESNO (DUMP, .FALSE.)
      IF(DUMP) THEN
        PRINT *, '***************************************************'
        PRINT *, '* Data written to ASCII output file: EMFFT_1D.TMP *'
        PRINT *, '***************************************************'
        OPEN (UNIT=8, FILE='FFT_1D.TMP', STATUS='NEW',
     1  FORM='FORMATTED', ACCESS='SEQUENTIAL')
      ENDIF
C      
      DO I = 1,INUM
        IF (IRNUM(I).GT.1) RINT(I) = 5.0*RINT(I)/FLOAT(IRNUM(I))
        RINT_MIN = MIN(RINT_MIN,RINT(I))
        RINT_MAX = MAX(RINT_MAX,RINT(I))
        IF(RINT(I).GT.100000000.) THEN
           WRITE(*,'(I5,E12.5)') I-1, RINT(I)
         ELSE
           WRITE(*,'(I5,F12.2)') I-1, RINT(I)
        ENDIF
        IF(DUMP) WRITE (8,'(F5.0,F12.2)') FLOAT(I-1), RINT(I)
      ENDDO
      IF(DUMP) CLOSE (UNIT=8, STATUS='KEEP')
c      IF(.NOT.GRAPHICS) RETURN
C
c      CALL LX_ERASE
c      IF(LXDEV_1280) THEN
c         XOFF = 100
c         YOFF =  50
c         MULT =   2
c       ELSE
c         XOFF = 50
c         YOFF = 25
c         MULT =  1
c      ENDIF
c      TEST  = MAX(ABS(RINT_MIN),ABS(RINT_MAX))
c      SCALE = FLOAT(INUM)/TEST
c      INUM  = NINT(FLOAT(INUM)/1.2)
c      DO I = 1,INUM
c        VALUE = NINT(RINT(I)*SCALE) + YOFF
c        CALL LX_PUTPT (XOFF+MULT*(I-1), VALUE, ICOL)
c      ENDDO
c      ICOL = 4
c      CALL LX_VEC (XOFF, YOFF, XOFF+(MULT*INUM), YOFF, ICOL)
c      CALL LX_VEC (XOFF, YOFF, XOFF, YOFF+INUM, ICOL)
C
c      LXCUR_TYPE = 0
c      LXCUR_COL  = 0
c      CALL LX_CURON
c      PRINT *, '********************************************'
c      PRINT *, '* Use YELLOW, WHITE or BLUE to mark radius *'
c      PRINT *, '*             Hit GREEN to EXIT            *'
c      PRINT *, '********************************************'
c      WRITE(*,'(A)') ' Radius = '
c  500      CALL LX_TABPOS (IXTAB, IYTAB, IPUCK)
c      IF(IPUCK.NE.3) THEN
c        RADIUS = (FLOAT(IXTAB)-FLOAT(XOFF))/(FLOAT(MULT)*INTERP)
c        WRITE(*,'(A,F10.2)') '+Radius = ', RADIUS
c        GO TO 500
c      ENDIF
c      CALL LX_TABOFF
c      CALL LX_CUROFF
c      CALL LX_CLOSE
C
      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *               *****     SUBROUTINE FFT_FILL     *****               *
C  *                                                                     *
C  *    READ FFT DATA FROM FILE AND PACK INTO THE HALF-SIZE "FFT" ARRAY  *
C  *                                                                     *
C  *    JANUARY 1985  PURDUE UNIVERSITY  VAX 11/750, T. BAKER.           *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).    *
C  ***********************************************************************
C
      SUBROUTINE FFT_FILL (LUN, FFT, IDIM1, IDIM2, TMP, FFT_FULL)
C
      IMPLICIT NONE
C      
      INTEGER*4 IDIM1, IDIM2, LUN, I, J, JEND, JSTEP, MOVE_SIZE, IREC
      REAL*4    FFT(2*IDIM1,(IDIM2/2)+1), TMP(2*IDIM1), PERCENT, DIV
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
      FFT_FULL = .FALSE.                  ! SIGNIFY HALF-SIZE FFT ARRAY
      JEND     = ((IDIM2/2)+1)
      JSTEP    = JEND/10
      DIV      = 100.0/FLOAT(JEND)
C
      IF(JSTEP.LT.1) JSTEP = 1
C
      IREC  = 4                        ! FIRST DATA RECORD OF FFT FILE
CCC      MOVE_SIZE = 8*IDIM1                  ! # BYTES/RECORD
      DO J = 1,JEND
        READ (LUN,REC=IREC) TMP
        IREC = IREC + 1
        DO I = 1,2*IDIM1
          FFT(I,J) = TMP(I)
CCC          CALL LIB$MOVC3 (MOVE_SIZE, TMP(1), FFT(1,J))
        ENDDO
      ENDDO
C
      CLOSE (UNIT=LUN, STATUS='KEEP')            ! CLOSE AND SAVE FFT FILE
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FFT_FLIP     *****                 *
C  *                                                                          *
C  *    MULTIPLY ALL "B" PARTS OF THE FFT BY -1 TO EFFECTIVELY ROTATE THE     *
C  *     "IMAGE" BY 180 DEGREES.                                              *
C  *                                                                          *
C  *    NOTE: THE "IMAGE" IS EFFECTIVELY ROTATED WITHIN A FULL IDIM1 BY IDIM2 *
C  *          BOUNDARY, NOT WITHIN THE ORIGINAL NCOL BY NROW BOUNDARY.        *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FFT_FLIP (FFT, IDIM1, IDIM2, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 I, IDIM1, IDIM2, ARRAY_SIZE
      REAL*4    FFT(1)
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
      IF(FFT_FULL) THEN
         ARRAY_SIZE = 2*IDIM1*IDIM2
       ELSE
         ARRAY_SIZE = IDIM1*(IDIM2 + 2)
      ENDIF
C
      DO I = 2,ARRAY_SIZE,2
        FFT(I) = -FFT(I)
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                *****     SUBROUTINE FFT_FLT     *****              *
C  *                                                                    *
C  *    APPLY HARD OR GAUSSIAN LOW-PASS AND/OR HI-PASS FOURIER CUTOFF   *
C  *                                                                    *
C  *    THIS ROUTINE SUPERCEDES THE OLD FFT_FLTHIP AND FFT_FLTLOP.      *
C  *                                                                    *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).               *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).   *
C  **********************************************************************
C
      SUBROUTINE FFT_FLT (FFT, IDIM1, IDIM2, RES_MIN, RES_MAX,
     1 CUT_MIN, CUT_MAX, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2
      REAL*4    FFT(1), RES_MIN, RES_MAX, CUT_MIN, CUT_MAX
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
      IF(RES_MIN.GT.0.0) THEN
        CALL FFT_HIP (FFT, IDIM1, IDIM2, RES_MIN, CUT_MIN, FFT_FULL)
      ENDIF
      IF(RES_MAX.GT.0.0) THEN
        CALL FFT_LOP (FFT, IDIM1, IDIM2, RES_MAX, CUT_MAX, FFT_FULL)
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE FFT_FLTMSK     *****                *
C  *                                                                          *
C  *    SUB-OPTION "B" OF EMFFT_FILTER SUBPROGRAM                             *
C  *                                                                          *
C  *    APPLY 2D RECIPROCAL LATTICE MASK WITH SHARP OR SOFT HOLE EDGES        *
C  *     RADIUS = RADIUS OF THE CIRCULAR MASK HOLE.                           *
C  *                                                                          *
C  *    AUGUST 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                  *
C  *    LAST MODIFIED  2-MAR-1992 BY T. BAKER.                                *
C  ****************************************************************************
C
      SUBROUTINE FFT_FLTMSK (FFT, MASK, IDIM1, IDIM2, AX, AY, BX, BY)
C
      IMPLICIT NONE
C
      INTEGER*4 I, II, J, JJ, IDIM1, IDIM2, NH, NK, NH1, NK1, NHOLE
      INTEGER*4 IX, IX1, IX2, IY, IY1, IY2, ICUT, XTEST, NHOLE_LIM
      PARAMETER (NHOLE_LIM = 1000)
      INTEGER*2 MASK(IDIM1,((IDIM2/2)+1))
      REAL*4    FFT(2*IDIM1,((IDIM2/2)+1)), RTST, XLIM, YLIM
      REAL*4    TMP, X(3,NHOLE_LIM), Y(3,NHOLE_LIM)
      REAL*4    XCEN, YCEN, X1, X2, Y1, Y2, XB, YB, RADIUS, RSQ, RYSQ
      REAL*4    AX, AY, BX, BY
C  ----------------------------------------------------------------------------
C
Ca      CALL VT_CLS
      PRINT *, '***********************************************'
      PRINT *, '* SUB-OPTION B: Apply reciprocal lattice mask *'
      PRINT *, '***********************************************'
      PRINT *
C
  100      WRITE(*,'(2A)') '$GET LATTICE COORDINATES FROM FFTLAT.DAT',
     1 ' <CR> OR ENTER BY HAND (1)? : '
      READ (*,*) I
      IF(I.NE.1) THEN
         OPEN (UNIT=2, FILE='FFTLAT.DAT', STATUS='OLD',
     1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ')
Ca      1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ', SHARED)
         READ (2,*) AX, AY, BX, BY
         CLOSE (UNIT=2, STATUS='KEEP')
       ELSE
         WRITE(*,'(A)') '$ENTER AX,AY,BX,BY (4F): '
         READ (*,*) AX, AY, BX, BY
         IF(AX.EQ.0.0.AND.AY.EQ.0.0.AND.BX.EQ.0.0.AND.BY.EQ.0.0) THEN
           CALL BELL(2)
           PRINT *, '*****************************'
           PRINT *, '* ERROR...........Try again *'
           PRINT *, '*****************************'
           GO TO 100
         ENDIF
      ENDIF
C
      WRITE(*,'(A)') '$RADIUS OF MASK HOLES <CR>=1.0: '
      READ (*,*) RADIUS
      IF(RADIUS.LE.0.0) RADIUS = 1.0
      RTST = RADIUS*RADIUS
      WRITE(*,'(A)') '$HARD<CR> OR SOFT(1) CUTOFF OF MASK HOLES? : '
      READ (*,*) ICUT
      IF(ICUT.NE.1) ICUT = 0
C
      IF(AX.EQ.0.0) AX = 0.0001      ! TRAP FOR FLOATING DIVIDE BY ZERO
      IF(AY.EQ.0.0) AY = 0.0001
      IF(BX.EQ.0.0) BX = 0.0001
      IF(BY.EQ.0.0) BY = 0.0001
      I    = 1.5*FLOAT(IDIM1)/AX
      J    = 1.5*FLOAT(IDIM1)/AY
      NH   = MIN(IABS(I),IABS(J))
      I    = 1.5*FLOAT(IDIM2)/BX
      J    = 1.5*FLOAT(IDIM2)/BY
      NK   = MIN(IABS(I),IABS(J))
      NH1  = (NH-1)/2
      NK1  = (NK-1)/2
      XLIM = FLOAT(IDIM1)
      YLIM = FLOAT(IDIM2/2)
C  *******************************************************
C  *    DETERMINE CENTER AND BOUNDARIES FOR EACH HOLE    *
C  *******************************************************
      NHOLE = 0
      DO I = 1,NK
        II = I-NK1-1
        XB = II*BX
        YB = II*BY
        DO J = 1,NH
          JJ   = J-NH1-1
          XCEN = XB+JJ*AX+1.0+(IDIM1/2)
          X1   = XCEN-RADIUS
          X2   = XCEN+RADIUS
          IF(X1.LT.1.0)  X1 = 1.0
          IF(X2.LT.1.0)  X2 = 1.0
          IF(X1.GT.XLIM) X1 = XLIM
          IF(X2.GT.XLIM) X2 = XLIM
          YCEN = YB+JJ*AY+1.0
          Y1   = YCEN-RADIUS
          Y2   = YCEN+RADIUS
          IF(Y1.LT.1.0)  Y1 = 1.0
          IF(Y2.LT.1.0)  Y2 = 1.0
          IF(Y1.GT.YLIM) Y1 = YLIM
          IF(Y2.GT.YLIM) Y2 = YLIM
          IF(X1.NE.X2.AND.Y1.NE.Y2) THEN
            NHOLE      = NHOLE+1
            X(1,NHOLE) = XCEN
            X(2,NHOLE) = X1
            X(3,NHOLE) = X2
            Y(1,NHOLE) = YCEN
            Y(2,NHOLE) = Y1
            Y(3,NHOLE) = Y2
          ENDIF
        ENDDO
      ENDDO
C  ******************************
C  *    ZERO THE FILTER MASK    *
C  ******************************
      DO IY = 1,(IDIM2/2)+1
        DO IX = 1,IDIM1
          MASK(IX,IY) = 0
        ENDDO
      ENDDO
C  ******************************************
C  *    CONSTRUCT 2D LATTICE FILTER MASK    *
C  ******************************************
      DO I = 1,NHOLE
        IX1 = X(2,I)
        IX2 = X(3,I)
        IY1 = Y(2,I)
        IY2 = Y(3,I)
        DO IY = IY1,IY2
          RYSQ = (IY-Y(1,I))**2
          IF(RYSQ.LE.RTST) THEN
            DO IX = IX1,IX2
              RSQ = RYSQ + (IX-X(1,I))**2
              IF(RSQ.LE.RTST) THEN
                IF(ICUT.NE.1) THEN
                   MASK(IX,IY) = 1000                          ! HARD CUTOFF
                 ELSE
                   MASK(IX,IY) = NINT(1000.0*EXP(-RSQ/RTST))        ! SOFT CUTOFF
                ENDIF
              ENDIF
            ENDDO
          ENDIF
        ENDDO
      ENDDO
C  **********************************
C  *    APPLY FILTER MASK TO FFT    *
C  **********************************
      XTEST = IDIM1/2
      DO IY = 1,(IDIM2/2)+1
        DO IX = 1,IDIM1
          IF(IX.LE.XTEST) THEN
             IX1 = 2*IX-1+IDIM1
           ELSE
             IX1 = 2*IX-1-IDIM1
          ENDIF
          IX2         = IX1+1
          FFT(IX1,IY) = FFT(IX1,IY)*MASK(IX,IY)/1000.0
          FFT(IX2,IY) = FFT(IX2,IY)*MASK(IX,IY)/1000.0
        ENDDO
      ENDDO
C
      RETURN
      END

C  ***************************************************************
C  *                                                             *
C  *         *****     SUBROUTINE FFT_FLTLLMSK     *****         *
C  *                                                             *
C  *    SUB-OPTION "C" OF EMFFT_FILTER SUBPROGRAM                *
C  *                                                             *
C  *    APPLY 2D LAYERLINE MASK TO FFT                           *
C  *                                                             *
C  *    D IS THE PERPENDICULAR DISTANCE FROM THE POSITION X,Y    *
C  *     IN THE FFT TO THE EQUATORIAL LAYERLINE                  *
C  *                                                             *
C  *    NOTE: ONLY WORKS ON HALF-SIZE FFT ARRAYS.                *
C  *                                                             *
C  *    AUGUST 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.     *
C  *    LAST MODIFIED  5-SEP-1989 BY T. BAKER.                   *
C  ***************************************************************
C
      SUBROUTINE FFT_FLTLLMSK (FFT, IDIM1, IDIM2)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, XTEST, I, IX, IY
      REAL*4    Y, D, SPACE, COSANG, TANANG, FALL, WEIGHT
      REAL*4    FFT(2*IDIM1,((IDIM2/2)+1)), FRAC, ANGLE
      real*8    deg_to_rad
C  ----------------------------------------------------------------------------
C
Ca      CALL VT_CLS
      PRINT *, '**************************************'
      PRINT *, '* SUB-OPTION C: Apply layerline mask *'
      PRINT *, '**************************************'
      PRINT *
C
      PRINT *, '************************************************************'
      PRINT *, '*     Enter values for ANGLE, SPACING and FALLOFF where    *'
      PRINT *, '*                                                          *'
      PRINT *, '* ANGLE   = angle (degrees) of layerline slits relative to *'
      PRINT *, '*           the equatorial (horizontal) direction measured *'
      PRINT *, '*           positive in a counter-clockwise direction.     *'
      PRINT *, '*                                                          *'
      PRINT *, '* SPACE   = distance (transform pixel units) between slits.*'
      PRINT *, '*                                                          *'
      PRINT *, '* FALLOFF = sets the sensitivity of the Gaussian weighting *'
      PRINT *, '*           of transform sample points such that points at *'
      PRINT *, '*           a distance = FALLOFF from the nearest slit are *'
      PRINT *, '*           weighted by 1/exp.                             *'
      PRINT *, '************************************************************'
      PRINT *
C
 100  WRITE(*,'(A)') '$ANGLE,SPACING,FALLOFF (3F): '
      READ (*,*,ERR=100) ANGLE, SPACE, FALL

      deg_to_rad = acos(-1.0d0)/180.0d0
      COSANG = COS(ANGLE*deg_to_rad)
      TANANG = TAN(ANGLE*deg_to_rad)

      XTEST  = (IDIM1/2)
C
      DO IY = 1,(IDIM2/2)+1                  ! PROCESS FFT A ROW AT A TIME
        Y  = IY-1.0
        IX = 0
        DO I = 1,2*IDIM1,2                  ! PROCESS THE CURRENT ROW
          IF(IX.GE.XTEST) IX = IX-IDIM1
          D = (Y-IX*TANANG)*COSANG
          IF(D.LT.0.0) D = D+1000*SPACE
          FRAC = D/SPACE
          FRAC = FRAC-INT(FRAC)
          IF(FRAC.GT.0.5) FRAC = 1.0-FRAC
          WEIGHT      = EXP(-0.5*((FRAC*SPACE)/FALL)**2.0)
          FFT(I,IY)   = FFT(I,IY)  *WEIGHT
          FFT(I+1,IY) = FFT(I+1,IY)*WEIGHT
          IX          = IX+1
        ENDDO
      ENDDO
C
      RETURN
      END


C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE FFT_IMG_FILL     *****           *
C  *                                                                  *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW "IMAGE".      *
C  *                                                                  *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).        *
C  ********************************************************************
C
      SUBROUTINE FFT_IMG_FILL (FFT, IDIM1, IDIM2, IMG, NCOL, NROW,
     1 FFT_FULL, IMG_BKG, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J, EM_WRT
      real*4    IMG_BKG,  IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2)
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      IF(EM_WRT.EQ.1) THEN
        PRINT *, '*****************************'
        PRINT *, '* Load FFT into IMAGE array *'
        PRINT *, '*****************************'
      ENDIF
C
      FFT_FULL = .TRUE.
C
      DO J = 1,NROW
        DO I = 1,NCOL
          IMG(I,J) = FFT((2*I)-1,J) + IMG_BKG
        ENDDO
      ENDDO
C
      RETURN
      END

C  ***************************************************************************
C  *                                                                         *
C  *                   *****     SUBROUTINE FFT_MAP     *****                *
C  *                                                                         *
C  *    CONVERT "FFT" DATA TO "MAP" DATA                                     *
C  *                                                                         *
C  *    AMPS = .FALSE. TO CONVERT INTENSITY DATA TO MAP FORMAT               *
C  *         = .TRUE.  TO CONVERT AMPLITUDE DATA TO MAP FORMAT               *
C  *                                                                         *
C  *    JUNE 1987  PURDUE UNIVERSITY VAX 11/750, C. HO.                      *
C  *    SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                    *
C  *    LAST MODIFIED 12-AUG-1999 BY TSB (Fix XTEST,YTEST problems).          *
CTSB Need to check for XTEST,YTEST bugs
C  ***************************************************************************
C
      SUBROUTINE FFT_MAP (FFT, MAP, IDIM1, IDIM2, AMPS, FSQ_MAX, 
     1 FFT_FULL, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, SCR_STATUS, 
     2 MAP_INCORE, A_CELL, B_CELL, C_CELL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, INTAMP, IX, IY, X1, X2, Y1, Y2, IY2
      INTEGER*4 XTEST, YTEST, ICOUNT, SCR_STATUS
      REAL*4    FFT(1), FSQ_MAX, MAP(IDIM1,IDIM2), A_CELL, B_CELL, C_CELL
      REAL*4        VALUE, SCALE, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
      LOGICAL   AMPS, FFT_FULL, MAP_INCORE
C  ----------------------------------------------------------------------------
C
      IF(AMPS) THEN
         SCALE = 1000.0/SQRT(FSQ_MAX)
       ELSE
         SCALE = 1000.0/FSQ_MAX
      ENDIF
C
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      ICOUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST + 1
      ENDIF
C
      DO IY = 1,IY2
        Y1 = YTEST + IY
Ca  Possible indexing problem ???
        IF(FFT_FULL) THEN
           IF(Y1.GT.IDIM2) Y1 = Y1 - IDIM2
         ELSE
           Y2 = YTEST - IY + 2
        ENDIF
        DO IX = 1,IDIM1
          ICOUNT = ICOUNT + 2
          VALUE  = (FFT(ICOUNT)**2 + FFT(ICOUNT+1)**2)
          IF(AMPS) THEN
             VALUE = SQRT(VALUE)*SCALE
           ELSE
             VALUE = VALUE*SCALE
          ENDIF
Ca  Possible indexing problem ???
          IF(IX.LE.IDIM1+1) THEN
             X1 = XTEST + (IX+1)/2 - 1
           ELSE
             X1 = (IX-1-IDIM1)/2
          ENDIF
          MAP(X1,Y1) = VALUE
          IF(Y2.GT.1 .AND. .NOT.FFT_FULL) THEN
            X2 = -(X1-XTEST) + XTEST
            IF(X2.EQ.0) X2 = IDIM1
            MAP(X2,Y2)     = VALUE
          ENDIF
        ENDDO
      ENDDO
C
      CALL MAP_STATS (1, MAP, IDIM1, IDIM2, 1, MAP_MIN, MAP_MAX,
     1 MAP_AVG, MAP_STD, SCR_STATUS, MAP_INCORE)
C
      A_CELL = IDIM1
      B_CELL = IDIM2
      C_CELL = 1.0
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE FFT_MAP_FILL     *****            *
C  *                                                                    *
C  *    LOAD IDIM1 X IDIM2 "FFT" ARRAY INTO NCOL X NROW "MAP" SECTION.  *
C  *                                                                    *
C  *    JANUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).               *
C  *    LAST MODIFIED 2-SEP-1999 BY TSB (Rid of EM_WRT).                *
C  **********************************************************************
C
      SUBROUTINE FFT_MAP_FILL (FFT, IDIM1, IDIM2, MAP, NCOL, NROW,
     1 NSEC, SEC, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, NSEC, SEC, IDIM1, IDIM2, I, J
      REAL*4    MAP(NCOL,NROW,NSEC), FFT(2*IDIM1,IDIM2)
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      FFT_FULL = .TRUE.
C
      DO J = 1,NROW
        DO I = 1,NCOL
          MAP(I,J,SEC) = FFT((2*I)-1,J)
        ENDDO
      ENDDO
C
      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE FFT_MRCTRANSF     *****               *
C  *                                                                          *
C  *    BRING DOWN REQUIRED AREA OF ANY SQUARE TRANSFORM AND APPLY ORIGIN     *
C  *    CORRECTION.                                                           *
C  *                                                                          *
C  *    ARRAY DIMENSIONS: AB(2*IDIM), AT AND BT((IRAD+2)*2*IRAD+1)            *
C  *                                                                          *
C  *    THIS IS ADAPTED FROM "TRANSF" FROM THE ORIGINAL MRC PROGRAM "ROTFLT"  *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM.).               *
C  ****************************************************************************
C
      SUBROUTINE FFT_MRCTRANSF (LUN, AT, BT, AB, IRAD, IDIM, FFT_ORIGX,
     1 FFT_ORIGY)
C
      IMPLICIT NONE
C
      INTEGER*4 IRAD, LUN, IDIM, I, J, K, IK, OFFSET, IREC
      REAL*4    AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
      REAL*4    AB(2*IDIM), PI, RADIAN, DEGREE, SHIFT, YSHIFT, TMP
      REAL*4    X, XSHIFT_STEP, YSHIFT_STEP, COS_SHIFT, SIN_SHIFT
      REAL*4    FFT_ORIGX, FFT_ORIGY
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
C
      IREC        = 4
      OFFSET      = IDIM - 2*IRAD - 1
      XSHIFT_STEP = FFT_ORIGX*2.0*PI/FLOAT(IDIM)
      YSHIFT_STEP = FFT_ORIGY*2.0*PI/FLOAT(IDIM)
C
      IK = 1
      DO J = 1,(IRAD+2)
        YSHIFT = YSHIFT_STEP*(J-1)
        READ(LUN,REC=IREC) AB
        IREC = IREC + 1
        DO I = 1,IDIM
          TMP        = AB(I)
          AB(I)      = AB(I+IDIM)
          AB(I+IDIM) = TMP
        ENDDO
        DO I = 1,(2*IRAD+1)
          X         = FLOAT(I-IRAD-1)
          SHIFT     = XSHIFT_STEP*X + YSHIFT
          COS_SHIFT = COS(SHIFT)
          SIN_SHIFT = SIN(SHIFT)
          K         = 2*I + OFFSET
          AT(IK)    =  AB(K)*COS_SHIFT + AB(K+1)*SIN_SHIFT
          BT(IK)    = -AB(K)*SIN_SHIFT + AB(K+1)*COS_SHIFT
          IK        = IK + 1
        ENDDO
      ENDDO
C
      CLOSE (UNIT=LUN, STATUS='KEEP')
      RETURN
C
      END

C  ***********************************************************************
C  *                                                                     *
C  *                *****     SUBROUTINE FFT_MULT     *****              *
C  *                                                                     *
C  *    MULTIPLY FFT ARRAY BY MULTIPLICATION FACTOR.                     *
C  *                                                                     *
C  *    APRIL 1993  PURDUE UNIVERSITY VAXSTATION 4000-90, T. BAKER.      *
C  *    LAST MODIFIED 16-APR-1993 BY T. BAKER.                           *
C  ***********************************************************************
C
      SUBROUTINE FFT_MULT (FFT, IDIM1, IDIM2, FACTOR)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, I, J
      REAL*4    FFT(2*IDIM1,IDIM2), FACTOR
C  -----------------------------------------------------------------------------
C
C
      DO J = 1,IDIM2
        DO I = 1,2*IDIM1
          FFT(I,J) = FFT(I,J)*FACTOR
        ENDDO
      ENDDO
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                *****     SUBROUTINE FFT_OPEN     *****                 *
C  *                                                                        *
C  *    THIS ALLOWS OPENING UP A DIRECT ACCESS TYPE FFT FILE WITHOUT        *
C  *      HAVING TO KNOW THE FILE RECORD LENGTH                             *
C  *    IT MAY BE CRUDE, BUT IT WORKS AND IS CONVENIENT                     *
C  *                                                                        *
C  *    PLEASE NOTE THAT THE INTEGERS IN THIS ROUTINE ARE EXPLICITLY        *
C  *      DECLARED.  THE INTEGERS READ IN FROM THE FIRST RECORD OF THE FFT  *
C  *      FILE *MUST* STAY THIS WAY; THE OTHERS CAN CHANGE IF YOU WISH      *
C  *                                                                        *
C  *    MARCH 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                 *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                   *
C  *    LAST MODIFIED 30-NOV-1999 BY T. BAKER (Rid of FFT_IEND).            *
C  **************************************************************************
C
      SUBROUTINE FFT_OPEN (LUN, IDIM1, IDIM2, FFT_TITLE, FFT_ORIGX, 
     1 FFT_ORIGY, FSQ_AVG, FSQ_STD, FSQ_MAX, FFT_FZERO, IMG_BKG, 
     2 FILE_ID, FFT_FILENAME, EM_ERR)
C
      IMPLICIT NONE
C
      INTEGER*4 LRECL, LUN, IDIM1, IDIM2, NCOL, NROW, ISTEP, FFT_IREC
      INTEGER*4 FILENAME_LEN, COUNT, FFT_TITLE(18)
      real*4    IMG_BKG
      INTEGER*4 FILE_ID, EM_ERR
      INTEGER*4 STRING_LEN, NBKG
      REAL*4    FFT_ORIGX, FFT_ORIGY, FSQ_AVG, FSQ_STD, FSQ_MAX
      REAL*4    FFT_FZERO
      CHARACTER FFT_FILENAME*64, FILENAME*64
C  ---------------------------------------------------------------------------
C
      COUNT  = 0
      ISTEP  = 1
      EM_ERR = 0            ! SET ERROR FLAG
  100      IF(FFT_FILENAME.EQ.' ') THEN
        PRINT *, '************************************************'
        PRINT *, '*        ENTER NAME OF FFT FILE TO OPEN        *'
        PRINT *, '* "RETURN" or <CR> = RETURN to calling routine *'
        PRINT *, '* "EXIT"           = terminate program         *'
        PRINT *, '************************************************'
        WRITE(*,'(A)') '$FILENAME: '
        READ (*,'(A)',ERR=900) FFT_FILENAME
        IF(FFT_FILENAME(1:6).EQ.'RETURN') RETURN
        IF(FFT_FILENAME.EQ.' ') RETURN
        IF(FFT_FILENAME(1:4).EQ.'EXIT') CALL TSBEND
      ENDIF
C
      OPEN (UNIT=LUN, FILE=FFT_FILENAME, ERR=400, STATUS='OLD', 
     1    ACCESS='DIRECT')
      INQUIRE (FILE=FFT_FILENAME, RECL=LRECL)            ! DETERMINE REC LENGTH
      CLOSE (UNIT=LUN, STATUS='KEEP')
      OPEN (UNIT=LUN, FILE=FFT_FILENAME, ERR=400, STATUS='OLD', 
     1     ACCESS='DIRECT', RECL=LRECL, !RSS ASSOCIATEVARIABLE=FFT_IREC,
     2    ACTION='READ')
Ca      2    ACTION='READ', SHARED)
C  ************************************
C  *    HERE IF OPEN IS SUCCESSFUL    *
C  ************************************
  200      ISTEP  = 2
      READ(LUN,REC=1,ERR=300) FILE_ID
      ISTEP = 3
      IF(FILE_ID.NE.1) GO TO 300
      ISTEP = 4
      READ(LUN,REC=2,ERR=300) FFT_TITLE
      ISTEP = 5
      READ(LUN,REC=3,ERR=300) FFT_FZERO, FFT_ORIGX, FFT_ORIGY,
     1 IDIM1, IDIM2, FSQ_AVG, FSQ_STD, FSQ_MAX, NBKG
      IMG_BKG = NBKG
      NCOL    = 2*FFT_ORIGX + 1.5
      NROW    = 2*FFT_ORIGY + 1.5
        WRITE(*,'(1X,18A4)') FFT_TITLE
      WRITE(*,1000) IDIM1, IDIM2, NCOL, NROW, IMG_BKG
      WRITE(*,1010) FFT_ORIGX, FFT_ORIGY
      WRITE(*,1020) FFT_FZERO, FSQ_AVG, FSQ_STD, FSQ_MAX
      RETURN
C
C       *******************************************
C       *              ERROR FOUND                *
C       *                                         *
C  -----*    CHECK FOR OTHER NON-FFT TYPE FILE    *
C       *******************************************
C
  300      EM_ERR = 2                   ! ERROR DETECTED IN OPENING FFT FILE
      PRINT *, '*********************************'
      PRINT *, '* An ERROR has been detected in *'
      PRINT *, '*  trying to open an FFT file   *'
      PRINT *, '*********************************'
      CALL FILE_CHECK (LUN, FFT_FILENAME, FILE_ID, IMG_BKG)
      IF(FILE_ID.EQ.-999) THEN
        IF(ISTEP.EQ.1) THEN
           PRINT *
           PRINT *, 'Error detected in the file OPEN statement'
         ELSE IF(ISTEP.EQ.2) THEN
           PRINT *
           PRINT *, 'Error reading FILE_ID record'
         ELSE IF(ISTEP.EQ.3) THEN
           PRINT *
           PRINT *, 'FFT FILE_ID should be 1: FILE_ID =', FILE_ID
         ELSE IF(ISTEP.EQ.4) THEN
           PRINT *
             PRINT *, 'Error reading FFT_TITLE'
         ELSE IF(ISTEP.EQ.5) THEN
           PRINT *
           PRINT *, 'Error reading FFT_FZERO,FFT_ORIGX,FFT_ORIGY,',
     1     'IDIM1,IDIM2,FSQ_AVG,FSQ_STD,FSQ_MAX,IMG_BKG'
        ENDIF
      ENDIF
      RETURN
C  *************************************************************************
C  *    AS A FIRST RESORT SEE IF ADDING ".FFT" TO FFT_FILENAME WORKS.      *
C  *************************************************************************
  400      FILENAME = FFT_FILENAME(1:STRING_LEN(FFT_FILENAME))//'.FFT'
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=300, STATUS='OLD', 
     1    ACCESS='DIRECT')
      INQUIRE (FILE=FFT_FILENAME, RECL=LRECL)
      CLOSE (UNIT=LUN, STATUS='KEEP')
      OPEN (UNIT=LUN, FILE=FILENAME, ERR=300, STATUS='OLD', 
     1     ACCESS='DIRECT', RECL=LRECL, !RSS ASSOCIATEVARIABLE=FFT_IREC,
     2    ACTION='READ')
Ca      2    ACTION='READ', SHARED)
      FFT_FILENAME = FILENAME
      GO TO 200                                    ! TRY AGAIN
  900      COUNT = COUNT + 1
      IF(COUNT.GT.10) THEN
        PRINT *, '********************************************'
        PRINT *, '* Program terminating because of inability *'
        PRINT *, '*  to find FFT file after 10 tries         *'
        CALL TSBEND
      ENDIF
      FFT_FILENAME = ' '
      GO TO 100
C
 1000      FORMAT(I6,'  by',I5,' FFT from original',I5,'  by',I5,' image',
     1 ' with IMG_BKG of',I5)
 1010      FORMAT(' The origin is at position:',F10.4,',',F10.4)
 1020      FORMAT(' FFT_FZERO =',E12.5,'     FSQ_AVG =',E12.5,/,
     1      '   FSQ_STD =',E12.5,'     FSQ_MAX =',E12.5)
      END

C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE FFT_PSHIFT     *****           *
C  *                                                                 *
C  *    APPLY PHASE SHIFT TO FFT TO MOVE ORIGIN TO A POSITION        *
C  *     X_SHIFT,Y_SHIFT IN THE "IMAGE".                             *
C  *                                                                 *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).            *
C  *    LAST MODIFIED 12-AUG-1999 BY TSB (Fix YTEST problem).        *
C  *******************************************************************
C
      SUBROUTINE FFT_PSHIFT (FFT, IDIM1, IDIM2, X_SHIFT, Y_SHIFT,
     1 FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IX, IY, IDIM1, IDIM2, COUNT, IY2, YTEST
      REAL*4    FFT(1), SHIFT, XSHIFT, YSHIFT, DX, DY
      REAL*4    APART, BPART, PI, TWOPI, RADIAN, DEGREE
      REAL*8    DPI, DTWOPI, DRADIAN, DDEGREE, DDX, DDY, DSHIFT
      REAL*4    X_SHIFT, Y_SHIFT, SIN_SHIFT, COS_SHIFT
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
Ca      CALL PIRADDEG (PI, RADIAN, DEGREE)
      CALL PIRADDEG_D (DPI, DRADIAN, DDEGREE)
      DTWOPI = 2.0*DPI
      DDX    = X_SHIFT*DTWOPI/FLOAT(IDIM1)
      DDY    = Y_SHIFT*DTWOPI/FLOAT(IDIM2)
      DX = DDX
      DY = DDY
      YTEST = IDIM2/2
      COUNT = -1
C       
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST
      ENDIF
C
      DO IY = 1,IY2
        IF(IY.GT.YTEST) THEN
           YSHIFT = FLOAT(IY-IDIM2-1)*DY
         ELSE
           YSHIFT = FLOAT(IY-1)*DY
        ENDIF
        DO IX = 1,2*IDIM1,2
          COUNT = COUNT + 2
          IF(IX.LE.IDIM1) THEN
             XSHIFT = (FLOAT(IX-1)/2.0)*DX
           ELSE
             XSHIFT = (FLOAT(IX-1-(2*IDIM1))/2.0)*DX
          ENDIF
          SHIFT        = XSHIFT + YSHIFT
          DSHIFT = SHIFT
          SHIFT        = DMOD(DSHIFT,DTWOPI)
          SIN_SHIFT    = SIN(SHIFT)
          COS_SHIFT    = COS(SHIFT)
          APART        = FFT(COUNT)
          BPART        = FFT(COUNT+1)
          FFT(COUNT)   =  APART*COS_SHIFT + BPART*SIN_SHIFT
          FFT(COUNT+1) = -APART*SIN_SHIFT + BPART*COS_SHIFT
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE FFT_SETDIM     *****                *
C  *                                                                          *
C  *    SET UP FOURIER TRANSFORM DIMENSIONS: POWERS OF TWO BETWEEN 128 AND    *
C  *     1024.                                                                *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FFT_SETDIM (NCOL, NROW, IDIM1, IDIM2)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, I, IDIM1, IDIM2, ITMP1, ITMP2
      INTEGER*4 N1, N2, NUMCHARI4, ID1, ID2
C  ----------------------------------------------------------------------------
C
C  ********************************************
C  *    DETERMINE MINIMUM ALLOWED FFT SIZE    *
C  ********************************************
      ID1 = (IFIX((NCOL-0.5)/128.0)+1)
      ID2 = (IFIX((NROW-0.5)/128.0)+1)
      IF(ID1.EQ.3) ID1 = 4
      IF(ID2.EQ.3) ID2 = 4
      IF(ID1.GE.5 .AND. ID1.LE.7) ID1 = 8
      IF(ID2.GE.5 .AND. ID2.LE.7) ID2 = 8
      IDIM1 = ID1*128
      IDIM2 = ID2*128
      IF(IDIM1.EQ.1024 .AND. IDIM2.EQ.1024) THEN
        PRINT *, '******************************************'
        PRINT *, '* FFT size automatically set = 1024x1024 *'
        PRINT *, '******************************************'
        RETURN
      ENDIF
      IF(IDIM1.GT.1024 .OR. IDIM2.GT.1024) THEN
        PRINT *, '***********************************'
        PRINT *, '* FFT size exceeds allowed limits *'
        PRINT *, '*                                 *'
        PRINT *, '*          EXIT PROGRAM           *'
        CALL TSBEND
      ENDIF
      N1    = NUMCHARI4(IDIM1)
      N2    = NUMCHARI4(IDIM2)
  100      PRINT *, '***************************************************'
      PRINT *, '*        Enter 2D FFT size (IDIM1,IDIM2):         *'
      PRINT *, '*  Allowed dimension sizes: 128,256,512 or 1024   *'
      PRINT *, '*                                                 *'
      PRINT *, '* NOTE1: Size must equal or exceed DEFAULT size   *'
      PRINT *, '* NOTE2: If IDIM2 not specified, then IDIM2=IDIM1 *'
      PRINT *, '***************************************************'
 200  WRITE(*,'(A,I5,A,I5,A)') '$FFT SIZE (<CR>=', IDIM1,
     1     ',', IDIM2, '): '
  300      READ (*,*,ERR=200) ITMP1, ITMP2
      IF(ITMP1.LE.0 .AND. ITMP2.LE.0) RETURN
C
      IF(ITMP1.GE.128 .AND. ITMP2.LE.0) ITMP2 = ITMP1
C
      IF(ITMP1.LT.IDIM1 .OR. ITMP2.LT.IDIM2) THEN
        PRINT *, '******************************'
        PRINT *, '* FFT_SETDIM: SIZE too small *'
        CALL TSBEND
      ENDIF
C
      IF(ITMP1.GT.1024 .OR. ITMP2.GT.1024) THEN
        PRINT *, '*********************************'
        PRINT *, '* FFT_SETDIM: Size exceeds 1024 *'
        CALL TSBEND
      ENDIF
C
      IF(MOD(ITMP1,128).NE.0 .OR. MOD(ITMP2,128).NE.0) THEN
        PRINT *, '*****************************************'
        PRINT *, '* FFT_SETDIM: Size not divisible by 128 *'
        CALL TSBEND
      ENDIF
C
      IF(ITMP1.EQ.384 .OR. ITMP1.EQ.640 .OR. ITMP1.EQ.896 .OR.
     1  ITMP2.EQ.384 .OR. ITMP2.EQ.640 .OR. ITMP2.EQ.896) THEN
        PRINT *, '**************************************************'
        PRINT *, '* FFT_SETDIM: Size does not match allowed values *'
        CALL TSBEND
      ENDIF
C
      IDIM1 = ITMP1
      IDIM2 = ITMP2
      RETURN
C
      END

C  **************************************************************************
C  *                                                                        *
C  *              *****     SUBROUTINE FFT_SETDIM_DEF     *****             *
C  *                                                                        *
C  *    SET UP DEFAULT FOURIER TRANSFORM DIMENSIONS.                        *
C  *                                                                        *
C  *    THE SIZE OF THE FFT DIMENSIONS (IDIM1 BY IDIM2) WILL BE THE         *
C  *     SMALLEST POWER OF 2 ALLOWED FOR THE NCOL BY NROW DIMENSIONS,       *
C  *     MULTIPLIED BY NFAC.                                                *
C  *                                                                        *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                     *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).              *
C  **************************************************************************
C
      SUBROUTINE FFT_SETDIM_DEF (NCOL, NROW, IDIM1, IDIM2, NFAC)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, I, IDIM1, IDIM2, NFAC
      INTEGER*4 NUMCHARI4, N1, N2
      REAL*4    TEST
C  ----------------------------------------------------------------------------
C
C  ********************************************
C  *    DETERMINE MINIMUM ALLOWED FFT SIZE    *
C  ********************************************
      TEST = FLOAT(NCOL*(2**(NFAC-1)))
      I    = 0
      DO WHILE (TEST.GT.1.0)
        I    = I + 1
        TEST = TEST/2.0
      ENDDO
      IDIM1 = 2**I
C
      TEST = FLOAT(NROW*(2**(NFAC-1)))
      I    = 0
      DO WHILE (TEST.GT.1.0)
        I    = I + 1
        TEST = TEST/2.0
      ENDDO
      IDIM2 = 2**I
C
      N1 = NUMCHARI4(IDIM1) + 1
      N2 = NUMCHARI4(IDIM2) + 1
CCC      WRITE(*,'(A,I5,A,I5)') ' FFT dimensions are', IDIM1, ' by',
CCC      1 IDIM2
C
      RETURN
      END


C  *******************************************************************
C  *                                                                 *
C  *          *****       SUBROUTINE FFT_SPT_BACK     *****          *
C  *                                                                 *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).            *
C  *    LAST MODIFIED 10-MAR-1999 BY TSB (XSHIFT_STEP,YSHIFT_STEP).  *
C  *******************************************************************
C
      SUBROUTINE FFT_SPT_BACK (FFT, IDIM1, IDIM2, JDIM, NSPOT, 
     1 NBLOCK, XT, YT, IH, IK, NSMPL, GAUSS, XCEN, YCEN, WPOWER, 
     2 BSUMI, NBSMPL, BACKG, STD_BACK, NREJECT, DSTAR, X, Y, AMPTYPE,
     3 XSHIFT_STEP, YSHIFT_STEP, T_MATRIX, TILTAXIS, TILTANG)
C
      IMPLICIT NONE
C
      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, JDIM, KDIM, NBLOCK, KBOX, AMPTYPE
      INTEGER*4 I, IT1, IL, IB, J, JJ, NS, JY1, JY2, JX1, JX2
      INTEGER*4 MX, NSPOT, NSPOT1, NBACK, KK, TWOJJ, T_MATRIX(2,3)
      INTEGER*2 NSMPL(NSPOT), NBSMPL(NSPOT), NREJECT(NSPOT), IX, IY
      INTEGER*2 H, K, L, IH(NSPOT), IK(NSPOT)
      REAL*4    XT(NSPOT), YT(NSPOT), XCEN(NSPOT), YCEN(NSPOT)
      REAL*4    BSUMI(NSPOT), GAUSS(NSPOT), WPOWER(NSPOT), POW, WPOWB
      REAL*4    DSTAR(NSPOT), STD_BACK(NSPOT), BACKG(500,NSPOT), BACK
      REAL*4    GAU, PTOT, CUT, ACEN, PCEN, RADIUS, SHIFT, SUMSQ, STD
      REAL*4    X(IDIM1,JDIM/2), Y(IDIM1,JDIM/2), SF_SCALEF
      REAL*4    RSS, XR, YR, R, APART, BPART, XN, YN, R1
      REAL*4    PI, TWOPI, DEGREE, RADIAN, RASTER, RESLIM
      REAL*4    X1, X2, Y1, Y2, FFT(2*IDIM1,(IDIM2/2)+1)
      REAL*4    XSHIFT_STEP, YSHIFT_STEP, TILTAXIS, TILTANG
C  ----------------------------------------------------------------------------
C
C  ******************************
C  *    INITIALIZE VARIABLES    *
C  ******************************
      IT1   = 0
      IL    = 0
      PTOT  = 0.0
      DO I = 1,NSPOT
        NSMPL(I)    =   0
        NBSMPL(I)   =   0
          NREJECT(I)  =   0
        XCEN(I)     = 0.0
        YCEN(I)     = 0.0
        WPOWER(I)   = 0.0
        BSUMI(I)    = 0.0
        GAUSS(I)    = 0.0
        STD_BACK(I) = 0.0
      ENDDO
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      TWOPI = 2.0*PI
C  ***********************************************
C  *    READ DOWN TRANSFORM A BLOCK AT A TIME    *
C  ***********************************************
      KDIM = IDIM1/2
      KK   = 0
      DO IB = 1,NBLOCK
        DO J = 1,(JDIM/2)
          KK = KK + 1
          DO JJ = 1,KDIM
            TWOJJ        = 2*JJ
            X(JJ+KDIM,J) = FFT(TWOJJ-1,KK)
            Y(JJ+KDIM,J) = FFT(TWOJJ,KK)
             X(JJ,J)      = FFT(IDIM1+TWOJJ-1,KK)
            Y(JJ,J)      = FFT(IDIM1+TWOJJ,KK)
          ENDDO
        ENDDO
C
        DO 500 NS = 1,NSPOT
          RSS = RADIUS*CUT
          JY1 = YT(NS) - RSS*2.0
          IF(JY1.GT.IT1+(JDIM/2)) GO TO 500
          JY2 = YT(NS) + RSS*2.0
          IF(JY2.LE.IT1) GO TO 500
          IF(JY1.LT.IT1+1)        JY1 = IT1 + 1
          IF(JY2.GT.IT1+(JDIM/2)) JY2 = IT1 + (JDIM/2)
          JX1 = XT(NS) - RSS*2.0*(KBOX+1)
          JX2 = XT(NS) + RSS*2.0*(KBOX+1)
          IF(JX1.LT.1)     JX1 = 1
          IF(JX2.GT.IDIM1) JX2 = IDIM1
C
          DO IY = JY1,JY2
            YR = (YT(NS)-IY)
            DO 300 IX = JX1,JX2
              XR = (XT(NS)-IX)/(KBOX+1)
              R  = SQRT(XR*XR+YR*YR)
C               ****************************************************
C---------------*    IF POINT OUTSIDE BACKGROUND AREA GO TO 300    *
C               ****************************************************
              IF(R.GT.RSS*2.0) GO TO 300
              APART  = X(IX,IY-IT1)
              BPART  = Y(IX,IY-IT1)
              POW    = APART*APART + BPART*BPART
C               ***********************************************
C---------------*    CHECK IF POINT INSIDE BACKGROUND AREA    *
C               ***********************************************
              IF(R.GT.RSS) THEN
                BSUMI(NS)            = BSUMI(NS) + POW
                NBSMPL(NS)           = NBSMPL(NS) + 1
                BACKG(NBSMPL(NS),NS) = POW
                GO TO 300
              ENDIF
C               *********************************
C  -------------*    POINT INSIDE MASK HOLE     *
C               *********************************
              R1           = R/RADIUS
              GAU          = (EXP(-0.5*R1*R1))**2
              GAUSS(NS)    = GAUSS(NS) + GAU
              WPOWER(NS)   = WPOWER(NS) + POW*GAU
              NSMPL(NS)    = NSMPL(NS) + 1
              IF(ABS(XR*(KBOX+1)).GT.1.0) GO TO 300
              IF(ABS(YR).GT.1.0) GO TO 300
              MX       = IX-1-IDIM1/2
              SHIFT    = XSHIFT_STEP*MX + YSHIFT_STEP*(IY-1)
              SHIFT    = AMOD(SHIFT,TWOPI)
              XN       =  APART*COS(SHIFT) + BPART*SIN(SHIFT)
              YN       = -APART*SIN(SHIFT) + BPART*COS(SHIFT)
              XCEN(NS) = XCEN(NS) + XN*(1.0-ABS(XR)*
     1                   (KBOX+1))*(1.0-ABS(YR))
              YCEN(NS) = YCEN(NS) + YN*(1.0-ABS(XR)*
     1                   (KBOX+1))*(1.0-ABS(YR))
  300            CONTINUE
          ENDDO
  500        CONTINUE                              ! MAIN LOOP OVER NSPOTS
        IT1 = IT1 + (JDIM/2)
      ENDDO
C
C  **********************************************
C  *    CHECK FOR BAD BACKGROUND MEASUREMENTS   *
C  **********************************************
      DO NS = 1,NSPOT
        IF(NBSMPL(NS).NE.0) THEN
          BACK  = BSUMI(NS)/NBSMPL(NS)
          SUMSQ = 0.0
          DO I = 1,NBSMPL(NS)
            SUMSQ = SUMSQ + (BACKG(I,NS)-BACK)**2
          ENDDO
          STD_BACK(NS) = SQRT(SUMSQ/(NBSMPL(NS)-1))
          NBACK     = 0
          BSUMI(NS) = 0.0                  ! REINITIALIZE THE BACKGROUND SUM
          DO I = 1,NBSMPL(NS)
            IF((BACKG(I,NS)-BACK).LE.(2*STD_BACK(NS))) THEN
              NBACK     = NBACK+1
              BSUMI(NS) = BSUMI(NS)+BACKG(I,NS)
            ENDIF
          ENDDO
          NREJECT(NS) = NBSMPL(NS)-NBACK
          NBSMPL(NS)  = NBACK
        ENDIF
      ENDDO
C  ********************************
C  *    OUTPUT SF DATA TO FILE    *
C  ********************************
      TILTAXIS  = 0.0
      TILTANG   = 0.0
      SF_SCALEF = 1.0
      WRITE(2,1000)
      WRITE(3,'(3F10.4)') TILTAXIS, TILTANG, SF_SCALEF
      NSPOT1 = NSPOT+1
C  *****************************************************
C  *    CALCULATE AMPLITUDE AND PHASE FOR EACH SPOT    *
C  *     AND WRITE TO OUTPUT FILE                      *
C  *****************************************************
      PRINT *, '************************************************'
      PRINT *, '* Structure factors are written as AMPS/PHASES *'
      PRINT *, '************************************************'
      DO NS = 1,NSPOT
        ACEN = SQRT(XCEN(NS)**2+YCEN(NS)**2)/1000.0          ! SCALE DOWN "ACEN"
        PCEN = 0.0
        IF(ACEN.GT.0.0.AND.XCEN(NS).NE.0.0) THEN
          PCEN = ATAN2(YCEN(NS),XCEN(NS))*DEGREE
          PCEN = AMOD(PCEN,360.0)
          IF(PCEN.LT.0.0) PCEN = PCEN + 360.0
        ENDIF
        BACK = BSUMI(NS)
        IF(NBSMPL(NS).NE.0) BACK = BACK/NBSMPL(NS)
        IF(GAUSS(NS).NE.0.0) WPOWER(NS) = WPOWER(NS)/GAUSS(NS)
        WPOWB = WPOWER(NS) - BACK
        IF(WPOWB.LT.0.0) WPOWB = 0.0
        WPOWB  = SQRT(WPOWB)/1000.0            ! PUT "WPOWB" ON SMALLER SCALE
        ACEN   = ACEN/100.0                    ! PUT "ACEN" ON SMALLER SCALE
        GAU    = SQRT(GAUSS(NS))
        BACK   = SQRT(BACK)/100.0            ! PUT "BACK" ON SMALLER SCALE
        STD    = SQRT(STD_BACK(NS))/100.0
        YT(NS) = YT(NS) - 1.0
        XT(NS) = XT(NS) - 1.0 - IDIM1/2
        CALL HK_TO_HKL (IH(NS), IK(NS), H, K, L, T_MATRIX)
        IF(AMPTYPE.EQ.1) THEN
           WRITE(3,1010) H, K, L, ACEN, PCEN, BACK, DSTAR(NS)
           WRITE(2,1020) H, K, L, XT(NS), YT(NS), ACEN, PCEN,
     1      NSMPL(NS), GAU, BACK, STD, NBSMPL(NS), NREJECT(NS)
         ELSE
           WRITE(3,1010) H, K, L, WPOWB, PCEN, BACK, DSTAR(NS)
           WRITE(2,1020) H, K, L, XT(NS), YT(NS), WPOWB, PCEN,
     1      NSMPL(NS), GAU, BACK, STD, NBSMPL(NS), NREJECT(NS)
        ENDIF
      ENDDO
C
      RETURN
 1000      FORMAT(/' CENTRAL VALUE AT EACH POINT, PLUS ROOT MEAN INTENSITY',
     1 ' WEIGHTED WITH BACKGROUND',/,'  SUBTRACTED'/
     1 3X,'H',2X,'K',2X,'L',4X,'X',7X,'Y',5X,'WPOWB',2X,' PCEN',2X,
     1 'NSMPL   GAU   BACK    STD  NBACK  NREJ')
 1010      FORMAT(3I5,3F10.4,F10.5)
 1020      FORMAT(1X,3I3,1X,2F7.2,2F8.1,I5,F7.2,F7.0,F7.1,I6,I6)
      END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE FFT_SPT_BOX     *****             *
C  *                                                                    *
C  *    GENERATES LATTICE COORDINATES AND INITIALIZES VARIABLES         *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).               *
C  *    LAST MODIFIED 27-FEB-1998 BY T. BAKER (Make UNIX compatible).   *
C  **********************************************************************
C
      SUBROUTINE FFT_SPT_BOX (FFT, IDIM1, IDIM2, NSPOT, XT, YT, IH, IK,
     1 BACKGROUND)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, NSPOT, IBOX, I
      INTEGER*2 IH(NSPOT), IK(NSPOT)
      REAL*4    FFT(2*IDIM1,(IDIM2/2)+1), XT(NSPOT), YT(NSPOT)
      REAL*4    PI, RADIAN, DEGREE, TWOPI
      real*4, allocatable :: apart(:), bpart(:)
      LOGICAL   BACKGROUND
      integer   alloc_stat
C  ----------------------------------------------------------------------------

      allocate(apart(idim1), bpart(idim1), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate memory in FFT_SPT_BOX'
         call tsbend
      endif

      CALL PIRADDEG (PI, RADIAN, DEGREE)
      TWOPI = 2.0*PI
      IBOX  = 0
C
      WRITE(4,'(7X,A,I10,//)') 'SPOTS', NSPOT
C
      DO I = 1,NSPOT
        IBOX = IBOX + 1
        CALL FFT_SPT_BOXES (FFT, IDIM1, IDIM2, APART, BPART, DEGREE, 
     1  TWOPI, XT(I), YT(I), BACKGROUND, IBOX, IH(I), IK(I))
        IF(IBOX.EQ.2) IBOX = 0
CCC        IF(MOD(I+2,8).EQ.0) WRITE(4,'(''1'')')
      ENDDO
C
      IF(IBOX.NE.0) THEN
        IBOX = -2
        CALL FFT_SPT_BOXES (FFT, IDIM1, IDIM2, APART, BPART, DEGREE,
     1  TWOPI, XT(I), YT(I), BACKGROUND, IBOX, IH(I), IK(I))
      ENDIF
      
      deallocate(apart, bpart)

      CLOSE (UNIT=4, STATUS='KEEP')
      RETURN
C
      END

C  ****************************************************************
C  *                                                              *
C  *         *****     SUBROUTINE FFT_SPT_BOXES     *****         *
C  *                                                              *
C  *    GENERATE 9X9 WINDOWS OF TRANSFORM AT SELECTED POINTS.     *
C  *                                                              *
C  *    JULY 1981  BRANDEIS PDP 11/40, T. BAKER.                  *
C  *    NOVEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.    *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.         *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).          *
C  *    LAST MODIFIED 10-MAR-1999 BY TSB (XSHIFT_STEP,YSHIFT_STEP).  *
C  ****************************************************************
C
      SUBROUTINE FFT_SPT_BOXES (FFT, IDIM1, IDIM2, APART, BPART, 
     1 DEGREE, TWOPI, XT, YT, BACKGROUND, IBOX, IH, IK, XSHIFT_STEP,
     2 YSHIFT_STEP, FSQ_MAX)
C
      IMPLICIT NONE
C
      INTEGER*2 IAMP(9,9), IFASE(9,9), KK, MH(2), MK(2), IH, IK
      INTEGER*2 IX, IX1, IX2, IY, IY1, IY2, ISTOP, I, J, JJ, J2
      INTEGER*2 IXX, IYY, MX, IIX, IIY, TWOJ, NREC, ISWAP
      INTEGER*2 NSAMPL, NBSMPL, NSMPL
      INTEGER*4 IDIM1, IDIM2, KDIM, IAMPI, IBOX
      REAL*4    FFT(2*IDIM1,(IDIM2/2)+1), APART(IDIM1), BPART(IDIM1)
      REAL*4    AMP(2), PHASE(2), BACK(2), XC(2), YC(2), XT, YT
      REAL*4    DEGREE, TWOPI, FRIEDL, GAUSS, XN, YN, XX, YY, SHIFT
      REAL*4    XR, YR, R, PHA, POW, GAU, XCEN, YCEN, P
      REAL*4    XSHIFT_STEP, YSHIFT_STEP, FSQ_MAX
      LOGICAL   BACKGROUND
C  ----------------------------------------------------------------------------
C
      KDIM  = IDIM1/2
      ISTOP = IBOX
      IBOX  = IABS(IBOX)
      IF(XT.GE.0.0) IX = XT + 0.5
      IF(XT.LT.0.0) IX = XT - 0.5
      IY = YT + 0.5
      XC(IBOX) = XT - IX
      YC(IBOX) = YT - IY
      FRIEDL   = 1.0
C  ***********************************************
C  *    CHECK IF HAVE SELECTED A FRIEDEL MATE    *
C  ***********************************************
      IF(IY.GE.0) GO TO 10
      IX     = -IX
      FRIEDL = -1.0
 10   IY  = ABS(IY) + 1
      IX  = IX + KDIM + 1
      IX1 = IX - 4
      IX2 = IX + 4
      IF(IX1.LT.1)     IX1 = 1
      IF(IX2.GT.IDIM1) IX2 = IDIM1
      IY1 = IY - 4
      IY2 = IY + 4
      IF(IY1.LT.1)     IY1 = 1
      IF(IY2.GT.IDIM1) IY2 = IDIM1
C  ***********************************************
C  *    INITIALIZE AMPLITUDE AND PHASE ARRAYS    *
C  *    AND OTHER VARIABLES                      *
C  ***********************************************
      DO J = 1,9
        DO I = 1,9
          IAMP(I,J)  = 0
          IFASE(I,J) = 0
        ENDDO
      ENDDO
      AMP(IBOX)  = 0.0
      NSMPL      =   0
      GAUSS      = 0.0
      XCEN       = 0.0
      YCEN       = 0.0
      BACK(IBOX) = 0.0
      NBSMPL     =   0
      IF(ISTOP.LT.0) THEN
        MH(IBOX)    =   0
        MK(IBOX)    =   0
        XC(IBOX)    = 0.0
        YC(IBOX)    = 0.0
        PHASE(IBOX) = 0.0
        GO TO 800
      ENDIF
      NREC = IY2 - IY1 + 1
      KK   = IY1 - 1
      IF(IY1.EQ.1) IY1 = IY2 - 8
      DO 700 I = 1,NREC
        ISWAP = 1
        KK    = KK + 1
        DO J = 1,KDIM
          TWOJ         = 2*J
          APART(J+KDIM) = FFT(TWOJ-1,KK)
          BPART(J+KDIM) = FFT(TWOJ,KK)
          APART(J)      = FFT(IDIM1+TWOJ-1,KK)
          BPART(J)      = FFT(IDIM1+TWOJ,KK)
        ENDDO
        IIY = 9 - NREC + I
C  ****************************************************
C  *    FILL UP AMPLITUDE AND PHASE ARRAYS            *
C  *    AMPLITUDES ARE SCALED SO SQRT(FSQ_MAX) = 99   *
C  ****************************************************
  300        IXX = 0
        YR  = (YT + 2.0 - IIY - IY1)
        DO 400 IIX = IX1,IX2
          IXX   = IXX + 1
          MX    = IIX - 1 - KDIM
          IF(MX.GE.KDIM) MX = MX - IDIM1
          XR    = XT - MX
          R     = SQRT (XR*XR + YR*YR)
          XX    = APART(IIX)
          YY    = BPART(IIX)
          SHIFT = XSHIFT_STEP*MX + YSHIFT_STEP*(IIY+IY1-1-1)
          PHA   = ATAN2(YY,XX) - SHIFT
          PHA   = (AMOD(PHA,TWOPI)*DEGREE/10.0)*FRIEDL
          IF(PHA.LT.0.0) PHA = PHA + 36.0
          IFASE(IXX,IIY) = PHA + 0.5
          POW            = XX*XX + YY*YY
          IAMPI          = NINT( SQRT(POW/FSQ_MAX)*99.0 )
          IF(IAMPI.GT.99) IAMPI = 99
          IAMP(IXX,IIY) = IAMPI
          IF(R.LE.4.0) THEN
            IF(R.GT.2.0) THEN                        ! BACKGROUND POINT
               BACK(IBOX) = BACK(IBOX) + POW
               NBSMPL     = NBSMPL + 1
             ELSE
               GAU       = EXP(-0.5*R*R)            ! POINT INSIDE HOLE
               GAUSS     = GAUSS + GAU*GAU
               AMP(IBOX) = AMP(IBOX) + POW*GAU*GAU
               NSMPL     = NSMPL + 1
               IF(ABS(XR).LE.1.0) THEN
                IF(ABS(YR).LE.1.0) THEN
                   SHIFT = AMOD(SHIFT,TWOPI)
                   XN    = XX*COS(SHIFT) + YY*SIN(SHIFT)
                   YN    = -XX*SIN(SHIFT) + YY*COS(SHIFT)
                   XCEN  = XCEN + XN*(1.0-ABS(XR))*(1.0-ABS(YR))
                   YCEN  = YCEN + YN*(1.0-ABS(XR))*(1.0-ABS(YR))
                 ENDIF
               ENDIF
            ENDIF
          ENDIF
  400        CONTINUE
C
        IF(ISWAP.EQ.-1 .OR. I.EQ.1) GO TO 700
        IIY = IIY-2*(I-1)
        IF(IIY.GT.0) THEN
C         ***********************************************************
C  -------*    HERE IF NEED TO GENERATE THE FRIEDEL LINE OF DATA    *
C         ***********************************************************
          J2 = KDIM - 1
          DO J = 1,KDIM
            APART(KDIM+2-J) = FFT(2*J-1,KK)
            BPART(KDIM+2-J) = -FFT(2*J,KK)
          ENDDO
          APART(1) = FFT(IDIM1+1,KK)
          BPART(1) = FFT(IDIM1+2,KK)
          JJ       = 2*IDIM1 + 1
          DO J = 1,J2
            JJ              =  JJ - 2
            APART(KDIM+1+J) =  FFT(JJ,KK)
            BPART(KDIM+1+J) = -FFT(JJ+1,KK)
          ENDDO
          ISWAP = -1
          GO TO 300
        ENDIF
  700      CONTINUE
C
C  ***********************************************
C  *    FINISHED GOING THROUGH THE 9 X 9 AREA    *
C  *    NOW CALCULATE THE SPOT STATISTICS        *
C  ***********************************************
      YCEN = YCEN*FRIEDL
      IF(XCEN.EQ.0.0.AND.YCEN.EQ.0.0) XCEN = 1.0
      P = ATAN2(YCEN,XCEN)*DEGREE
      P = AMOD(P,360.0)
      IF(P.LT.0.0) P = P + 360.0
      PHASE(IBOX) = P
      IF(NBSMPL.NE.0) BACK(IBOX) = BACK(IBOX)/NBSMPL
      IF(GAUSS.NE.0.0) AMP(IBOX) = AMP(IBOX)/GAUSS
      IF(BACKGROUND)   AMP(IBOX) = AMP(IBOX) - BACK(IBOX)
      IF(AMP(IBOX).LT.0.0) AMP(IBOX) = 0.0
      AMP(IBOX)  = SQRT( AMP(IBOX))/1000.0
      BACK(IBOX) = SQRT(BACK(IBOX))/1000.0
      MH(IBOX)   = IH
      MK(IBOX)   = IK
  800      CALL FFT_SPT_BOXOUT (IAMP, IFASE, AMP, PHASE, BACK, XC, YC, 
     1 MH, MK, IBOX, FRIEDL)
C
      RETURN
      END

C  ****************************************************************
C  *                                                              *
C  *        *****     SUBROUTINE FFT_SPT_BOXOUT     *****         *
C  *                                                              *
C  *    CREATES OUTPUT IN THE TEMPORARY FILE EMSPOTS.TMP          *
C  *    FOR EACH DIFFRACTION SPOT THE FOLLOWING ARE LISTED:       *
C  *     1. SPOT INDICIES                                         *
C  *     2. 9 X 9 ARRAY OF AMPLITUDES SCALED TO FSQ_MAX           *
C  *     3. 9 X 9 ARRAY OF PHASES (0-36 = 0-360)                  *
C  *     4. INTEGRATED, BACKGROUND SUBTRACTED AMPLITUDE           *
C  *     5. PHASE MEASUREMENT CALCULTED BY INTERPOLATION FROM     *
C  *        THE FOUR STRUCTURE FACTORS SURROUNDING THE IDEAL      *
C  *        LATTICE POSITION                                      *
C  *     6. BACKGROUND MEASUREMENT                                *
C  *     7. CALCULATED SPOT CENTER RELATIVE TO THE CENTRAL POINT  *
C  *        OF THE 9 X 9 ARRAY                                    *
C  *                                                              *
C  *    JULY 1981  BRANDEIS PDP 11/40, T. BAKER.                  *
C  *    NOVEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.    *
C  *    MARCH 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.         *
C  *    LAST MODIFIED  5-MAR-1992 BY T. BAKER.                    *
C  ****************************************************************
C
      SUBROUTINE FFT_SPT_BOXOUT(IAMP, IFASE, AMP, PHASE, BACK, XC,
     1 YC, MH, MK, IBOX, FRIEDL)
C
      IMPLICIT NONE
C
      REAL*4    AMP(2), PHASE(2), BACK(2), XC(2), YC(2), FRIEDL
      INTEGER*2 IAMP(9,9), IFASE(9,9), IA(9,9,2), IP(9,9,2)
      INTEGER*2 I, J, K, MH(2), MK(2)
      INTEGER*4 IBOX
C  ----------------------------------------------------------------------------
C
      IF(FRIEDL.EQ.-1.0) THEN
         DO J = 1,9
           DO I = 1,9
             IA(I,J,IBOX) = IAMP(10-I,J)
             IP(I,J,IBOX) = IFASE(10-I,J)
           ENDDO
         ENDDO
       ELSE
         DO J = 1,9
           DO I = 1,9
             IA(I,J,IBOX) = IAMP(I,J)
             IP(I,J,IBOX) = IFASE(I,J)
           ENDDO
         ENDDO
      ENDIF
      IF(IBOX.EQ.1) RETURN
C  ****************************************
C  *    HERE IF HAVE TWO SPOTS TO LIST    *
C  ****************************************
      WRITE(4,1000) (MH(I),MK(I),I=1,2)
      WRITE(4,1010)
      WRITE(4,1020) (((IA(I,J,K),I=1,9),(IP(I,J,K),I=1,9),K=1,2),
     1 J=9,1,-1)
      WRITE(4,1030) (AMP(I),PHASE(I),I=1,2)
      WRITE(4,1040) (BACK(I),XC(I),YC(I),I=1,2)
C
      RETURN
C
 1000      FORMAT(/,24X,'SPOT',2I4,T85,'SPOT',2I4)
 1010      FORMAT(/,11X,'AMPLITUDES',20X,'PHASES',26X,'AMPLITUDES',20X,
     1 'PHASES')
 1020      FORMAT(1X,9I3,3X,9I3,5X,9I3,3X,9I3)
 1030      FORMAT(8X,'AMP =',F8.1,16X,'PHASE =',F8.1,18X,'AMP =',F8.1,
     1 16X,'PHASE =',F8.1)
 1040      FORMAT(7X,'BACK =',F8.1,3X,'CALCULATED CENTER AT',F6.2,',',F6.2,
     1 12X,'BACK =',F8.1,3X,'CALCULATED CENTER AT',F6.2,',',F6.2)
      END

C  *********************************************************************
C  *                                                                   *
C  *           *****     SUBROUTINE FFT_SPT_GRID     *****             *
C  *                                                                   *
C  *    GENERATES INDICIES AND COORDINATES FOR ALL SPOTS WITHIN THE    *
C  *     RESOLUTION LIMIT                                              *
C  *                                                                   *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).         *
C  *********************************************************************
C
      SUBROUTINE FFT_SPT_GRID (IDIM1, IDIM2, JDIM, NSPOT, NBLOCK,
     1 XT, YT, IH, IK, DSTAR, NH, NK, BACKGROUND, AX, AY, BX, BY)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 I, IDIM1, IDIM2, JDIM, NSPOT, NBLOCK, NS, KBOX
      INTEGER*4 NH, NK, NH1, HH, H_INDEX, K_INDEX
      INTEGER*2 IH(NSPOT), IK(NSPOT)
      REAL*4    CUT, RADIUS, RASTER, RESLIM, X, XX, XTEST, Y, YY, R
      REAL*4    XT(NSPOT), YT(NSPOT), DSTAR(NSPOT), YMAX, YTS, Y1, Y2
      REAL*4    XB1, XB2, YB1, YB2, X1, X2, X_BOUNDS, Y_BOUNDS
      REAL*4    AX, AY, BX, BY
      LOGICAL   BACKGROUND
C  ----------------------------------------------------------------------------
C
      IF(BACKGROUND) THEN
         X_BOUNDS = (IDIM1/2) - (2.0*CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (2.0*CUT*RADIUS)
       ELSE
         X_BOUNDS = (IDIM1/2) - (CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (CUT*RADIUS)
      ENDIF
      NS  = 0
      NH1 = (NH-1)/2
C  ******************************************
C  *    DETERMINE X AND Y CENTERS OF ALL    *
C  *    SPOTS WITHIN RESOLUTION LIMITS      *
C  ******************************************
      DO K_INDEX = 0,(NK-1)
        XB1 = K_INDEX*X2         ! XB1,YB1 REFER TO LOCATION WHERE SF DATA IS
        YB1 = K_INDEX*Y2         !  OBTAINED
        XB2 = K_INDEX*BX         ! XB2,YB2 REFER TO REFINED LATTICE FROM
        YB2 = K_INDEX*BY         !  ORIGINAL DATA
        DO HH = 1,NH
          H_INDEX = HH - NH1
          IF(K_INDEX.NE.0 .OR. H_INDEX.GT.0) THEN
            X  = XB1 + H_INDEX*X1                ! TO OBTAIN SF MEASUREMENTS
            IF(ABS(X).LE.X_BOUNDS) THEN
              XX = XB2 + H_INDEX*AX         ! USED FOR DSTAR AND RESOLUTION TEST
              IF(ABS(XX).LE.0.00001) THEN
                 XTEST = R4_MAX
               ELSE
                 XTEST = RASTER*IDIM1/ABS(XX)
              ENDIF
              IF(XTEST.GE.RESLIM) THEN
                Y  = YB1 + H_INDEX*Y1                ! TO OBTAIN SF MEASUREMENTS
                IF(Y.LE.Y_BOUNDS) THEN
                  YY = YB2 + H_INDEX*AY  ! USED FOR DSTAR AND RESOLUTION TEST
                  R  = RASTER*IDIM1/SQRT(XX*XX+YY*YY)
                  IF(RESLIM.LE.R) THEN             ! RESOLUTION TEST
                    NS = NS+1                     ! COUNT # OF SPOTS WITHIN LIMITS
                    DSTAR(NS) = 1.0/R
                    IF(Y.LT.0.0) THEN             ! CHECK IF SPOT IS FRIEDEL MATE
                       XT(NS) = -X + 1.0 + IDIM1/2
                       YT(NS) = -Y + 1.0
                       IH(NS) = -H_INDEX
                       IK(NS) = -K_INDEX
                     ELSE
                       XT(NS) = X + 1.0 + IDIM1/2
                       YT(NS) = Y + 1.0
                       IH(NS) = H_INDEX
                       IK(NS) = K_INDEX
                    ENDIF
                  ENDIF                        ! RESOLUTION TEST
                ENDIF                              ! Y BOUNDARY TEST
              ENDIF                              ! RESOLUTION TEST
            ENDIF                              ! X BOUNDARY TEST
          ENDIF
          ENDDO
      ENDDO
C
      YMAX = R4_MIN
      DO I = 1,NSPOT
        YTS = YT(I) + RADIUS*CUT*(KBOX + 1)
        IF(KBOX.EQ.1) YTS = YT(I) + (CUT/2.0)
        YMAX = AMAX1(YMAX,YTS)
      ENDDO
      NBLOCK = (YMAX + (JDIM/2) - 0.5)/(JDIM/2)
C
      RETURN
      END

C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE FFT_SPT_INFO     *****              *
C  *                                                                         *
C  *    BACKGROUND = TRUE   AMPLITUDES ARE BACKGROUND SUBTRACTED             *
C  *               = FALSE  BACKGROUNDS ARE NOT SUBTRACTED                   *
C  *    BOXES      = TRUE   DUMP 9X9 WINDOWS TO FFTBOXES.TMP FILE            *
C  *    KBOX       = 0 CIRCULAR HOLES, = 1 ELLIPTICAL HOLES                  *
C  *    RESLIM     = RESOLUTION LIMIT OF DATA                                *
C  *    AX, AY, BX, BY = X AND Y COORDINATES OF THE [1,0] AND [0,1] SPOTS    *
C  *                      FROM THE ORIGINAL LATTICE REFINEMENT               *
C  *                     THESE PARAMETERS ARE STORED IN THE SF FILE          *
C  *    X1, Y1, X2, Y2 = X AND Y COORDINATES OF THE [1,0] AND [0,1] SPOTS    *
C  *                      THAT ARE ACTUALLY USED TO EXTRACT DATA.  IF, FOR   *
C  *                      EXAMPLE, IN EMCORAVG, THE RECIPROCAL LATTICE       *
C  *                      DIMENSIONS WERE FORCED TO BE INTEGER VALUES IN     *
C  *                      ORDER TO FORCE THE TRANSFORM PEAKS TO BE SAMPLED   *
C  *                      EXACTLY, THEN X1, Y1, ETC. WILL BE WHOLE NUMBERS.  *
C  *                                                                         *
C  *    RADIUS = SIZE OF FILTER HOLE WHERE GAUSSIAN WT=0.6 AT THE EDGE.      *
C  *                                                                         *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).               *
C  ***************************************************************************
C
      SUBROUTINE FFT_SPT_INFO (IDIM1, IDIM2, BACKGROUND, AMPTYPE,
     1 BOXES, T_MATRIX, FILE_ID, EM_NDIM)
C
      IMPLICIT NONE
C
      COMMON       CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4    I, IDIM1, IDIM2, KBOX, LUN, AMPTYPE, SF_TITLE(18)
      INTEGER*4    SF_TYPE, FILE_ID, EM_NDIM, T_MATRIX(2,3)
      REAL*4       CUT, RADIUS, RASTER, RESLIM, RESTST
      REAL*4       X1, Y1, X2, Y2, VSTAR1, VSTAR2, PI, DEGREE, RADIAN
      REAL*4       ANGSTR1, ANGSTR2, DIFNG, AX, AY, BX, BY
      LOGICAL      BACKGROUND, BOXES
      CHARACTER*64 FILENAME
C  ----------------------------------------------------------------------------
C
C  ********************************************************
C  *    GET LATTICE VECTOR COORDINATES FROM FFTLAT.DAT    *
C  *     OR BY ENTERING THEM BY HAND                      *
C  ********************************************************
  100      WRITE(*,1111)
 1111      FORMAT(' GET RECIPROCAL LATTICE COORDINATES FROM FFTLAT.DAT 
     1 <CR>',/,T40,'EMCORAVG.DAT  (1)',/,T37,'OR ENTER BY HAND
     2 (2)? : ',$)
      READ (*,*,ERR=100) I
      IF(I.NE.1.AND.I.NE.2) THEN
         OPEN (UNIT=4, FILE='FFTLAT.DAT', STATUS='OLD', ERR=900,
     1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ')
Ca      1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ', SHARED)
         READ (4,*) AX, AY, BX, BY
         WRITE(*,'(4f12.3)') AX, AY, BX, BY
         X1 = AX
         Y1 = AY
         X2 = BX
         Y2 = BY
         CLOSE (UNIT=4, STATUS='KEEP')
       ELSE IF (I.EQ.1) THEN
         OPEN (UNIT=4, FILE='EMCORAVG.DAT', STATUS='OLD', ERR=925,
     1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ')
Ca      1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ', SHARED)
         READ (4,*)                             ! SKIP FIRST RECORD
         READ (4,*) AX, AY, BX, BY, X1, Y1, X2, Y2
         IF(AX.NE.X1.OR.AY.NE.Y1.OR.BX.NE.X2.OR.BY.NE.Y2) THEN
           IF(FLOAT(NINT(X1)).EQ.X1.AND.FLOAT(NINT(Y1)).EQ.Y1.AND.
     1       FLOAT(NINT(X2)).EQ.X2.AND.FLOAT(NINT(Y2)).EQ.Y2) THEN
              PRINT *, '***********************************************'
              PRINT *, '* Reciprocal lattice forced to integer values *'
              PRINT *, '***********************************************'
            ELSE
              PRINT *, '**********************************************'
              PRINT *, '* Unit cell lattice forced to integer values *'
              PRINT *, '**********************************************'
           ENDIF
         ENDIF
         WRITE(*,*) AX, AY, BX, BY
         CLOSE (UNIT=4, STATUS='KEEP')
       ELSE
         WRITE(*,'(A)') '$ENTER AX, AY, BX, BY (4F): '
         READ (*,*,ERR=950) AX, AY, BX, BY
         WRITE(*,*) AX, AY, BX, BY
         X1 = AX
         Y1 = AY
         X2 = BX
         Y2 = BY
         IF(AX.EQ.0.0.AND.AY.EQ.0.0.AND.BX.EQ.0.0.AND.BY.EQ.0.0)
     1     GO TO 950
      ENDIF
C  ****************************************************************
C  *    OPEN OUTPUT TEMPORARY FILE FFTSPT.TMP FOR STORING DATA    *
C  *     PERTAINING TO THE STRUCTURE FACTOR DATA PROCESSING       *
C  ****************************************************************
      OPEN (UNIT=2, FILE='FFTSPT.TMP', STATUS='NEW',
     1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
C  ****************************************************
C  *    OPEN OUTPUT FILE FOR STRUCTURE FACTOR DATA    *
C  ****************************************************
  150      WRITE(*,'(A)') '$OUTPUT FILENAME FOR STRUCTURE FACTORS: '
      READ (*,'(A)',ERR=150) FILENAME
      OPEN (UNIT=3, FILE=FILENAME, STATUS='NEW', ACCESS='SEQUENTIAL',
     1     FORM='FORMATTED')
      FILE_ID = 2                        ! STRUCTURE FACTOR FILE ID
      WRITE(3,*) FILE_ID
  200      WRITE(*,'(A)') '$SF FILE HEADER: '      ! GET NEW HEADER FOR SF FILE
      READ (*,'(18A4)',ERR=200) SF_TITLE
      WRITE(2,'(1X,18A4)') SF_TITLE
      WRITE(3,'(1X,18A4)') SF_TITLE
      SF_TYPE = 1                        ! AMPS AND PHASES
      EM_NDIM = 2                        ! 2D STRUCTURE FACTORS
      WRITE(3,'(2I8)') SF_TYPE, EM_NDIM
  250      WRITE(*,'(A)') '$AMPLITUDES: INTEGRATED <CR> OR PEAK(1) ?: '
      READ (*,*,ERR=250) AMPTYPE
      IF(AMPTYPE.NE.1) AMPTYPE = 0
  300      KBOX   = 0
      RADIUS = 1.0
      IF(AMPTYPE.EQ.0) THEN
         WRITE(*,'(2A)') '$CIRCULAR<CR> OR ELLIPTICAL(1) INTEGRATION',
     1   ' WINDOW?: '
         READ (*,*,ERR=300) KBOX
         IF(KBOX.NE.1) KBOX = 0
  350         WRITE(*,'(2A)') '$HORIZONTAL RADIUS OF INTEGRATION WINDOW',
     1   ' <CR>=1.0: '
         READ (*,*,ERR=350) RADIUS
         IF(RADIUS.LE.0.0) RADIUS = 1.0
         IF(KBOX.EQ.0) WRITE(2,1000) RADIUS
         IF(KBOX.EQ.1) WRITE(2,1010) RADIUS
  400         WRITE(*,1020)
         READ (*,*,ERR=400) I
         IF(I.NE.1) THEN
            BACKGROUND = .TRUE.
          ELSE
            BACKGROUND = .FALSE.
         ENDIF
       ELSE
         WRITE(2,1030)
         BACKGROUND = .TRUE.
      ENDIF
      IF(BACKGROUND) THEN
         PRINT *, '***************************************************'
         PRINT *, '* CUT*RADIUS establishes the radius within which  *'
         PRINT *, '*  data points are integrated.  Background values *'
         PRINT *, '*  are obtained within an annulus from CUT*RADIUS *'
         PRINT *, '*  to 2*CUT*RADIUS. (DEFAULT CUT = 2.0)           *'
         PRINT *, '***************************************************'
  450         WRITE(*,'(A)') '$CUT (<CR>=2.0): '
         READ (*,*,ERR=450) CUT
         IF(CUT.LE.0.0) CUT = 2.0
         WRITE(2,'(A)') ' AMPLITUDES ARE BACKGROUND CORRECTED'
       ELSE
         WRITE(2,'(A)') ' AMPLITUDES ARE NOT BACKGROUND CORRECTED'
         CUT = 2.0
      ENDIF
C  ************************************************************
C  *    IDENTIFY THE INDICIES OF THE TWO PRINCIPAL VECTORS    *
C  ************************************************************
  500      WRITE(*,'(A)') '$A* HKL INDEX <CR>=1,0,0: '
      READ (*,*,ERR=500) (T_MATRIX(1,I),I=1,3)
      IF(T_MATRIX(1,1).EQ.0.AND.T_MATRIX(1,2).EQ.0.AND.
     1 T_MATRIX(1,3).EQ.0) T_MATRIX(1,1) = 1            ! DEFAULT HKL = 1,0,0
  550      WRITE(*,'(A)') '$B* HKL INDEX <CR>=0,1,0: '
      READ (*,'(3I7)',ERR=550) (T_MATRIX(2,I),I=1,3)
      IF(T_MATRIX(2,1).EQ.0.AND.T_MATRIX(2,2).EQ.0.AND.
     1 T_MATRIX(2,3).EQ.0) T_MATRIX(2,2) = 1            ! DEFAULT HKL = 0,1,0
      WRITE(3,'(6I5)') T_MATRIX
C  ****************************************************
C  *    SPECIFY RASTER STEP SIZE OF DIGITIZED DATA    *
C  ****************************************************
  600      WRITE(*,'(A)') '$RASTER STEP SIZE (in nm): '
      READ (*,*,ERR=600) RASTER
      IF(RASTER.LE.0.0) THEN
        PRINT *, '******************************************'
        PRINT *, '* RASTER can''t be equal or less than 0.0 *'
        PRINT *, '******************************************'
        GO TO 600
      ENDIF
C  **************************************
C  *    SET RESOLUTION LIMIT OF DATA    *
C  **************************************
C      RESTST = 1.0/( ((IDIM1/2)-(CUT*RADIUS)+1.0)/(IDIM1*RASTER))
C      RESTST = (IDIM1*RASTER)/((IDIM1/2) - (CUT*RADIUS) + 1.0)
      RESTST = (IDIM1*RASTER)/(SQRT(2.0*((IDIM1/2)**2)))
  650      WRITE(*,'(A,F8.3,A)') '$RESOLUTION LIMIT OF DATA (in nm) <CR>=',
     1 RESTST, ': '
      READ (*,*,ERR=650) RESLIM
      IF(RESLIM.LE.0.0) THEN
         RESLIM = RESTST
       ELSE IF(RESLIM.LT.RESTST) THEN
         CALL BELL(2)
         PRINT *, 'Can''t use resolution finer than', RESTST
         GO TO 650
      ENDIF
  700      WRITE(*,'(A)') '$DUMP 9X9 WINDOWS TO FFTBOXES.TMP FILE? Y/<N>: '
      CALL GET_YESNO (BOXES, .FALSE.)
      IF(BOXES) THEN
        OPEN (UNIT=4, FILE='FFTBOXES.TMP', ACCESS='SEQUENTIAL',
     1  FORM='FORMATTED', STATUS='NEW')
        CALL PIRADDEG (PI, RADIAN, DEGREE)
        VSTAR1  = SQRT(AX*AX+AY*AY)
        VSTAR2  = SQRT(BX*BX+BY*BY)
        ANGSTR1 = ATAN2(AY,AX)*DEGREE
        ANGSTR2 = ATAN2(BY,BX)*DEGREE
        DIFNG = AMIN1(ABS(ANGSTR1-ANGSTR2),ABS(ANGSTR1-ANGSTR2+360.0))
        DIFNG = AMIN1(DIFNG,ABS(360.0-DIFNG))
        WRITE(4,1040) AX, AY, BX, BY, VSTAR1, VSTAR2, DIFNG
      ENDIF
C
      RETURN
C
  900      CALL BELL(2)
      PRINT *, '*********************************'
      PRINT *, '* ERROR opening FFTLAT.DAT file *'
      PRINT *, '*********************************'
      GO TO 100
  925      CALL BELL(2)
      PRINT *, '***********************************'
      PRINT *, '* ERROR opening EMCORAVG.DAT file *'
      PRINT *, '***********************************'
      GO TO 100
  950      CALL BELL(2)
      PRINT *, '****************************************************'
      PRINT *, '* ERROR entering lattice coordinates ... Try again *'
      PRINT *, '****************************************************'
      GO TO 100
C
 1000      FORMAT(6X,' AMPLITUDES INTEGRATED INSIDE CIRCULAR WINDOW',
     1 ' WITH RADIUS =',F5.1)
 1010      FORMAT(6X,' AMPLITUDES INTEGRATED INSIDE ELLIPTICAL WINDOW',
     1 ' WITH RADIUS =',F5.1)
 1020      FORMAT('$APPLY BACKGROUND CORRECTION TO AMPLITUDES? YES<CR> ',
     1 'NO(1): ')
 1030      FORMAT(6X,' AMPLITUDES INTERPOLATED FROM DATA SURROUNDING
     1 IDEAL LATTICE POSITION')
 1040      FORMAT(7X,'LATTICE PARAMETERS',//9X,'1 0',2F10.3,/9X,'0 1',
     1 2F10.3,/7X,'VSTAR1',F10.3,/7X,'VSTAR2',F10.3,
     2 /,7X,'ANGLE',F10.3)
      END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE FFT_SPT_LAT     *****                 *
C  *                                                                          *
C  *    GENERATES LATTICE COORDINATES AND DETERMINES THE NUMBER OF SPOTS      *
C  *                                                                          *
C  *    AX,AY,BX,BY ARE USED TO CALCULATE RESOLUTION VALUES AND THE LATTICE   *
C  *     PARAMETERS REFINED FROM THE ORIGINAL DATA (I.E. UNFORCED).           *
C  *                                                                          *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FFT_SPT_LAT (IDIM1, IDIM2, NSPOT, NH, NK, BACKGROUND,
     1 T_MATRIX, A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR,
     2 AX, AY, BX, BY)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, NSPOT, NH, NK, NH1, HH, N, NUMCHARI4
      INTEGER*4 I, KBOX, H_INDEX, K_INDEX, T_MATRIX(2,3)
      REAL*4    PI, RADIAN, DEGREE, RASTER, RESLIM, BX_SPOT, BY_SPOT
      REAL*4    VSTAR1, VSTAR2, VSTAR(3), ANGSTR1, ANGSTR2, DIFNG
      REAL*4    Y, R, VX(3), VY(3), CUT, RADIUS, X1, X2, Y1, Y2
      REAL*4    X, XTEST, X_BOUNDS, Y_BOUNDS
      REAL*4    A_STAR,   B_STAR,   C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    AX, AY, BX, BY
      LOGICAL   BACKGROUND
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
C
      VSTAR1 = SQRT(AX*AX+AY*AY)
      VSTAR2 = SQRT(BX*BX+BY*BY)
      DO I = 1,3
        IF(T_MATRIX(1,I).NE.0) THEN
          VSTAR(I) = VSTAR1/(RASTER*FLOAT(IDIM1))
        ENDIF
        IF(T_MATRIX(2,I).NE.0) THEN
          VSTAR(I) = VSTAR2/(RASTER*FLOAT(IDIM1))
        ENDIF
      ENDDO
      ANGSTR1  = ATAN2(AY,AX)*DEGREE
      ANGSTR2  = ATAN2(BY,BX)*DEGREE
      DIFNG = AMIN1(ABS(ANGSTR1-ANGSTR2),ABS(ANGSTR1-ANGSTR2+360.0))
      DIFNG = AMIN1(DIFNG,ABS(360.0-DIFNG))
      ALP_STAR = (T_MATRIX(1,2)+T_MATRIX(2,2)) * 
     1          (T_MATRIX(1,3)+T_MATRIX(2,3)) * DIFNG
      IF(ALP_STAR.LT.0.0) ALP_STAR = 180.0+ALP_STAR
      BET_STAR = (T_MATRIX(1,3)+T_MATRIX(2,3)) * 
     1          (T_MATRIX(1,1)+T_MATRIX(2,1)) * DIFNG
      IF(BET_STAR.LT.0.0) BET_STAR = 180.0+BET_STAR
      GAM_STAR = (T_MATRIX(1,1)+T_MATRIX(2,1)) *
     1          (T_MATRIX(1,2)+T_MATRIX(2,2)) * DIFNG
      IF(GAM_STAR.LT.0.0) GAM_STAR = 180.0+GAM_STAR
      A_STAR = VSTAR(1)
      B_STAR = VSTAR(2)
      C_STAR = VSTAR(3)
      WRITE(3,'(6F10.4)') A_STAR,   B_STAR,   C_STAR,
     1                 ALP_STAR, BET_STAR, GAM_STAR
      WRITE(2,1000) AX, AY, BX, BY, VSTAR1, VSTAR2, DIFNG
      DO I = 1,3
        IF(T_MATRIX(1,I).NE.0) THEN
          VX(I) = AX
          VY(I) = AY
        ENDIF
        IF(T_MATRIX(2,I).NE.0) THEN
          VX(I) = BX
          VY(I) = BY
        ENDIF
      ENDDO
      WRITE(3,'(6F10.4)') VX(1), VY(1), VX(2), VY(2), VX(3), VY(3)
C
      NSPOT = 0
      NK    = 1 +     (RASTER*FLOAT(IDIM1)/(RESLIM*VSTAR2))
      NH    = 1 + 2.0*(RASTER*FLOAT(IDIM1)/(RESLIM*VSTAR1))
C      PRINT *, ' NUMBER OF H AND K INDICIES =', NH, NK
      NH1   = (NH-1)/2
C  ******************************************
C  *    DETERMINE X AND Y CENTERS OF ALL    *
C  *    SPOTS WITHIN RESOLUTION LIMITS      *
C  ******************************************
      IF(BACKGROUND) THEN
         X_BOUNDS = (IDIM1/2) - (2.0*CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (2.0*CUT*RADIUS)
       ELSE
         X_BOUNDS = (IDIM1/2) - (CUT*RADIUS)
         Y_BOUNDS = (IDIM2/2) - (CUT*RADIUS)
      ENDIF
C
      DO K_INDEX = 0,(NK-1)
        BX_SPOT = K_INDEX*BX
        BY_SPOT = K_INDEX*BY
        DO HH = 1,NH
          H_INDEX = HH-NH1
          IF(K_INDEX.NE.0 .OR. H_INDEX.GT.0) THEN
            X = BX_SPOT + H_INDEX*AX
            IF(ABS(X).LE.X_BOUNDS) THEN
              IF(ABS(X).LE.0.00001) THEN
                 XTEST = R4_MAX
               ELSE
                 XTEST = RASTER*IDIM1/ABS(X)
              ENDIF
              IF(XTEST.GE.RESLIM) THEN
                Y = BY_SPOT + H_INDEX*AY
                IF(Y.LE.Y_BOUNDS) THEN
                  R = RASTER*IDIM1/SQRT(X*X+Y*Y)
C      PRINT '(A,2I4,A,2F10.3,F10.5)', ' SPOT', H_INDEX, K_INDEX, 
C      1     '   X,Y,R=', X, Y, R
                  IF(RESLIM.LE.R) THEN             ! RESOLUTION TEST
                    NSPOT = NSPOT+1             ! COUNT # OF SPOTS WITHIN LIMITS
                  ENDIF
                ENDIF
              ENDIF
            ENDIF                        ! X BOUNDARY TEST
          ENDIF
          ENDDO
      ENDDO
C
CCC      WRITE(2,'(7X,''SPOTS'',I10//)') NSPOT
      N = NUMCHARI4(NSPOT)
CCC      WRITE(*,'('' There are '',I<N>,'' spots'')') NSPOT
      RETURN
C
 1000      FORMAT(7X,'LATTICE PARAMETERS',//9X,'1 0',2F10.3,/9X,'0 1',
     1 2F10.3,/7X,'VSTAR1',F10.3,/7X,'VSTAR2',F10.3,
     2 /,7X,'ANGLE',F10.3)
      END

C  **********************************************************************
C  *                                                                    *
C  *           *****     SUBROUTINE FFT_SPT_NOBACK     *****            *
C  *                                                                    *
C  *    SAME AS FFT_SPT_BACK ROUTINE WITHOUT BACKGROUND SUBTRACTION.    *
C  *                                                                    *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).               *
C  *    LAST MODIFIED 10-MAR-1999 BY TSB (XSHIFT_STEP,YSHIFT_STEP).     *
C  **********************************************************************
C
      SUBROUTINE FFT_SPT_NOBACK (FFT, IDIM1, IDIM2, JDIM, NSPOT, 
     1 NBLOCK, XT, YT, IH, IK, NSMPL, GAUSS, XCEN, YCEN, WPOWER, 
     2 DSTAR, APARTS, BPARTS, AMPTYPE, T_MATRIX, XSHIFT_STEP, 
     3 YSHIFT_STEP, TILTAXIS, TILTANG)
C
      IMPLICIT NONE
C
      COMMON    CUT, KBOX, RADIUS, RASTER, RESLIM, X1, X2, Y1, Y2
      INTEGER*4 IDIM1, IDIM2, JDIM, KDIM, NBLOCK, KBOX, AMPTYPE
      INTEGER*4 I, IT1, IL, IB, J, JJ, NS, JY1, JY2, JX1, JX2
      INTEGER*4 T_MATRIX(2,3)
      INTEGER*4 MX, NSPOT, NSPOT1, KK, TWOJJ
      INTEGER*2 IH(NSPOT), IK(NSPOT), NSMPL(NSPOT), H, K, L, IX, IY
      REAL*4    XT(NSPOT), YT(NSPOT), XCEN(NSPOT), YCEN(NSPOT)
      REAL*4    GAUSS(NSPOT), WPOWER(NSPOT), POW, WPOWB
      REAL*4    DSTAR(NSPOT), FFT(2*IDIM1,(IDIM2/2)+1)
      REAL*4    GAU, PTOT, CUT, ACEN, PCEN, RADIUS, SHIFT, STD
      REAL*4    APARTS(IDIM1,JDIM/2), BPARTS(IDIM1,JDIM/2)
      REAL*4    RSS, XDIST, YDIST, RSQ, RTEST, APART, BPART
      REAL*4    PI, TWOPI, DEGREE, RADIAN, RASTER, RESLIM
      REAL*4    X1, X2, Y1, Y2, XN, YN, SF_SCALEF, TILTAXIS, TILTANG
      REAL*4    XSHIFT_STEP, YSHIFT_STEP
C  ----------------------------------------------------------------------------
C
C  ******************************
C  *    INITIALIZE VARIABLES    *
C  ******************************
      IT1   = 0
      IL    = 0
      PTOT  = 0.0
      DO I = 1,NSPOT
        NSMPL(I)    =   0
        XCEN(I)     = 0.0
        YCEN(I)     = 0.0
        WPOWER(I)   = 0.0
        GAUSS(I)    = 0.0
      ENDDO
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      TWOPI = 2.0*PI
C  ***********************************************
C  *    READ DOWN TRANSFORM A BLOCK AT A TIME    *
C  ***********************************************
      KDIM = IDIM1/2
      KK   = 0
      DO IB = 1,NBLOCK
        DO J = 1,(JDIM/2)
          KK = KK + 1
          DO JJ = 1,KDIM
            TWOJJ             = 2*JJ
            APARTS(JJ+KDIM,J) = FFT(TWOJJ-1,KK)
            BPARTS(JJ+KDIM,J) = FFT(TWOJJ,KK)
             APARTS(JJ,J)      = FFT(IDIM1+TWOJJ-1,KK)
            BPARTS(JJ,J)      = FFT(IDIM1+TWOJJ,KK)
          ENDDO
        ENDDO
C
        DO 500 NS = 1,NSPOT
          RSS   = RADIUS*CUT
          RTEST = RSS**2
          JY1   = YT(NS) - RSS*2.0
          IF(JY1.GT.IT1+(JDIM/2)) GO TO 500
          JY2 = YT(NS) + RSS*2.0
          IF(JY2.LE.IT1) GO TO 500
          IF(JY1.LT.IT1+1)     JY1 = IT1 + 1
          IF(JY2.GT.IT1+(JDIM/2)) JY2 = IT1 + (JDIM/2)
          JX1 = XT(NS) - RSS*2.0*(KBOX+1)
          JX2 = XT(NS) + RSS*2.0*(KBOX+1)
          IF(JX1.LT.1)     JX1 = 1
          IF(JX2.GT.IDIM1) JX2 = IDIM1
C
          DO IY = JY1,JY2
            YDIST = (YT(NS)-IY)
            DO 300 IX = JX1,JX2
              XDIST = (XT(NS)-IX)/(KBOX+1)
              RSQ   = XDIST*XDIST + YDIST*YDIST
              IF(RSQ.GT.RTEST) GO TO 300
C               *********************************
C  -------------*    POINT INSIDE MASK HOLE     *
C               *********************************
              APART        = APARTS(IX,IY-IT1)
              BPART        = BPARTS(IX,IY-IT1)
              POW          = APART*APART + BPART*BPART
              GAU          = (EXP(-0.5*RSQ/(RADIUS**2)))**2
              GAUSS(NS)    = GAUSS(NS) + GAU
              WPOWER(NS)   = WPOWER(NS) + POW*GAU
              NSMPL(NS)    = NSMPL(NS) + 1
              IF(ABS(XDIST*(KBOX+1)).GT.1.0) GO TO 300
              IF(ABS(YDIST).GT.1.0) GO TO 300
              MX       = IX-1-IDIM1/2
              SHIFT    = XSHIFT_STEP*MX + YSHIFT_STEP*(IY-1)
              SHIFT    = AMOD(SHIFT,TWOPI)
              XN       =  APART*COS(SHIFT) + BPART*SIN(SHIFT)
              YN       = -APART*SIN(SHIFT) + BPART*COS(SHIFT)
              XCEN(NS) = XCEN(NS) + XN*(1.0-ABS(XDIST)*
     1                   (KBOX+1))*(1.0-ABS(YDIST))
              YCEN(NS) = YCEN(NS) + YN*(1.0-ABS(XDIST)*
     1                   (KBOX+1))*(1.0-ABS(YDIST))
  300            CONTINUE
          ENDDO
  500        CONTINUE                              ! MAIN LOOP OVER NSPOTS
        IT1 = IT1 + (JDIM/2)
      ENDDO
C
C  ********************************
C  *    OUTPUT SF DATA TO FILE    *
C  ********************************
      TILTAXIS  = 0.0
      TILTANG   = 0.0
      SF_SCALEF = 1.0
      IF(AMPTYPE.EQ.1) THEN
         WRITE(2,1000)
       ELSE
         WRITE(2,1010)
      ENDIF
      WRITE(3,'(3F10.4)') TILTAXIS, TILTANG, SF_SCALEF
      NSPOT1 = NSPOT+1
C  *****************************************************
C  *    CALCULATE AMPLITUDE AND PHASE FOR EACH SPOT    *
C  *     AND WRITE TO OUTPUT FILE                      *
C  *****************************************************
      PRINT *, '************************************************'
      PRINT *, '* Structure factors are written as AMPS/PHASES *'
      PRINT *, '************************************************'
      DO NS = 1,NSPOT
        ACEN = SQRT(XCEN(NS)**2 + YCEN(NS)**2)/1000.0          ! SCALE DOWN "ACEN"
        PCEN = 0.0
        IF(ACEN.GT.0.0.AND.XCEN(NS).NE.0.0) THEN
          PCEN = ATAN2(YCEN(NS),XCEN(NS))*DEGREE
          PCEN = AMOD(PCEN,360.0)
          IF(PCEN.LT.0.0) PCEN = PCEN+360.0
        ENDIF
        IF(GAUSS(NS).NE.0.0) WPOWER(NS) = WPOWER(NS)/GAUSS(NS)
        WPOWB  = SQRT(WPOWER(NS))/1000.0      ! PUT "WPOWB" ON SMALLER SCALE
        GAU    = SQRT(GAUSS(NS))
        YT(NS) = YT(NS) - 1.0
        XT(NS) = XT(NS) - 1.0 - (IDIM1/2)
        CALL HK_TO_HKL (IH(NS), IK(NS), H, K, L, T_MATRIX)
        IF(AMPTYPE.EQ.1) THEN
           WRITE(3,1020) H, K, L, ACEN, PCEN, 0.0, DSTAR(NS)
           WRITE(2,1030) H,K,L,XT(NS),YT(NS),ACEN,PCEN,NSMPL(NS),GAU
         ELSE
           WRITE(3,1020) H, K, L, WPOWB, PCEN, 0.0, DSTAR(NS)
           WRITE(2,1030) H,K,L,XT(NS),YT(NS),WPOWB,PCEN,NSMPL(NS),GAU
        ENDIF
      ENDDO
C
      RETURN
 1000      FORMAT(/' INTEGRATED VALUE AT EACH POINT, PLUS ROOT MEAN',
     1 ' WEIGHTED INTENSITY WITH',/,'  BACKGROUND NOT SUBTRACTED'/
     1 3X,'H',2X,'K',2X,'L',4X,'X',7X,'Y',5X,' ACEN',2X,' PCEN',2X,
     1 'NSMPL   GAU')
 1010      FORMAT(/' CENTRAL VALUE AT EACH POINT, PLUS ROOT MEAN INTENSITY',
     1 ' WEIGHTED WITH',/,'  BACKGROUND NOT SUBTRACTED'/
     1 3X,'H',2X,'K',2X,'L',4X,'X',7X,'Y',5X,' WPOW',2X,' PCEN',2X,
     1 'NSMPL   GAU')
 1020      FORMAT(3I5,3F10.4,F10.5)
 1030      FORMAT(1X,3I3,1X,2F7.2,2F8.1,I5,F7.2)
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FFT_STATS     *****                *
C  *                                                                          *
C  *    CALCULATE THE AVERAGE, STD AND FSQ_MAX FOR "FFT" ARRAYS               *
C  *                                                                          *
C  *    JANUARY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                 *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FFT_STATS (FFT, IDIM1, IDIM2, FSQ_AVG, FSQ_STD, 
     1 FSQ_MAX, FFT_FULL)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      INTEGER*4 IDIM1, IDIM2, I, ARRAY_SIZE
      REAL*4    FFT(1), ASQ, BSQ, TOT, VAR, SUMSQ, DIV
      REAL*4    FSQ_AVG, FSQ_STD, FSQ_MAX
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
      IF(FFT_FULL) THEN
         ARRAY_SIZE = 2*IDIM1*IDIM2
       ELSE
         ARRAY_SIZE = IDIM1*(IDIM2+2)
      ENDIF
C
      WRITE(*,1000)
      SUMSQ   = 0.0
      TOT     = 0.0
      FSQ_MAX = R4_MIN
C
      DO I = 1,ARRAY_SIZE,2
        ASQ     = FFT(I)**2
        BSQ     = FFT(I+1)**2
        SUMSQ   = SUMSQ + (ASQ+BSQ)**2
        TOT     = TOT + ASQ + BSQ
        FSQ_MAX = AMAX1(FSQ_MAX,(ASQ+BSQ))
      ENDDO
C
      DIV     = FLOAT(ARRAY_SIZE)/2.0
      FSQ_AVG = TOT/DIV
      FSQ_STD = 0.0
      VAR     = (SUMSQ/DIV) - (FSQ_AVG**2)
      IF(VAR.GE.0.0) FSQ_STD = SQRT(VAR)
C
      WRITE(*,1010) FSQ_AVG, FSQ_STD, FSQ_MAX
      RETURN
C
 1000      FORMAT(' AVG,STD and MAX of FSQ: Wait for answer')
 1010      FORMAT('+AVG,STD and MAX of FSQ: ',3E12.5)
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE FFT_TO_ATBT     *****                *
C  *                                                                          *
C  *    CUTS OUT A SQUARE REGION (BOUNDED BY A RADIUS = IRAD TRANSFORM PIXEL  *
C  *     UNITS) OF ANY "POWER-2" TRANSFORM AND APPLYS AN ORIGIN CORRECTION.   *
C  *                                                                          *
C  *    ARRAY DIMENSIONS: AT AND BT((IRAD+2)*2*IRAD+1), FFT(2*IDIM1,2*IDIM2)  *
C  *                                                                          *
C  *    THIS IS ADAPTED FROM "TRANSF" FROM THE ORIGINAL MRC PROGRAM "ROTFLT". *
C  *     IT IS USED IN SEVERAL PROGRAMS SUCH AS EMROTFLT, EMICOFV*, EMICOORG, *
C  *     AND EMICOGRAD AMONG OTHERS.                                          *
C  *                                                                          *
C  *    JULY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                      *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FFT_TO_ATBT (FFT, IDIM1, IDIM2, AT, BT, IRAD, 
     1 FFT_ORIGX, FFT_ORIGY, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IRAD, IDIM1, IDIM2, I, J, K, IK, OFFSET
      REAL*4    PI, RADIAN, DEGREE, SHIFT, YSHIFT, TMP
      REAL*4    X, XSHIFT_STEP, YSHIFT_STEP, COS_SHIFT, SIN_SHIFT
      REAL*4    AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
      REAL*4    FFT(2*IDIM1,IDIM2), FFT_ORIGX, FFT_ORIGY
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
      IF(.NOT.FFT_FULL) THEN
        CALL BELL (2)
        PRINT *, '*************************************************'
        PRINT *, '* Option does not work with HALF-SIZE FFT array *'
        PRINT *, '*                                               *'
        PRINT *, '* Program terminated in subroutine FFT_TO_ATBT  *'
        CALL TSBEND
      ENDIF
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
C
      OFFSET      = IDIM1 - 2*IRAD - 1
      XSHIFT_STEP = FFT_ORIGX*2.0*PI/FLOAT(IDIM1)
      YSHIFT_STEP = FFT_ORIGY*2.0*PI/FLOAT(IDIM1)
C
      IK = 1
      DO J = 1,(IRAD+2)
        YSHIFT = YSHIFT_STEP*(J-1)
        DO I = 1,IDIM1
          TMP            = FFT(I,J)
          FFT(I,J)       = FFT(I+IDIM1,J)
          FFT(I+IDIM1,J) = TMP
        ENDDO
        DO I = 1,(2*IRAD+1)
          X         = FLOAT(I-IRAD-1)
          SHIFT     = XSHIFT_STEP*X + YSHIFT
          COS_SHIFT = COS(SHIFT)
          SIN_SHIFT = SIN(SHIFT)
          K         = 2*I + OFFSET
          AT(IK)    =  FFT(K,J)*COS_SHIFT + FFT(K+1,J)*SIN_SHIFT
          BT(IK)    = -FFT(K,J)*SIN_SHIFT + FFT(K+1,J)*COS_SHIFT
          IK        = IK + 1
        ENDDO
      ENDDO

      RETURN
C
      END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE FFT_TO_ATBT2     *****                *
C  *                                                                          *
C  *    CUTS OUT A SQUARE REGION (BOUNDED BY A RADIUS = IRAD TRANSFORM PIXEL  *
C  *     UNITS) OF ANY "POWER-2" TRANSFORM AND APPLYS AN ORIGIN CORRECTION.   *
C  *                                                                          *
C  *    ARRAY DIMENSIONS: AT AND BT((IRAD+2)*(2*IRAD+1)*(NVIEWS))             *
C  *                      FFT(2*IDIM1,2*IDIM2)                                *
C  *                                                                          *
C  *    THIS IS ADAPTED FROM FFT_TO_ATBT WHICH ONLY WORKS WITH A SINGLE       *
C  *     PARTICLE AT A TIME.  THIS ROUTINE ACCUMULATES THE ATBT ARRAYS FOR    *
C  *     NVIEWS NUMBER OF PARTICLES.                                          *
C  *                                                                          *
C  *    DECEMBER 16, 1996 PURDUE UNIVERSITY VAX 3000-300, T. BAKER.           *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  ****************************************************************************
C
      SUBROUTINE FFT_TO_ATBT2 (FFT, IDIM1, IDIM2, AT, BT, IRAD, 
     1 NVIEWS, VIEW, FFT_ORIGX, FFT_ORIGY, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IRAD, IDIM1, IDIM2, I, J, K, IK, OFFSET
      INTEGER*4 NVIEWS, VIEW
      REAL*4    PI, RADIAN, DEGREE, SHIFT, YSHIFT, TMP
      REAL*4    X, XSHIFT_STEP, YSHIFT_STEP, COS_SHIFT, SIN_SHIFT
      REAL*4    AT((2*IRAD+1)*(IRAD+2)*NVIEWS)
      REAL*4    BT((2*IRAD+1)*(IRAD+2)*NVIEWS)
      REAL*4    FFT(2*IDIM1,IDIM2), FFT_ORIGX, FFT_ORIGY
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C
      IF(.NOT.FFT_FULL) THEN
        PRINT *, '*************************************************'
        PRINT *, '* Option does not work with HALF-SIZE FFT array *'
        PRINT *, '*                                               *'
        PRINT *, '* Program terminated in subroutine FFT_TO_ATBT2 *'
        CALL TSBEND
      ENDIF
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
C
      OFFSET      = IDIM1 - 2*IRAD - 1
      XSHIFT_STEP = FFT_ORIGX*2.0*PI/FLOAT(IDIM1)
      YSHIFT_STEP = FFT_ORIGY*2.0*PI/FLOAT(IDIM1)
C
      IK = (VIEW-1)*(2*IRAD+1)*(IRAD+2) + 1
C
      DO J = 1,(IRAD+2)
        YSHIFT = YSHIFT_STEP*(J-1)
        DO I = 1,IDIM1
          TMP            = FFT(I,J)
          FFT(I,J)       = FFT(I+IDIM1,J)
          FFT(I+IDIM1,J) = TMP
        ENDDO
        DO I = 1,(2*IRAD+1)
          X         = FLOAT(I-IRAD-1)
          SHIFT     = XSHIFT_STEP*X + YSHIFT
          COS_SHIFT = COS(SHIFT)
          SIN_SHIFT = SIN(SHIFT)
          K         = 2*I + OFFSET
          AT(IK)    =  FFT(K,J)*COS_SHIFT + FFT(K+1,J)*SIN_SHIFT
          BT(IK)    = -FFT(K,J)*SIN_SHIFT + FFT(K+1,J)*COS_SHIFT
          IK        = IK + 1
        ENDDO
      ENDDO
      RETURN
C
      END

C  ***********************************************************************
C  *                                                                     *
C  *                *****     SUBROUTINE FFT_TYPE     *****              *
C  *                                                                     *
C  *    USER SPECIFIES IF FFT IS WRITTEN AS A/B PARTS OR IN LINEPRINTER  *
C  *     OUTPUT FORMAT.                                                  *
C  *                                                                     *
C  *    APRIL 1993  PURDUE UNIVERSITY VAXSTATION 4000-90, T. BAKER.      *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).           *
C  ***********************************************************************
C
      SUBROUTINE FFT_TYPE (ITYPE)
C
      IMPLICIT NONE
C
      INTEGER*4 ITYPE
C  -----------------------------------------------------------------------------
C
  100      WRITE(*,'(A)') '$NORMAL<CR> OR LINEPRINTER(1) TYPE OUTPUT?: '
      READ (*,*,ERR=100) ITYPE
      IF(ITYPE.NE.1) ITYPE = 0
C
      RETURN
      END

C    **************************************************************************
C    *                                                                        *
C    *                  *****     SUBROUTINE FOURT     *****                  *
C    *                                                                        *
C    *    THE COOLEY-TUKEY FAST FOURIER TRANSFORM IN USASI BASIC FORTRAN      *
C    *                                                                        *
C    *    TRANSFORM(K1,K2,...) = SUM(DATA(J1,J2,...)*EXP(ISIGN*2*PI*SQRT(-1)  *
C    *     *((J1-1)*(K1-1)/NN(1)+(J2-1)*(K2-1)/NN(2)+...))), SUMMED FOR ALL   *
C    *     J1, K1 BETWEEN 1 AND NN(1), J2, K2 BETWEEN 1 AND NN(2), ETC.       *
C    *     THERE IS NO LIMIT TO THE NUMBER OF SUBSCRIPTS.  DATA IS A          *
C    *     MULTIDIMENSIONAL COMPLEX ARRAY WHOSE REAL AND IMAGINARY            *
C    *     PARTS ARE ADJACENT IN STORAGE, SUCH AS FORTRAN IV PLACES THEM.     *
C    *     IF ALL IMAGINARY PARTS ARE ZERO (DATA ARE DISGUISED REAL), SET     *
C    *     IFORM TO ZERO TO CUT THE RUNNING TIME BY UP TO FORTY PERCENT.      *
C    *     OTHERWISE, IFORM = +1.  THE LENGTHS OF ALL DIMENSIONS ARE          *
C    *     STORED IN ARRAY NN, OF LENGTH NDIM.  THEY MAY BE ANY POSITIVE      *
C    *     INTEGERS, THO THE PROGRAM RUNS FASTER ON COMPOSITE INTEGERS, AND   *
C    *     ESPECIALLY FAST ON NUMBERS RICH IN FACTORS OF TWO.  ISIGN IS +1    *
C    *     OR -1.  IF A -1 TRANSFORM IS FOLLOWED BY A +1 ONE (OR A +1         *
C    *     BY A -1) THE ORIGINAL DATA REAPPEAR, MULTIPLIED BY NTOT (=NN(1)*   *
C    *     NN(2)*...).  TRANSFORM VALUES ARE ALWAYS COMPLEX, AND ARE RETURNED *
C    *     IN ARRAY DATA, REPLACING THE INPUT.  IN ADDITION, IF ALL           *
C    *     DIMENSIONS ARE NOT POWERS OF TWO, ARRAY WORK MUST BE SUPPLIED,     *
C    *     COMPLEX OF LENGTH EQUAL TO THE LARGEST NON 2**K DIMENSION.         *
C    *     OTHERWISE, REPLACE WORK BY ZERO IN THE CALLING SEQUENCE.           *
C    *     NORMAL FORTRAN DATA ORDERING IS EXPECTED, FIRST SUBSCRIPT VARYING  *
C    *     FASTEST.  ALL SUBSCRIPTS BEGIN AT ONE.                             *
C    *                                                                        *
C    *    RUNNING TIME IS MUCH SHORTER THAN THE NAIVE NTOT**2, BEING          *
C    *     GIVEN BY THE FOLLOWING FORMULA.  DECOMPOSE NTOT INTO               *
C    *     2**K2 * 3**K3 * 5**K5 * ....  LET SUM2 = 2*K2, SUMF = 3*K3 + 5*K5  *
C    *     + ... AND NF = K3 + K5 + ....  THE TIME TAKEN BY A MULTI-          *
C    *     DIMENSIONAL TRANSFORM ON THESE NTOT DATA IS T = T0 + NTOT*(T1+     *
C    *     T2*SUM2+T3*SUMF+T4*NF).  ON THE CDC 3300 (FLOATING POINT ADD TIME  *
C    *     OF SIX MICROSECONDS), T = 3000 + NTOT*(500+43*SUM2+68*SUMF+        *
C    *     320*NF) MICROSECONDS ON COMPLEX DATA.  IN ADDITION, THE            *
C    *     ACCURACY IS GREATLY IMPROVED, AS THE RMS RELATIVE ERROR IS         *
C    *     BOUNDED BY 3*2**(-B)*SUM(FACTOR(J)**1.5), WHERE B IS THE NUMBER    *
C    *     OF BITS IN THE FLOATING POINT FRACTION AND FACTOR(J) ARE THE       *
C    *     PRIME FACTORS OF NTOT.                                             *
C    *                                                                        *
C    *    PROGRAM BY NORMAN BRENNER FROM THE BASIC PROGRAM BY CHARLES         *
C    *     RADER.  RALPH ALTER SUGGESTED THE IDEA FOR THE DIGIT REVERSAL.     *
C    *     MIT LINCOLN LABORATORY, AUGUST 1967.  THIS IS THE FASTEST AND MOST *
C    *     VERSATILE VERSION OF THE FFT KNOWN TO THE AUTHOR.  SHORTER PRO-    *
C    *     GRAMS FOUR1 AND FOUR2 RESTRICT DIMENSION LENGTHS TO POWERS OF TWO. *
C    *     SEE-- IEEE AUDIO TRANSACTIONS (JUNE 1967), SPECIAL ISSUE ON FFT.   *
C    *                                                                        *
C    *    THE DISCRETE FOURIER TRANSFORM PLACES THREE RESTRICTIONS UPON THE   *
C    *     DATA.                                                              *
C    *     1.  THE NUMBER OF INPUT DATA AND THE NUMBER OF TRANSFORM VALUES    *
C    *         MUST BE THE SAME.                                              *
C    *     2.  BOTH THE INPUT DATA AND THE TRANSFORM VALUES MUST REPRESENT    *
C    *         EQUISPACED POINTS IN THEIR RESPECTIVE DOMAINS OF TIME AND      *
C    *         FREQUENCY.  CALLING THESE SPACINGS DELTAT AND DELTAF, IT       *
C    *         MUST BE TRUE THAT DELTAF=2*PI/(NN(I)*DELTAT).  OF COURSE,      *
C    *         DELTAT NEED NOT BE THE SAME FOR EVERY DIMENSION.               *
C    *     3.  CONCEPTUALLY AT LEAST, THE INPUT DATA AND THE TRANSFORM        *
C    *         OUTPUT REPRESENT SINGLE CYCLES OF PERIODIC FUNCTIONS.          *
C    *                                                                        *
C    *     EXAMPLE 1.  THREE-DIMENSIONAL FORWARD FOURIER TRANSFORM OF A       *
C    *       COMPLEX ARRAY DIMENSIONED 32 BY 25 BY 13 IN FORTRAN IV.          *
C    *       DIMENSION DATA(32,25,13),WORK(50),NN(3)                          *
C    *       COMPLEX DATA                                                     *
C    *       DATA NN/32,25,13/                                                *
C    *       DO 1 I=1,32                                                      *
C    *         DO 1 J=1,25                                                    *
C    *           DO 1 K=1,13                                                  *
C    *    1        DATA(I,J,K)=COMPLEX VALUE                                  *
C    *       CALL FOURT(DATA,NN,3,-1,1,WORK)                                  *
C    *                                                                        *
C    *     EXAMPLE 2.  ONE-DIMENSIONAL FORWARD TRANSFORM OF A REAL ARRAY OF   *
C    *       LENGTH 64 IN FORTRAN II.                                         *
C    *       DIMENSION DATA(2,64)                                             *
C    *       DO 2 I=1,64                                                      *
C    *         DATA(1,I)=REAL PART                                            *
C    *    2    DATA(2,I)=0.                                                   *
C    *       CALL FOURT(DATA,64,1,-1,0,work)                                  *
C    **************************************************************************
C
      SUBROUTINE FOURT (DATA, NN, NDIM, KSIGN, IFORM, WORK)
C
      DIMENSION DATA(1), NN(1), IFACT(32), WORK(1)  
C  ----------------------------------------------------------------------------
C
      TWOPI = 6.283185307         
      IF(NDIM-1)920,1,1         
    1      NTOT = 2
      DO 2 IDIM = 1,NDIM
      IF(NN(IDIM))920,920,2
    2      NTOT = NTOT*NN(IDIM)
C  **************************************
C  *    MAIN LOOP FOR EACH DIMENSION    *
C  **************************************
      NP1 = 2
      DO 910 IDIM = 1,NDIM
      N   = NN(IDIM)
      NP2 = NP1*N
      IF(N-1)920,900,5
C  ******************
C  *    FACTOR N    *
C  ******************
    5      M     = N
      NTWO  = NP1
      IF    = 1
      IDIV  = 2
   10      IQUOT = M/IDIV
      IREM  = M-IDIV*IQUOT
      IF(IQUOT-IDIV)50,11,11
   11      IF(IREM)20,12,20
   12      NTWO = NTWO+NTWO
      M    = IQUOT
      GO TO 10
   20      IDIV  = 3
   30      IQUOT = M/IDIV
      IREM  = M-IDIV*IQUOT
      IF(IQUOT-IDIV)60,31,31
   31      IF(IREM)40,32,40
   32      IFACT(IF) = IDIV
      IF        = IF+1
      M         = IQUOT
      GO TO 30
   40      IDIV = IDIV+2
      GO TO 30
   50      IF(IREM)60,51,60
   51      NTWO = NTWO+NTWO
      GO TO 70
   60      IFACT(IF) = M
C  ************************************************************************
C  *    SEPARATE FOUR CASES--                                             *
C  *      1. COMPLEX TRANSFORM OR REAL TRANSFORM FOR THE 4TH, 5TH,ETC.    *
C  *         DIMENSIONS.                                                  *
C  *      2. REAL TRANSFORM FOR THE 2ND OR 3RD DIMENSION.  METHOD--       *
C  *         TRANSFORM HALF THE DATA, SUPPLYING THE OTHER HALF BY CON-    *
C  *         JUGATE SYMMETRY.                                             *
C  *      3. REAL TRANSFORM FOR THE 1ST DIMENSION, N ODD.  METHOD--       *
C  *         TRANSFORM HALF THE DATA AT EACH STAGE, SUPPLYING THE OTHER   *
C  *         HALF BY CONJUGATE SYMMETRY.                                  *
C  *      4. REAL TRANSFORM FOR THE 1ST DIMENSION, N EVEN.  METHOD--      *
C  *         TRANSFORM A COMPLEX ARRAY OF LENGTH N/2 WHOSE REAL PARTS     *
C  *         ARE THE EVEN NUMBERED REAL VALUES AND WHOSE IMAGINARY PARTS  *
C  *         ARE THE ODD NUMBERED REAL VALUES.  SEPARATE AND SUPPLY       *
C  *         THE SECOND HALF BY CONJUGATE SYMMETRY.                       *
C  ************************************************************************
   70      NON2  = NP1*(NP2/NTWO)
      ICASE = 1
      IF(IDIM-4) 71,90,90
   71      IF(IFORM) 72,72,90
   72      ICASE = 2
      IF(IDIM-1) 73,73,90
   73      ICASE = 3
       IF(NTWO-NP1) 90,90,74
   74      ICASE = 4
      NTWO = NTWO/2
      N    = N/2
      NP2  = NP2/2
      NTOT = NTOT/2
      I    = 3
      DO J = 2,NTOT
        DATA(J) = DATA(I)
        I       = I+2
      ENDDO
   90      I1RNG = NP1
      IF(ICASE-2) 100,95,100
   95      I1RNG = NP0*(1+NPREV/2)
C  ***********************************************************************
C  *    SHUFFLE ON THE FACTORS OF TWO IN N.  AS THE SHUFFLING            *
C  *    CAN BE DONE BY SIMPLE INTERCHANGE, NO WORKING ARRAY IS NEEDED    *
C  ***********************************************************************
  100      IF(NTWO-NP1) 600,600,110
  110      NP2HF = NP2/2
      J     = 1
      DO 150 I2 = 1,NP2,NON2 
        IF(J-I2) 120,130,130
  120        I1MAX = I2+NON2-2
        DO I1 = I2,I1MAX,2
          DO I3 = I1,NTOT,NP2
            J3         = J+I3-I2
            TEMPR      = DATA(I3)
            TEMPI      = DATA(I3+1)
            DATA(I3)   = DATA(J3)
            DATA(I3+1) = DATA(J3+1)
            DATA(J3)   = TEMPR
            DATA(J3+1) = TEMPI
          ENDDO
        ENDDO
  130        M = NP2HF
  140        IF(J-M) 150,150,145
  145        J = J-M
        M = M/2
        IF(M-NON2) 150,140,140
  150      J = J+M
C  ****************************************************************************
C  *    MAIN LOOP FOR FACTORS OF TWO.  PERFORM FOURIER TRANSFORMS OF          *
C  *     LENGTH FOUR, WITH ONE OF LENGTH TWO IF NEEDED.  THE TWIDDLE FACTOR   *
C  *     W=EXP(ISIGN*2*PI*SQRT(-1)*M/(4*MMAX)).  CHECK FOR W=ISIGN*SQRT(-1)   *
C  *     AND REPEAT FOR W=ISIGN*SQRT(-1)*CONJUGATE(W).                        *
C  ****************************************************************************
      NON2T = NON2+NON2
      IPAR  = NTWO/NP1
  310      IF(IPAR-2) 350,330,320
  320      IPAR = IPAR/4
      GO TO 310
  330      DO I1 = 1,I1RNG,2
        DO J3 = I1,NON2,NP1
          DO K1 = J3,NTOT,NON2T
            K2         = K1+NON2
            TEMPR      = DATA(K2)
            TEMPI      = DATA(K2+1)
            DATA(K2)   = DATA(K1)-TEMPR
            DATA(K2+1) = DATA(K1+1)-TEMPI
            DATA(K1)   = DATA(K1)+TEMPR
            DATA(K1+1) = DATA(K1+1)+TEMPI
          ENDDO
        ENDDO
      ENDDO
  350      MMAX = NON2
  360      IF(MMAX-NP2HF) 370,600,600
  370      LMAX = MAX0(NON2T,MMAX/2)
      IF(MMAX-NON2) 405,405,380
  380      THETA = -TWOPI*FLOAT(NON2)/FLOAT(4*MMAX)
      IF (KSIGN) 400,390,390
  390      THETA = -THETA
  400      WR    = COS(THETA)
      WI    = SIN(THETA)
      WSTPR = -2.*WI*WI
      WSTPI =  2.*WR*WI
  405      DO 570 L = NON2,LMAX,NON2T
      M = L
      IF(MMAX-NON2) 420,420,410
  410      W2R = WR*WR-WI*WI
      W2I = 2.*WR*WI
      W3R = W2R*WR-W2I*WI
      W3I = W2R*WI+W2I*WR
  420      DO 530 I1 = 1,I1RNG,2
      DO 530 J3 = I1,NON2,NP1
      KMIN = J3+IPAR*M
      IF(MMAX-NON2) 430,430,440
  430      KMIN  = J3
  440      KDIF  = IPAR*MMAX
  450      KSTEP = 4*KDIF
      DO 520 K1 = KMIN,NTOT,KSTEP
      K2 = K1+KDIF
      K3 = K2+KDIF
      K4 = K3+KDIF
      IF(MMAX-NON2) 460,460,480
  460      U1R = DATA(K1)+DATA(K2)
      U1I = DATA(K1+1)+DATA(K2+1)
      U2R = DATA(K3)+DATA(K4)
      U2I = DATA(K3+1)+DATA(K4+1)
      U3R = DATA(K1)-DATA(K2)
      U3I = DATA(K1+1)-DATA(K2+1)
      IF (KSIGN) 470,475,475
  470      U4R = DATA(K3+1)-DATA(K4+1)
      U4I = DATA(K4)-DATA(K3)
      GO TO 510
  475      U4R = DATA(K4+1)-DATA(K3+1)
      U4I = DATA(K3)-DATA(K4)
      GO TO 510
  480      T2R = W2R*DATA(K2)-W2I*DATA(K2+1)
      T2I = W2R*DATA(K2+1)+W2I*DATA(K2)
      T3R = WR*DATA(K3)-WI*DATA(K3+1)
      T3I = WR*DATA(K3+1)+WI*DATA(K3)
      T4R = W3R*DATA(K4)-W3I*DATA(K4+1)
      T4I = W3R*DATA(K4+1)+W3I*DATA(K4)
      U1R = DATA(K1)+T2R
      U1I = DATA(K1+1)+T2I
      U2R = T3R+T4R
      U2I = T3I+T4I
      U3R = DATA(K1)-T2R
      U3I = DATA(K1+1)-T2I
      IF (KSIGN) 490,500,500
  490      U4R = T3I-T4I
      U4I = T4R-T3R
      GO TO 510
  500      U4R = T4I-T3I
      U4I = T3R-T4R
  510      DATA(K1)   = U1R+U2R
      DATA(K1+1) = U1I+U2I
      DATA(K2)   = U3R+U4R
      DATA(K2+1) = U3I+U4I
      DATA(K3)   = U1R-U2R
      DATA(K3+1) = U1I-U2I
      DATA(K4)   = U3R-U4R
  520      DATA(K4+1) = U3I-U4I
      KMIN       = 4*(KMIN-J3)+J3
      KDIF       = KSTEP
      IF(KDIF-NP2) 450,530,530
  530      CONTINUE
      M = MMAX-M
      IF (KSIGN) 540,550,550
  540      TEMPR = WR
      WR    = -WI
      WI    = -TEMPR
      GO TO 560
  550      TEMPR = WR
      WR    = WI
      WI    = TEMPR
  560      IF(M-LMAX) 565,565,410
  565      TEMPR = WR
      WR    = WR*WSTPR-WI*WSTPI+WR
  570      WI    = WI*WSTPR+TEMPR*WSTPI+WI
      IPAR  = 3-IPAR
      MMAX  = MMAX+MMAX
      GO TO 360
  600      CALL L6TO9(DATA,NN,NDIM,KSIGN,IFORM,WORK,NTWO,NPT2,
     1  NP2,NP0,KSTEP,I1MAX,IFACT,
     1  NON2,IF,NP1,ICASE,THETA,TWOPI,WSTPR,WSTPI,WR,WI,I1RNG,
     2  I1,I3,NTOT,J3,TEMPR,N,KMIN,TEMPI,I2,J,I,IDIM)
C  ***************************************
C  *    END OF LOOP ON EACH DIMENSION    *
C  ***************************************
  900      NP0   = NP1
      NP1   = NP2
  910      NPREV = N
  920      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE L6TO9     *****                   *
C  *                                                                          *
C  *    MAIN LOOP FOR FACTORS NOT EQUAL TO TWO.  APPLY THE TWIDDLE FACTOR     *
C  *     W=EXP(ISIGN*2*PI*SQRT(-1)*(J2-1)*(J1-J2)/(NP2*IFP1)), THEN           *
C  *     PERFORM A FOURIER TRANSFORM OF LENGTH IFACT(IF), MAKING USE OF       *
C  *     CONJUGATE SYMMETRIES.                                                *
C  ****************************************************************************
C
      SUBROUTINE L6TO9 (DATA, NN, NDIM, KSIGN, IFORM, WORK, NTWO,
     1  NPT2, NP2, NP0, KSTEP, I1MAX, IFACT, NON2, IF, NP1, ICASE,
     2  THETA, TWOPI, WSTPR, WSTPI, WR, WI, I1RNG, I1, I3, NTOT,
     3  J3, TEMPR, N, KMIN, TEMPI, I2, J, I, IDIM)
C
      DIMENSION DATA(1), NN(1), IFACT(32), WORK(1)
C  ---------------------------------------------------------------------------
C
  600      IF(NTWO-NP2)605,700,700      
  605      IFP1  = NON2  
      IF    = 1  
      NP1HF = NP1/2
  610      IFP2  = IFP1/IFACT(IF)  
      J1RNG = NP2  
      IF(ICASE-3)612,611,612
  611      J1RNG = (NP2+IFP1)/2   
      J2STP = NP2/IFACT(IF)  
      J1RG2 = (J2STP+IFP2)/2 
  612      J2MIN = 1+IFP2  
      IF(IFP1-NP2)615,640,640      
  615      DO 635 J2 = J2MIN,IFP1,IFP2    
        THETA = -TWOPI*FLOAT(J2-1)/FLOAT(NP2)  
        IF (KSIGN) 625,620,620
  620        THETA = -THETA  
  625        SINTH = SIN(THETA/2.)  
        WSTPR = -2.*SINTH*SINTH
        WSTPI = SIN(THETA)     
        WR    = WSTPR+1.
        WI    = WSTPI   
        J1MIN = J2+IFP1 
        DO 635 J1 = J1MIN,J1RNG,IFP1   
          I1MAX = J1+I1RNG-2     
          DO I1 = J1,I1MAX,2 
            DO I3 = I1,NTOT,NP2
            J3MAX = I3+IFP2-NP1    
              DO J3 = I3,J3MAX,NP1
                TEMPR = DATA(J3)
                DATA(J3) = DATA(J3)*WR-DATA(J3+1)*WI
                DATA(J3+1) = TEMPR*WI+DATA(J3+1)*WR
              ENDDO
            ENDDO
          ENDDO
        TEMPR = WR
        WR    = WR*WSTPR-WI*WSTPI+WR
  635      WI    = TEMPR*WSTPI+WI*WSTPR+WI
  640      THETA = -TWOPI/FLOAT(IFACT(IF))
      IF (KSIGN) 650,645,645
  645      THETA = -THETA
  650      SINTH = SIN(THETA/2.)
      WSTPR = -2.*SINTH*SINTH
      WSTPI = SIN(THETA)
      KSTEP = 2*N/IFACT(IF)
      KRANG = KSTEP*(IFACT(IF)/2)+1
      DO 698 I1 = 1,I1RNG,2
      DO 698 I3 = I1,NTOT,NP2
        DO 690 KMIN = 1,KRANG,KSTEP
        J1MAX = I3+J1RNG-IFP1
        DO 680 J1 = I3,J1MAX,IFP1
          J3MAX = J1+IFP2-NP1
          DO 680 J3 = J1,J3MAX,NP1
            J2MAX = J3+IFP1-IFP2
            K     = KMIN+(J3-J1+(J1-I3)/IFACT(IF))/NP1HF
            IF(KMIN-1)655,655,665
  655            SUMR = 0.0
            SUMI = 0.0
            DO J2 = J3,J2MAX,IFP2
              SUMR = SUMR+DATA(J2)
              SUMI = SUMI+DATA(J2+1)
            ENDDO
            WORK(K)   = SUMR
            WORK(K+1) = SUMI
            GO TO 680  
  665            KCONJ = K+2*(N-KMIN+1) 
            J2    = J2MAX   
            SUMR  = DATA(J2) 
            SUMI  = DATA(J2+1)
            OLDSR = 0.0
            OLDSI = 0.0
            J2    = J2-IFP2
  670            TEMPR = SUMR
            TEMPI = SUMI
            SUMR  = TWOWR*SUMR-OLDSR+DATA(J2)
            SUMI  = TWOWR*SUMI-OLDSI+DATA(J2+1)
            OLDSR = TEMPR
            OLDSI = TEMPI
            J2    = J2-IFP2
             IF(J2-J3)675,675,670
  675            TEMPR         = WR*SUMR-OLDSR+DATA(J2)
            TEMPI         = WI*SUMI
            WORK(K)       = TEMPR-TEMPI
            WORK(KCONJ)   = TEMPR+TEMPI
            TEMPR         = WR*SUMI-OLDSI+DATA(J2+1)
            TEMPI         = WI*SUMR
            WORK(K+1)     = TEMPR+TEMPI
            WORK(KCONJ+1) = TEMPR-TEMPI
  680        CONTINUE
        IF(KMIN-1)685,685,686
  685        WR = WSTPR+1.0
        WI = WSTPI
        GO TO 690
  686        TEMPR=WR
        WR    = WR*WSTPR-WI*WSTPI+WR
        WI    = TEMPR*WSTPI+WI*WSTPR+WI
  690      TWOWR = WR+WR
      IF(ICASE-3)692,691,692
  691      IF(IFP1-NP2)695,692,692
  692      K     = 1
      I2MAX = I3+NP2-NP1
      DO I2 = I3,I2MAX,NP1
        DATA(I2)   = WORK(K)
        DATA(I2+1) = WORK(K+1)
        K          = K+2
      ENDDO
      GO TO 698
C  *******************************************************************
C  *    COMPLETE A REAL TRANSFORM IN THE 1ST DIMENSION, N ODD, BY    *
C  *     CONJUGATE SYMMETRIES AT EACH STAGE.                         *
C  *******************************************************************
  695      J3MAX = I3+IFP2-NP1
      DO 697 J3 = I3,J3MAX,NP1
        J2MAX = J3+NP2-J2STP
        DO 697 J2 = J3,J2MAX,J2STP
          J1MAX = J2+J1RG2-IFP2
          J1CNJ = J3+J2MAX+J2STP-J2
          DO 697 J1 = J2,J1MAX,IFP2
            K          = 1+J1-I3
            DATA(J1)   = WORK(K)
            DATA(J1+1) = WORK(K+1)
            IF(J1-J2)697,697,696
  696            DATA(J1CNJ)   =  WORK(K)
            DATA(J1CNJ+1) = -WORK(K+1)
  697            J1CNJ         = J1CNJ-IFP2
  698      CONTINUE
      IF    = IF+1
      IFP1  = IFP2
      IF(IFP1-NP1)700,700,610
C  ********************************************************************
C  *    COMPLETE A REAL TRANSFORM IN THE 1ST DIMENSION, N EVEN, BY    *
C  *     CONJUGATE SYMMETRIES.                                        *
C  ********************************************************************
  700      GO TO (900,800,900,701),ICASE
  701      NHALF = N
      N     = N+N
      THETA = -TWOPI/FLOAT(N)
      IF (KSIGN) 703,702,702
  702      THETA = -THETA
  703      SINTH = SIN(THETA/2.)
      WSTPR = -2.*SINTH*SINTH
      WSTPI = SIN(THETA)
      WR    = WSTPR+1.0
      WI    = WSTPI
      IMIN  = 3
      JMIN  = 2*NHALF-1
      GO TO 725
  710      J = JMIN
      DO I = IMIN,NTOT,NP2
        SUMR      = (DATA(I)  +DATA(J))  /2.
        SUMI      = (DATA(I+1)+DATA(J+1))/2.
        DIFR      = (DATA(I)  -DATA(J))  /2.
        DIFI      = (DATA(I+1)-DATA(J+1))/2.
        TEMPR     = WR*SUMI+WI*DIFR
        TEMPI     = WI*SUMI-WR*DIFR
        DATA(I)   = SUMR+TEMPR
        DATA(I+1) = DIFI+TEMPI
        DATA(J)   = SUMR-TEMPR
        DATA(J+1) = -DIFI+TEMPI
        J         = J+NP2
      ENDDO
      IMIN  = IMIN+2
      JMIN  = JMIN-2
      TEMPR = WR
      WR    = WR*WSTPR-WI*WSTPI+WR
      WI    = TEMPR*WSTPI+WI*WSTPR+WI
  725      IF(IMIN-JMIN)710,730,740
  730      IF (KSIGN) 731,740,740
  731      DO I = IMIN,NTOT,NP2
        DATA(I+1) = -DATA(I+1)
      ENDDO
  740      NP2  = NP2+NP2
      NTOT = NTOT+NTOT
      J    = NTOT+1
      IMAX = NTOT/2+1
  745      IMIN = IMAX-2*NHALF
      I    = IMIN
      GO TO 755
  750      DATA(J)   =  DATA(I)
      DATA(J+1) = -DATA(I+1)
  755      I         = I+2
      J         = J-2
      IF(I-IMAX)750,760,760
  760      DATA(J)   = DATA(IMIN)-DATA(IMIN+1)
      DATA(J+1) = 0.0
      IF(I-J)770,780,780
  765      DATA(J)   = DATA(I)
      DATA(J+1) = DATA(I+1)
  770      I         = I-2
      J         = J-2
      IF(I-IMIN)775,775,765
  775      DATA(J)   = DATA(IMIN)+DATA(IMIN+1)
      DATA(J+1) = 0.0
      IMAX      = IMIN
      GO TO 745
  780      DATA(1) = DATA(1)+DATA(2)
      DATA(2) = 0.0
      GO TO 900
C  *******************************************************************
C  *    COMPLETE A REAL TRANSFORM FOR THE 2ND OR 3RD DIMENSION BY    *
C  *     CONJUGATE SYMMETRIES.                                       *
C  *******************************************************************
  800      IF(I1RNG-NP1)805,900,900
  805      DO I3 = 1,NTOT,NP2
        I2MAX = I3+NP2-NP1
        DO I2 = I3,I2MAX,NP1
          IMIN = I2+I1RNG
          IMAX = I2+NP1-2
          JMAX = 2*I3+NP1-IMIN
          IF(I2-I3)820,820,810
  810          JMAX = JMAX+NP2
  820          IF(IDIM-2)850,850,830
  830          J = JMAX+NP0
          DO I = IMIN,IMAX,2
            DATA(I)   =  DATA(J)
            DATA(I+1) = -DATA(J+1)
            J         = J-2
          ENDDO
  850          J = JMAX
          DO I = IMIN,IMAX,NP0
            DATA(I)   =  DATA(J)
            DATA(I+1) = -DATA(J+1)
            J         = J-NP0
          ENDDO
        ENDDO
      ENDDO
  900      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE REALFT     *****                   *
C  *                                                                          *
C  *    REAL FOURIER TRANSFORM                                                *
C  *                                                                          *
C  *    GIVEN A REAL SEQUENCE OF LENGTH 2N THIS SUBROUTINE CALCULATES THE     *
C  *     UNIQUE PART OF THE FOURIER TRANSFORM. THE FOURIER TRANSFORM HAS      *
C  *     N + 1 UNIQUE REAL PARTS AND N - 1 UNIQUE IMIGINARY PARTS. SINCE      *
C  *     THE REAL PART AT X(N) IS FREQUENTLY OF INTEREST. THIS SUBROUTINE     *
C  *     STORES IT AT X(N) RATHER THAN IN Y(0). THERFORE X AND Y MUST BE      *
C  *     OF LENGTH N + 1 INSTEAD OF N. NOTE THAT THIS STORAGE ARRANGEMENT     *
C  *     IS DIFFERENT FROM THAT EMPLOYED BY THE HERMITIAN FOURIER             *
C  *     TRANSFORM SUBROUTINE                                                 *
C  *                                                                          *
C  *    FOR CONVENIENCE THE DATA IS PRESENTED IN TWO PARTS. THE FIRST         *
C  *     CONTAINING THE EVEN NUMBERED REAL TERMS AND THE SECOND CONTAINING    *
C  *     THE ODD NUMBERED TERMS (NUMBERING STARTING AT 0). ON RETURN THE      *
C  *     REAL PART OF THE TRANSFORM REPLACES THE EVEN TERMS AND THE           *
C  *     IMAGINARY PART OF THE TRANSFORM REPLACES THE ODD TERMS               *
C  ****************************************************************************
C
      SUBROUTINE REALFT (EVEN, ODD, N, DIM)
C
      REAL*4  EVEN(1), ODD(1)
      REAL*4  A, B, C, D, E, F, ANGLE, CO, ST, TWOPI, TWON
      INTEGER N, DIM(5), NT, D2, D3, D4, D5
      INTEGER I, J, K, L, NOVER2, I0, I1, I2
C  ----------------------------------------------------------------------------
C
      TWOPI = 6.283185
      TWON  = FLOAT(2*N)
C
      CALL CMPLFT (EVEN, ODD, N, DIM)
      NT     = DIM(1)
      D2     = DIM(2)
      D3     = DIM(3)
      D4     = DIM(4)
      D5     = DIM(5)
      NOVER2 = N/2+1
C
      IF (NOVER2.LT.2) GO TO 104
      DO 103 I = 2,NOVER2
        ANGLE = TWOPI*FLOAT(I-1)/TWON
        CO    = COS(ANGLE)
        SI    = SIN(ANGLE)
        I0    = (I-1)*D2+1
        J     = (N+2-2*I)*D2
        DO 102 I1 = I0,NT,D3
          I2 = I1+D4
          DO 101 K = I1,I2,D5
            L       = K+J
            A       = (EVEN(L)+EVEN(K))/2.0
            C       = (EVEN(L)-EVEN(K))/2.0
            B       = (ODD(L)+ODD(K))/2.0
            D       = (ODD(L)-ODD(K))/2.0
            E       = C*SI+B*CO
            F       = C*CO-B*SI
            EVEN(K) = A+E
            EVEN(L) = A-E
            ODD(K)  = F-D
            ODD(L)  = F+D
  101          CONTINUE
  102        CONTINUE
  103      CONTINUE
C
  104      CONTINUE
      IF (N.LT.1) RETURN
      J = N*D2
      DO 105 I1 = 1,NT,D3
        I2 = I1+D4
        DO 105 K = I1,I2,D5
          L       = K+J
          EVEN(L) = EVEN(K)-ODD(K)
          ODD(L)  = 0.0
          EVEN(K) = EVEN(K)+ODD(K)
          ODD(K)  = 0.0
  105      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE CMPLFT     *****                   *
C  *                                                                          *
C  *    COMPLEX FINITE DISCRETE FOURIER TRANSFORM                             *
C  *     TRANSFORMS ONE DIMENSION OF MULTI-DIMENSIONAL DATA                   *
C  *     MODIFIED BY L. F. TEN EYCK FROM A ONE-DIMENSIONAL VERSION WRITTEN    *
C  *     BY G. T. SANDE, 1969.                                                *
C  *                                                                          *
C  *    THIS PROGRAM CALCULATES THE TRANSFORM                                 *
C  *            (X(T) + I*Y(T))*(COS(2*PI*T/N) - I*SIN(2*PI*T/N))             *
C  *                                                                          *
C  *    INDEXING -- THE ARRANGEMENT OF THE MULTI-DIMENSIONAL DATA IS          *
C  *     SPECIFIED BY THE INTEGER ARRAY D, THE VALUES OF WHICH ARE USED AS    *
C  *     CONTROL PARAMETERS IN DO LOOPS.  WHEN IT IS DESIRED TO COVER ALL     *
C  *     ELEMENTS OF THE DATA FOR WHICH THE SUBSCRIPT BEING TRANSFORMED HAS   *
C  *     THE VALUE I0, THE FOLLOWING IS USED.                                 *
C  *                                                                          *
C  *             I1 = (I0 - 1)*D(2) + 1                                       *
C  *             DO 100 I2 = I1, D(1), D(3)                                   *
C  *             I3 = I2 + D(4) - 1                                           *
C  *             DO 100 I = I2, I3, D(5)                                      *
C  *                .                                                         *
C  *                .                                                         *
C  *         100 CONTINUE                                                     *
C  *                                                                          *
C  *    WITH THIS INDEXING IT IS POSSIBLE TO USE A NUMBER OF ARRANGEMENTS     *
C  *     OF THE DATA, INCLUDING NORMAL FORTRAN COMPLEX NUMBERS (D(5) = 2)     *
C  *     OR SEPARATE STORAGE OF REAL AND IMAGINARY PARTS.                     *
C  *                                                                          *
C  *    P MAX IS THE LARGEST PRIME FACTOR THAT WILL BE TOLERATED BY THIS      *
C  *     PROGRAM.                                                             *
C  *    TWO GRP IS THE LARGEST POWER OF TWO THAT IS TREATED AS A SPECIAL      *
C  *     CASE.                                                                *
C  ****************************************************************************
C
      SUBROUTINE CMPLFT (X, Y, N, D)
C
      INTEGER N, D(5)
      REAL*4  X(1), Y(1)
      LOGICAL ERROR
      INTEGER PMAX, PSYM, TWOGRP, FACTOR(15), SYM(15), UNSYM(15)
C  ----------------------------------------------------------------------------
C
      PMAX   = 19
      TWOGRP =  8
C
      IF (N.LE.1) GO TO 101
      CALL SRFP (N, PMAX, TWOGRP, FACTOR, SYM, PSYM, UNSYM, ERROR)
      IF (ERROR) GO TO 102
C
      CALL MDFTKD (N, FACTOR, D, X, Y)
      CALL DIPRP (N, SYM, PSYM, UNSYM, D, X, Y)
C
  101      CONTINUE
      RETURN
C
  102      CONTINUE
      WRITE(*,103) N
      STOP
C
  103      FORMAT (43H0INVALID NUMBER OF POINTS FOR CMPL FT.  N =,I10//)
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE SRFP     *****                   *
C  *                                                                          *
C  *    SYMMETRIZED REORDERING FACTORING SUBROUTINE                           *
C  ****************************************************************************
C
      SUBROUTINE SRFP (PTS,PMAX,TWOGRP,FACTOR,SYM,PSYM,UNSYM,ERROR)
C
      LOGICAL ERROR
      INTEGER PTS, PMAX, TWOGRP, PSYM, FACTOR(10), SYM(10), UNSYM(10)
      INTEGER PP(14), QQ(7), F, J, JJ, N, NEST, P, PTWO, Q, R
C  ----------------------------------------------------------------------------
C
      NEST = 14
C
      N    = PTS
      PSYM = 1
      F    = 2
      P    = 0
      Q    = 0
  101      CONTINUE
      IF (N.LE.1) GO TO 105
      DO 102 J = F,PMAX
        IF (N.EQ.(N/J)*J) GO TO 103
  102      CONTINUE
      GO TO 114
  103      CONTINUE
      IF (2*P+Q.GE.NEST) GO TO 115
      F = J
      N = N/F
      IF (N.EQ.(N/F)*F) GO TO 104
      Q     = Q+1
      QQ(Q) = F
      GO TO 101
  104      CONTINUE
      N     = N/F
      P     = P+1
      PP(P) = F
      PSYM  = PSYM*F
      GO TO 101
C
  105      CONTINUE
      R = 1
      IF (Q.EQ.0) R = 0
      IF (P.LT.1) GO TO 107
      DO 106 J = 1,P
        JJ         = P+1-J
        SYM(J)     = PP(JJ)
        FACTOR(J)  = PP(JJ)
        JJ         = P+Q+J
        FACTOR(JJ) = PP(J)
        JJ         = P+R+J
        SYM(JJ)    = PP(J)
  106      CONTINUE
  107      CONTINUE
      IF (Q.LT.1) GO TO 109
      DO 108 J = 1,Q
        JJ         = P+J
        UNSYM(J)   = QQ(J)
        FACTOR(JJ) = QQ(J)
  108      CONTINUE
      SYM(P+1) = PTS/PSYM**2
  109      CONTINUE
      JJ           = 2*P+Q
      FACTOR(JJ+1) = 0
      PTWO         = 1
      J            = 0
  110      CONTINUE
      J = J+1
      IF (FACTOR(J).EQ.0) GO TO 112
      IF (FACTOR(J).NE.2) GO TO 110
      PTWO      = PTWO*2
      FACTOR(J) = 1
      IF (PTWO.GE.TWOGRP) GO TO 111
      IF (FACTOR(J+1).EQ.2) GO TO 110
  111      CONTINUE
      FACTOR(J) = PTWO
      PTWO      = 1
      GO TO 110
  112      CONTINUE
      IF (P.EQ.0) R = 0
      JJ        = 2*P+R
      SYM(JJ+1) = 0
      IF (Q.LE.1) Q = 0
      UNSYM(Q+1) = 0
      ERROR      = .FALSE.
C
  113      CONTINUE
      RETURN
C
  114      CONTINUE
      WRITE(*,116) PMAX,PTS
      ERROR = .TRUE.
      GO TO 113
C
  115      CONTINUE
      WRITE(*,117) NEST,PTS
      ERROR = .TRUE.
      GO TO 113
C
  116      FORMAT (24H0LARGEST FACTOR EXCEEDS ,I3,7H.  N = ,I6,1H.)
  117      FORMAT (22H0FACTOR COUNT EXCEEDS ,I3,7H.  N = ,I6,1H.)
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE MDFTKD     *****                   *
C  *                                                                          *
C  *    MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL DRIVER             *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X(10),Y(10) array        *
C  *       declarations changed to X(1),Y(1); D. BELNAP/T. BAKER; 29-DEC-1993   *
C  ****************************************************************************
C
      SUBROUTINE MDFTKD (N,FACTOR,DIM,X,Y)
C
      INTEGER N, FACTOR(10), DIM(5), F, M, P, R, S
C      REAL X(10), Y(10)
      REAL X(1), Y(1)
C  ----------------------------------------------------------------------------
C
      S=DIM(2)
      F=0
      M=N
  101      CONTINUE
      F=F+1
      P=FACTOR(F)
      IF (P.EQ.0) RETURN
      M=M/P
      R=M*S
      IF (P.GT.8) GO TO 107
      GO TO (101,102,103,104,105,108,107,106), P
C
  102      CONTINUE
      CALL R2CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),DIM)
      GO TO 101
C
  103      CONTINUE
      CALL R3CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),DIM)
      GO TO 101
C
  104      CONTINUE
      CALL R4CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),
     1  X(3*R+1),Y(3*R+1),DIM)
      GO TO 101
C
  105      CONTINUE
      CALL R5CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),
     1  X(3*R+1),Y(3*R+1),X(4*R+1),Y(4*R+1),DIM)
      GO TO 101
C
  106      CONTINUE
      CALL R8CFTK (N,M,X(1),Y(1),X(R+1),Y(R+1),X(2*R+1),Y(2*R+1),
     1    X(3*R+1),Y(3*R+1),X(4*R+1),Y(4*R+1),X(5*R+1),Y(5*R+1),
     2    X(6*R+1),Y(6*R+1),X(7*R+1),Y(7*R+1),DIM)
      GO TO 101
C
  107      CONTINUE
      CALL RPCFTK (N, M, P, R, X, Y, DIM)
      GO TO 101
C
  108      CONTINUE
      WRITE(*,109)
      RETURN
C
  109      FORMAT (34H0TRANSFER ERROR DETECTED IN MDFTKD,//)
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R2CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 2 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R2CFTK (N, M, X0, Y0, X1, Y1, DIM)
C
      INTEGER N, M, DIM(5)
C      REAL*4  X0(10), Y0(10), X1(10), Y1(10)
      REAL*4  X0(1), Y0(1), X1(1), Y1(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M2, MOVER2
      INTEGER K1, K2, KK, L, L1, MM2, NT, SIZE, SEP, NS
      REAL*4  ANGLE, C, IS, IU, RS, RU, S, TWOPI, FJM1, FM2
      DATA TWOPI/6.283185/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M2     = M*2
      FM2    = FLOAT(M2)
      MOVER2 = M/2+1
      MM2    = SEP*M2
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM2
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C = COS(ANGLE)
        S = SIN(ANGLE)
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        C    = -C
  102     CONTINUE
C
        DO 107 KK = K0,NS,MM2
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              RS    = X0(K)+X1(K)
              IS    = Y0(K)+Y1(K)
              RU    = X0(K)-X1(K)
              IU    = Y0(K)-Y1(K)
              X0(K) = RS
              Y0(K) = IS
              IF (ZERO) GO TO 103
              X1(K) = RU*C+IU*S
              Y1(K) = IU*C-RU*S
              GO TO 104
  103              CONTINUE
              X1(K) = RU
              Y1(K) = IU
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R3CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 3 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R3CFTK (N, M, X0, Y0, X1, Y1, X2, Y2, DIM)
C
      INTEGER N, M, DIM(5)
C      REAL*4  X0(10), Y0(10), X1(10), Y1(10), X2(10), Y2(10)
      REAL*4  X0(1), Y0(1), X1(1), Y1(1), X2(1), Y2(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M3, MOVER2
      INTEGER K1, K2, KK, L, L1, MM3, NT, SIZE, SEP, NS
      REAL*4  ANGLE, A, B, C1, C2, S1, S2, T, TWOPI, I0, I1, I2
      REAL*4  IA, IB, IS, R0, R1, R2, RA, RB, RS, FJM1, FM3
      DATA TWOPI/6.283185/,A/-0.5/,B/0.86602540/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M3     = M*3
      FM3    = FLOAT(M3)
      MM3    = SEP*M3
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM3
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = C1*A+S1*B
        S1   = C1*B-S1*A
        C1   = T
        T    = C2*A-S2*B
        S2   = -C2*B-S2*A
        C2   = T
  102        CONTINUE
C
        DO 107 KK = K0,NS,MM3
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              R0    = X0(K)
              I0    = Y0(K)
              RS    = X1(K)+X2(K)
              IS    = Y1(K)+Y2(K)
              X0(K) = R0+RS
              Y0(K) = I0+IS
              RA    = R0+RS*A
              IA    = I0+IS*A
              RB    = (X1(K)-X2(K))*B
              IB    = (Y1(K)-Y2(K))*B
              IF (ZERO) GO TO 103
              R1    = RA+IB
              I1    = IA-RB
              R2    = RA-IB
              I2    = IA+RB
              X1(K) = R1*C1+I1*S1
              Y1(K) = I1*C1-R1*S1
              X2(K) = R2*C2+I2*S2
              Y2(K) = I2*C2-R2*S2
              GO TO 104
  103              CONTINUE
              X1(K) = RA+IB
              Y1(K) = IA-RB
              X2(K) = RA-IB
              Y2(K) = IA+RB
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R4CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 4 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R4CFTK (N, M, X0, Y0, X1, Y1, X2, Y2, X3, Y3, DIM)
C
      INTEGER N, M, DIM(5)
C      REAL*4 X0(10), Y0(10), X1(10), Y1(10)
C      REAL*4 X2(10), Y2(10), X3(10), Y3(10)
      REAL*4 X0(1), Y0(1), X1(1), Y1(1)
      REAL*4 X2(1), Y2(1), X3(1), Y3(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M4, MOVER2
      INTEGER K1, K2, KK, L, L1, MM4, NT, SIZE, SEP, NS
      REAL*4 ANGLE, C1, C2, C3, S1, S2, S3, T, TWOPI
      REAL*4 I1, I2, I3, IS0, IS1, IU0, IU1, R1, R2, R3
      REAL*4 RS0, RS1, RU0, RU1, FJM1, FM4
      DATA TWOPI/6.283185/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M4     = M*4
      FM4    = FLOAT(M4)
      MM4    = SEP*M4
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1, MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM4
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        C3 = C2*C1-S2*S1
        S3 = S2*C1+C2*S1
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = C1
        C1   = S1
        S1   = T
        C2   = -C2
        T    = C3
        C3   = -S3
        S3   = -T
  102        CONTINUE
C
        DO 107 KK = K0,NS,MM4
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              RS0   = X0(K)+X2(K)
              IS0   = Y0(K)+Y2(K)
              RU0   = X0(K)-X2(K)
              IU0   = Y0(K)-Y2(K)
              RS1   = X1(K)+X3(K)
              IS1   = Y1(K)+Y3(K)
              RU1   = X1(K)-X3(K)
              IU1   = Y1(K)-Y3(K)
              X0(K) = RS0+RS1
              Y0(K) = IS0+IS1
              IF (ZERO) GO TO 103
              R1  = RU0+IU1
              I1  = IU0-RU1
              R2  = RS0-RS1
              I2  = IS0-IS1
              R3  = RU0-IU1
              I3  = IU0+RU1
              X2(K) = R1*C1+I1*S1
              Y2(K) = I1*C1-R1*S1
              X1(K) = R2*C2+I2*S2
              Y1(K) = I2*C2-R2*S2
              X3(K) = R3*C3+I3*S3
              Y3(K) = I3*C3-R3*S3
              GO TO 104
  103              CONTINUE
              X2(K) = RU0+IU1
              Y2(K) = IU0-RU1
              X1(K) = RS0-RS1
              Y1(K) = IS0-IS1
              X3(K) = RU0-IU1
              Y3(K) = IU0+RU1
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R5CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 5 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
            SUBROUTINE R5CFTK (N,M,X0,Y0,X1,Y1,X2,Y2,X3,Y3,X4,Y4,DIM)
C
      INTEGER N, M, DIM(5)
C      REAL X0(10), Y0(10), X1(10), Y1(10), X2(10), Y2(10)
C      REAL X3(10), Y3(10), X4(10), Y4(10)
      REAL X0(1), Y0(1), X1(1), Y1(1), X2(1), Y2(1)
      REAL X3(1), Y3(1), X4(1), Y4(1)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M5, MOVER2
      INTEGER K1, K2, KK, L, L1, MM5, NT, SIZE, SEP, NS
      REAL*4 ANGLE, A1, A2, B1, B2, C1, C2, C3, C4, S1, S2, S3, S4
      REAL*4 R0, R1, R2, R3, R4, RA1, RA2, RB1, RB2, RS1, RS2
      REAL*4 RU1, RU2, I0, I1, I2, I3, I4, IA1, IA2, IB1, IB2
      REAL*4 IS1, IS2, IU1, IU2, T, TWOPI, FJM1, FM5
      DATA TWOPI/6.283185/,A1/0.30901699/,B1/0.95105652/,
     1  A2/-0.80901699/,B2/0.58778525/
C  ----------------------------------------------------------------------------
C
      NT     =  DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M5     = M*5
      FM5    = FLOAT(M5)
      MM5    = SEP*M5
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM5
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        C3 = C2*C1-S2*S1
        S3 = S2*C1+C2*S1
        C4 = C2*C2-S2*S2
        S4 = S2*C2+C2*S2
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = C1*A1+S1*B1
        S1   = C1*B1-S1*A1
        C1   = T
        T    = C2*A2+S2*B2
        S2   = C2*B2-S2*A2
        C2   = T
        T    = C3*A2-S3*B2
        S3   = -C3*B2-S3*A2
        C3   = T
        T    = C4*A1-S4*B1
        S4   = -C4*B1-S4*A1
        C4   = T
  102        CONTINUE
C
        DO 107 KK = K0,NS,MM5
          DO 106 L = KK,NT,L1
            K1 = L+SIZE
            DO 105 K = L,K1,K2
              R0    = X0(K)
              I0    = Y0(K)
              RS1   = X1(K)+X4(K)
              IS1   = Y1(K)+Y4(K)
              RU1   = X1(K)-X4(K)
              IU1   = Y1(K)-Y4(K)
              RS2   = X2(K)+X3(K)
              IS2   = Y2(K)+Y3(K)
              RU2   = X2(K)-X3(K)
              IU2   = Y2(K)-Y3(K)
              X0(K) = R0+RS1+RS2
              Y0(K) = I0+IS1+IS2
              RA1   = R0+RS1*A1+RS2*A2
              IA1   = I0+IS1*A1+IS2*A2
              RA2   = R0+RS1*A2+RS2*A1
              IA2   = I0+IS1*A2+IS2*A1
              RB1   = RU1*B1+RU2*B2
              IB1   = IU1*B1+IU2*B2
              RB2   = RU1*B2-RU2*B1
              IB2   = IU1*B2-IU2*B1
              IF (ZERO) GO TO 103
              R1    = RA1+IB1
              I1    = IA1-RB1
              R2    = RA2+IB2
              I2    = IA2-RB2
              R3    = RA2-IB2
              I3    = IA2+RB2
              R4    = RA1-IB1
              I4    = IA1+RB1
              X1(K) = R1*C1+I1*S1
              Y1(K) = I1*C1-R1*S1
              X2(K) = R2*C2+I2*S2
              Y2(K) = I2*C2-R2*S2
              X3(K) = R3*C3+I3*S3
              Y3(K) = I3*C3-R3*S3
              X4(K) = R4*C4+I4*S4
              Y4(K) = I4*C4-R4*S4
              GO TO 104
  103              CONTINUE
              X1(K) = RA1+IB1
              Y1(K) = IA1-RB1
              X2(K) = RA2+IB2
              Y2(K) = IA2-RB2
              X3(K) = RA2-IB2
              Y3(K) = IA2+RB2
              X4(K) = RA1-IB1
              Y4(K) = IA1+RB1
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R8CFTK     *****                   *
C  *                                                                          *
C  *    RADIX 8 MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL            *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X*(10),Y*(10) array      *
C  *       declarations changed to X*(1),Y*(1); D. BELNAP/T. BAKER; 29-DEC-1993 *
C  ****************************************************************************
C
      SUBROUTINE R8CFTK (N,M,X0,Y0,X1,Y1,X2,Y2,X3,Y3,X4,Y4,
     1  X5,Y5,X6,Y6,X7,Y7,DIM)
C
      INTEGER N, M, DIM(5)
      LOGICAL FOLD, ZERO
      INTEGER J, K, K0, M8, MOVER2
      INTEGER K1, K2, KK, L, L1, MM8, NT, SIZE, SEP, NS
C      REAL*4  X0(10), Y0(10), X1(10), Y1(10), X2(10), Y2(10)
C      REAL*4  X3(10), Y3(10), X4(10), Y4(10), X5(10), Y5(10)
C      REAL*4  X6(10), Y6(10), X7(10), Y7(10)
      REAL*4  X0(1), Y0(1), X1(1), Y1(1), X2(1), Y2(1)
      REAL*4  X3(1), Y3(1), X4(1), Y4(1), X5(1), Y5(1)
      REAL*4  X6(1), Y6(1), X7(1), Y7(1)
      REAL*4  ANGLE, C1, C2, C3, C4, C5, C6, C7, E
      REAL*4  S1, S2, S3, S4, S5, S6, S7, T, TWOPI, FJM1, FM8
      REAL*4  R1, R2, R3, R4, R5, R6, R7, RS0, RS1, RS2, RS3
      REAL*4  RU0, RU1, RU2, RU3, I1, I2, I3, I4, I5, I6, I7
      REAL*4  IS0, IS1, IS2, IS3, IU0, IU1, IU2, IU3
      REAL*4  RSS0, RSS1, RSU0, RSU1, RUS0, RUS1, RUU0, RUU1
      REAL*4  ISS0, ISS1, ISU0, ISU1, IUS0, IUS1, IUU0, IUU1
      DATA TWOPI/6.283185/, E/0.70710678/
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      M8     = M*8
      FM8    = FLOAT(M8)
      MM8    = SEP*M8
      MOVER2 = M/2+1
C
      FJM1 = -1.0
      DO 108 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FM8
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 102
        C1 = COS(ANGLE)
        S1 = SIN(ANGLE)
        C2 = C1*C1-S1*S1
        S2 = S1*C1+C1*S1
        C3 = C2*C1-S2*S1
        S3 = S2*C1+C2*S1
        C4 = C2*C2-S2*S2
        S4 = S2*C2+C2*S2
        C5 = C4*C1-S4*S1
        S5 = S4*C1+C4*S1
        C6 = C4*C2-S4*S2
        S6 = S4*C2+C4*S2
        C7 = C4*C3-S4*S3
        S7 = S4*C3+C4*S3
        GO TO 102
  101        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        T    = (C1+S1)*E
        S1   = (C1-S1)*E
        C1   = T
        T    = S2
        S2   = C2
        C2   = T
        T    = (-C3+S3)*E
        S3   = (C3+S3)*E
        C3   = T
        C4   = -C4
        T    = -(C5+S5)*E
        S5   = (-C5+S5)*E
        C5   = T
        T    = -S6
        S6   = -C6
        C6   = T
        T    = (C7-S7)*E
        S7   = -(C7+S7)*E
        C7   = T
  102        CONTINUE
C
        DO 107 KK=K0,NS,MM8
          DO 106 L=KK,NT,L1
            K1=L+SIZE
            DO 105 K=L,K1,K2
              RS0=X0(K)+X4(K)
              IS0=Y0(K)+Y4(K)
              RU0=X0(K)-X4(K)
              IU0=Y0(K)-Y4(K)
              RS1=X1(K)+X5(K)
              IS1=Y1(K)+Y5(K)
              RU1=X1(K)-X5(K)
              IU1=Y1(K)-Y5(K)
              RS2=X2(K)+X6(K)
              IS2=Y2(K)+Y6(K)
              RU2=X2(K)-X6(K)
              IU2=Y2(K)-Y6(K)
              RS3=X3(K)+X7(K)
              IS3=Y3(K)+Y7(K)
              RU3=X3(K)-X7(K)
              IU3=Y3(K)-Y7(K)
              RSS0=RS0+RS2
              ISS0=IS0+IS2
              RSU0=RS0-RS2
              ISU0=IS0-IS2
              RSS1=RS1+RS3
              ISS1=IS1+IS3
              RSU1=RS1-RS3
              ISU1=IS1-IS3
              RUS0=RU0-IU2
              IUS0=IU0+RU2
              RUU0=RU0+IU2
              IUU0=IU0-RU2
              RUS1=RU1-IU3
              IUS1=IU1+RU3
              RUU1=RU1+IU3
              IUU1=IU1-RU3
              T=(RUS1+IUS1)*E
              IUS1=(IUS1-RUS1)*E
              RUS1=T
              T=(RUU1+IUU1)*E
              IUU1=(IUU1-RUU1)*E
              RUU1=T
              X0(K)=RSS0+RSS1
              Y0(K)=ISS0+ISS1
              IF (ZERO) GO TO 103
              R1=RUU0+RUU1
              I1=IUU0+IUU1
              R2=RSU0+ISU1
              I2=ISU0-RSU1
              R3=RUS0+IUS1
              I3=IUS0-RUS1
              R4=RSS0-RSS1
              I4=ISS0-ISS1
              R5=RUU0-RUU1
              I5=IUU0-IUU1
              R6=RSU0-ISU1
              I6=ISU0+RSU1
              R7=RUS0-IUS1
              I7=IUS0+RUS1
              X4(K)=R1*C1+I1*S1
              Y4(K)=I1*C1-R1*S1
              X2(K)=R2*C2+I2*S2
              Y2(K)=I2*C2-R2*S2
              X6(K)=R3*C3+I3*S3
              Y6(K)=I3*C3-R3*S3
              X1(K)=R4*C4+I4*S4
              Y1(K)=I4*C4-R4*S4
              X5(K)=R5*C5+I5*S5
              Y5(K)=I5*C5-R5*S5
              X3(K)=R6*C6+I6*S6
              Y3(K)=I6*C6-R6*S6
              X7(K)=R7*C7+I7*S7
              Y7(K)=I7*C7-R7*S7
              GO TO 104
  103              CONTINUE
              X4(K)=RUU0+RUU1
              Y4(K)=IUU0+IUU1
              X2(K)=RSU0+ISU1
              Y2(K)=ISU0-RSU1
              X6(K)=RUS0+IUS1
              Y6(K)=IUS0-RUS1
              X1(K)=RSS0-RSS1
              Y1(K)=ISS0-ISS1
              X5(K)=RUU0-RUU1
              Y5(K)=IUU0-IUU1
              X3(K)=RSU0-ISU1
              Y3(K)=ISU0+RSU1
              X7(K)=RUS0-IUS1
              Y7(K)=IUS0+RUS1
  104              CONTINUE
  105            CONTINUE
  106          CONTINUE
  107        CONTINUE
        IF (FOLD) GO TO 101
  108      CONTINUE
C
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE RPCFTK     *****                   *
C  *                                                                          *
C  *    RADIX PRIME MULTI-DIMENSIONAL COMPLEX FOURIER TRANSFORM KERNEL        *
C  *                                                                          *
C  *      Modified 14 Jan 1994, by D. Belnap for the following reason:            *
C  *      EMMAP3DT crashed in this routine because the X and Y arrays were      *
C  *      two-dimensional and the input was one-dimensional.  I solved this     *
C  *      problem by changing X and Y to one-dimensional arrays.  This change   *
C  *      required that the index for the 1D array be computed from the 2D      *
C  *      array indices.  For 2D_ARRAY(R,P) and 1D_ARRAY(R*P)                   *
C  *                                                                          *
C  *                  NUM  =  (j - 1)R + i                              *
C  *                                                                          *
C  *      where NUM is the 1D index number and i,j the 2D indices.            *
C  *      (The 2D array code remains but is commented out.)                  *
C  ****************************************************************************
C
      SUBROUTINE RPCFTK (N, M, P, R, X, Y, DIM)
C
      IMPLICIT NONE
C
      INTEGER N, M, P, R, DIM(5)
C      REAL*4  X(R,P), Y(R,P)
      REAL*4  X(1), Y(1)
      REAL*4  ANGLE, IS, IU, RS, RU, T, TWOPI, XT, YT
      REAL*4  FU, FP, FJM1, FMP
      REAL*4  AA(9,9), BB(9, 9),  A(18), B(18), C(18), S(18)
      REAL*4  IA(9), IB(9), RA(9), RB(9)
      INTEGER J, JJ, K0, K, MOVER2, MP, PM, PP, U, V
      INTEGER K1, K2, KK, L, L1, MMP, NT, SIZE, SEP, NS
      LOGICAL FOLD, ZERO
      DATA TWOPI/6.283185/
      INTEGER*4  NUM1, NUM2
C  ----------------------------------------------------------------------------
C
      NT     = DIM(1)
      SEP    = DIM(2)
      L1     = DIM(3)
      SIZE   = DIM(4)-1
      K2     = DIM(5)
      NS     = N*SEP
      MOVER2 = M/2+1
      MP     = M*P
      FMP    = FLOAT(MP)
      MMP    = SEP*MP
      PP     = P/2
      PM     = P-1
      FP     = FLOAT(P)
      FU     = 0.0
      DO 101 U = 1,PP
        FU    =  FU+1.0
        ANGLE =  TWOPI*FU/FP
        JJ    =  P-U
        A(U)  =  COS(ANGLE)
        B(U)  =  SIN(ANGLE)
        A(JJ) =  A(U)
        B(JJ) = -B(U)
  101      CONTINUE
      DO 103 U = 1,PP
        DO 102 V = 1,PP
          JJ      = U*V-U*V/P*P
          AA(V,U) = A(JJ)
          BB(V,U) = B(JJ)
  102         CONTINUE
  103      CONTINUE
C
      FJM1 = -1.0
      DO 117 J = 1,MOVER2
        FOLD  = J.GT.1.AND.2*J.LT.M+2
        K0    = (J-1)*SEP+1
        FJM1  = FJM1+1.0
        ANGLE = TWOPI*FJM1/FMP
        ZERO  = ANGLE.EQ.0.0
        IF (ZERO) GO TO 107
        C(1) = COS(ANGLE)
        S(1) = SIN(ANGLE)
        DO 104 U = 2,PM
          C(U) = C(U-1)*C(1)-S(U-1)*S(1)
          S(U) = S(U-1)*C(1)+C(U-1)*S(1)
  104        CONTINUE
        GO TO 107
  105        CONTINUE
        FOLD = .FALSE.
        K0   = (M+1-J)*SEP+1
        DO 106 U = 1,PM
          T    =  C(U)*A(U)+S(U)*B(U)
          S(U) = -S(U)*A(U)+C(U)*B(U)
          C(U) =  T
  106        CONTINUE
  107        CONTINUE
C
        DO 116 KK = K0,NS,MMP
          DO 115 L = KK,NT,L1
            K1 = L+SIZE
            DO 114 K = L,K1,K2
C            *******************
C            *  X,Y array fix  *
C            *******************
C              XT = X(K,1)
C              YT = Y(K,1)
C            -------------------
            NUM1 = (1-1)*R + K
              XT = X(NUM1)
              YT = Y(NUM1)
C            ===================END OF FIX
C            *******************
C            *  X,Y array fix  *
C            *******************
C              RS = X(K,2)+X(K,P)
C              IS = Y(K,2)+Y(K,P)
C              RU = X(K,2)-X(K,P)
C              IU = Y(K,2)-Y(K,P)
C            ------------------
            NUM1 = (2-1)*R + K
            NUM2 = (P-1)*R + K
              RS = X(NUM1)+X(NUM2)
              IS = Y(NUM1)+Y(NUM2)
              RU = X(NUM1)-X(NUM2)
              IU = Y(NUM1)-Y(NUM2)
C            ====================END OF FIX
              DO 108 U = 1,PP
                RA(U) = XT+RS*AA(U,1)
                IA(U) = YT+IS*AA(U,1)
                RB(U) = RU*BB(U,1)
                IB(U) = IU*BB(U,1)
  108              CONTINUE
              XT = XT+RS
              YT = YT+IS
              DO 110 U = 2,PP
                JJ = P-U
C              *******************
C              *  X,Y array fix  *
C              *******************
C                RS = X(K,U+1)+X(K,JJ+1)
C                IS = Y(K,U+1)+Y(K,JJ+1)
C                RU = X(K,U+1)-X(K,JJ+1)
C                IU = Y(K,U+1)-Y(K,JJ+1)
C              -----------------------
              NUM1 = ((U+1)-1)*R + K
              NUM2 = ((JJ+1)-1)*R + K
                RS = X(NUM1)+X(NUM2)
                IS = Y(NUM1)+Y(NUM2)
                RU = X(NUM1)-X(NUM2)
                IU = Y(NUM1)-Y(NUM2)
C              ========================END OF FIX
                XT = XT+RS
                YT = YT+IS
                DO 109 V = 1,PP
                  RA(V) = RA(V)+RS*AA(V,U)
                  IA(V) = IA(V)+IS*AA(V,U)
                  RB(V) = RB(V)+RU*BB(V,U)
                  IB(V) = IB(V)+IU*BB(V,U)
  109                CONTINUE
  110              CONTINUE
C            *******************
C            *  X,Y array fix  *
C            *******************
C              X(K,1) = XT
C              Y(K,1) = YT
C            -------------------
            NUM1 = (1-1)*R + K
              X(NUM1) = XT
              Y(NUM1) = YT
C            ===================END OF FIX
              DO 113 U = 1,PP
                JJ = P-U
                IF (ZERO) GO TO 111
                XT        = RA(U)+IB(U)
                YT        = IA(U)-RB(U)
C              *******************
C              *  X,Y array fix  *
C              *******************
C                X(K,U+1)  = XT*C(U)+YT*S(U)
C                Y(K,U+1)  = YT*C(U)-XT*S(U)
C              ---------------------------
              NUM1 = ((U+1)-1)*R + K
                X(NUM1) = XT*C(U)+YT*S(U)
                Y(NUM1) = YT*C(U)-XT*S(U)
C              ========================END OF FIX
                XT        = RA(U)-IB(U)
                YT        = IA(U)+RB(U)
C              *******************
C              *  X,Y array fix  *
C              *******************
C                X(K,JJ+1) = XT*C(JJ)+YT*S(JJ)
C                Y(K,JJ+1) = YT*C(JJ)-XT*S(JJ)
C              -----------------------------
              NUM1 = ((JJ+1)-1)*R + K
                X(NUM1) = XT*C(JJ)+YT*S(JJ)
                Y(NUM1) = YT*C(JJ)-XT*S(JJ)
C              ===========================END OF FIX
                GO TO 112
  111                CONTINUE
C              *******************
C              *  X,Y array fix  *
C              *******************
C                X(K,U+1)  = RA(U)+IB(U)
C                Y(K,U+1)  = IA(U)-RB(U)
C                X(K,JJ+1) = RA(U)-IB(U)
C                Y(K,JJ+1) = IA(U)+RB(U)
C              -----------------------
              NUM1    = ((U+1)-1)*R + K
              NUM2    = ((JJ+1)-1)*R + K
                X(NUM1) = RA(U)+IB(U)
                Y(NUM1) = IA(U)-RB(U)
                X(NUM2) = RA(U)-IB(U)
                Y(NUM2) = IA(U)+RB(U)
C              =======================END OF FIX
  112                CONTINUE
  113              CONTINUE
  114            CONTINUE
  115          CONTINUE
  116        CONTINUE
        IF (FOLD) GO TO 105
  117      CONTINUE
C
      RETURN
C
      END
C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE DIPRP     *****                   *
C  *                                                                          *
C  *    DOUBLE IN PLACE REORDERING PROGRAMME                                  *
C  *                                                                          *
C  *      -To fix a SUBSCRIPT RANGE ERROR in EMMAP3DT, X(10),Y(10) array        *
C  *       declarations changed to X(1),Y(1); D. BELNAP/T. BAKER; 29-DEC-1993   *
C  ****************************************************************************
C
      SUBROUTINE DIPRP (PTS, SYM, PSYM, UNSYM, DIM, X, Y)
C
      INTEGER PTS, PSYM
C      REAL*4  X(10), Y(10), T
      REAL*4  X(1), Y(1), T
      INTEGER SYM(10), UNSYM(10), DIM(5), MODULO(14)
      INTEGER DK, JJ, KK, LK, MODS, MULT, NEST, PUNSYM, TEST
      INTEGER NT, SEP, DELTA, P, P0, P1, P2, P3, P4, P5, SIZE
      INTEGER S(14), U(14)
      INTEGER A, B, C, D, E, F, G, H, I, J, K, L, M, N
      INTEGER BS, CS, DS, ES, FS, GS, HS, IS, JS, KS, LS, MS, NS
      INTEGER AL, BL, CL, DL, EL, FL, GL, HL, IL, JL, KL, LL, ML, NL
      LOGICAL ONEMOD
      EQUIVALENCE (AL,U(1)), (BS,S(2)), (BL,U(2))
      EQUIVALENCE (CS,S(3)), (CL,U(3)), (DS,S(4)), (DL,U(4))
      EQUIVALENCE (ES,S(5)), (EL,U(5)), (FS,S(6)), (FL,U(6))
      EQUIVALENCE (GS,S(7)), (GL,U(7)), (HS,S(8)), (HL,U(8))
      EQUIVALENCE (IS,S(9)), (IL,U(9)), (JS,S(10)), (JL,U(10))
      EQUIVALENCE (KS,S(11)), (KL,U(11)), (LS,S(12)), (LL,U(12))
      EQUIVALENCE (MS,S(13)), (ML,U(13)), (NS,S(14)), (NL,U(14))
C  ----------------------------------------------------------------------------
C
      NEST = 14
      NT   = DIM(1)
      SEP  = DIM(2)
      P2   = DIM(3)
      SIZE = DIM(4)-1
      P4   = DIM(5)
      IF (SYM(1).EQ.0) GO TO 106
      DO 101 J = 1,NEST
        U(J) = 1
        S(J) = 1
  101      CONTINUE
      N = PTS
      DO 102 J = 1,NEST
        IF (SYM(J).EQ.0) GO TO 103
        JJ    = NEST+1-J
        U(JJ) = N
        S(JJ) = N/SYM(J)
        N     = N/SYM(J)
  102      CONTINUE
  103      CONTINUE
C
      JJ = 0
      DO 105 A = 1,AL
      DO 105 B = A,BL,BS
      DO 105 C = B,CL,CS
      DO 105 D = C,DL,DS
      DO 105 E = D,EL,ES
      DO 105 F = E,FL,FS
      DO 105 G = F,GL,GS
      DO 105 H = G,HL,HS
      DO 105 I = H,IL,IS
      DO 105 J = I,JL,JS
      DO 105 K = J,KL,KS
      DO 105 L = K,LL,LS
      DO 105 M = L,ML,MS
      DO 105 N = M,NL,NS
        JJ = JJ+1
        IF (JJ.GE.N) GO TO 105
        DELTA = (N-JJ)*SEP
        P1    = (JJ-1)*SEP+1
        DO 104 P0 = P1,NT,P2
          P3 = P0+SIZE
          DO 104 P = P0,P3,P4
            P5    = P+DELTA
            T     = X(P)
            X(P)  = X(P5)
            X(P5) = T
            T     = Y(P)
            Y(P)  = Y(P5)
            Y(P5) = T
  104        CONTINUE
  105      CONTINUE
C
  106      CONTINUE
      IF (UNSYM(1).EQ.0) GO TO 120
      PUNSYM = PTS/PSYM**2
      MULT   = PUNSYM/UNSYM(1)
      TEST   = (UNSYM(1)*UNSYM(2)-1)*MULT*PSYM
      LK     = MULT
      DK     = MULT
      DO 107 K = 2,NEST
        IF (UNSYM(K).EQ.0) GO TO 108
        LK   = LK*UNSYM(K-1)
        DK   = DK/UNSYM(K)
        U(K) = (LK-DK)*PSYM
        MODS = K
  107      CONTINUE
  108      CONTINUE
      ONEMOD = MODS.LT.3
      IF (ONEMOD) GO TO 110
      DO 109 J = 3,MODS
        JJ         = MODS+3-J
        MODULO(JJ) = U(J)
  109      CONTINUE
  110      CONTINUE
      MODULO(2) = U(2)
      JL        = (PUNSYM-3)*PSYM
      MS        = PUNSYM*PSYM
C
      DO 119 J = PSYM,JL,PSYM
        K = J
C
  111        CONTINUE
        K = K*MULT
        IF (ONEMOD) GO TO 113
        DO 112 I = 3,MODS
          K = K-(K/MODULO(I))*MODULO(I)
  112        CONTINUE
  113        CONTINUE
        IF (K.GE.TEST) GO TO 114
        K = K-(K/MODULO(2))*MODULO(2)
        GO TO 115
  114        CONTINUE
          K = K-(K/MODULO(2))*MODULO(2)+MODULO(2)
  115        CONTINUE
        IF (K.LT.J) GO TO 111
C
        IF (K.EQ.J) GO TO 118
        DELTA = (K-J)*SEP
        DO 117 L = 1,PSYM
          DO 116 M = L,PTS,MS
            P1 = (M+J-1)*SEP+1
            DO 116 P0 = P1,NT,P2
              P3 = P0+SIZE
              DO 116 JJ = P0,P3,P4
                KK    = JJ+DELTA
                T     = X(JJ)
                X(JJ) = X(KK)
                X(KK) = T
                T     = Y(JJ)
                Y(JJ) = Y(KK)
                Y(KK) = T
  116          CONTINUE
  117        CONTINUE
  118        CONTINUE
  119      CONTINUE
C
  120      CONTINUE
      RETURN
      END
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE HERMFT     *****                   *
C  *                                                                          *
C  *    HERMITIAN SYMMETRIC FOURIER TRANSFORM                                 *
C  *                                                                          *
C  *    GIVEN THE UNIQUE TERMS OF A HERMITIAN SYMMETRIC SEQUENCE OF LENGTH    *
C  *     2N THIS SUBROUTINE CALCULATES THE 2N REAL NUMBERS WHICH ARE ITS      *
C  *     FOURIER TRANSFORM.  THE EVEN NUMBERED ELEMENTS OF THE TRANSFORM      *
C  *     (0, 2, 4, . . ., 2N-2) ARE RETURNED IN X AND THE ODD NUMBERED        *
C  *     ELEMENTS (1, 3, 5, . . ., 2N-1) IN Y.                                *
C  *                                                                          *
C  *    A FINITE HERMITIAN SEQUENCE OF LENGTH 2N CONTAINS N + 1 UNIQUE        *
C  *     REAL NUMBERS AND N - 1 UNIQUE IMAGINARY NUMBERS.  FOR CONVENIENCE    *
C  *     THE REAL VALUE FOR X(N) IS STORED AT Y(0).                           *
C  ****************************************************************************
C
      SUBROUTINE HERMFT (X, Y, N, DIM)
C
      INTEGER N, DIM(5)
      REAL X(1),Y(1), A, B, C, D, E, F, ANGLE, CO, SI, TWON, TWOPI
      INTEGER NT, D2, D3, D4, D5, I, J, K, NOVER2, I0, I1, I2, K1
C  ----------------------------------------------------------------------------
C
      TWOPI = 6.283185
      TWON  = FLOAT(2*N)
C
      NT = DIM(1)
      D2 = DIM(2)
      D3 = DIM(3)
      D4 = DIM(4)-1
      D5 = DIM(5)
C
      DO 101 I0 = 1,NT,D3
        I1 = I0+D4
        DO 101 I = I0,I1,D5
          A    = X(I)
          B    = Y(I)
          X(I) = A+B
          Y(I) = A-B
  101      CONTINUE
C
      NOVER2 = N/2+1
      IF (NOVER2.LT.2) GO TO 105
      DO 104 I0 = 2,NOVER2
        ANGLE = TWOPI*FLOAT(I0-1)/TWON
        CO    = COS(ANGLE)
        SI    = SIN(ANGLE)
        K     = (N+2-2*I0)*D2
        K1    = (I0-1)*D2+1
        DO 103 I1 = K1,NT,D3
          I2 = I1+D4
          DO 102 I = I1,I2,D5
            J    = I+K
            A    = X(I)+X(J)
            B    = X(I)-X(J)
            C    = Y(I)+Y(J)
            D    = Y(I)-Y(J)
            E    = B*CO+C*SI
            F    = B*SI-C*CO
            X(I) = A+F
            X(J) = A-F
            Y(I) = E+D
            Y(J) = E-D
  102          CONTINUE
  103        CONTINUE
  104      CONTINUE
C
      CALL CMPLFT (X,Y,N,DIM)
C
  105      CONTINUE
      RETURN
      END
